@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2013 Stuart Parmenter and others,
+ * Copyright (C) 1997-2016 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -49,12 +47,6 @@
     do {gchar* dir = g_strconcat(g_get_home_dir(), "/.gpe", NULL);\
  mkdir(dir, S_IRUSR|S_IWUSR|S_IXUSR); g_free(dir);}while(0)
 
-static LibBalsaAddressBookClass *parent_class = NULL;
-
-static void
-libbalsa_address_book_gpe_class_init(LibBalsaAddressBookGpeClass *
-				      klass);
-static void libbalsa_address_book_gpe_init(LibBalsaAddressBookGpe * ab);
 static void libbalsa_address_book_gpe_finalize(GObject * object);
 
 static LibBalsaABErr libbalsa_address_book_gpe_load(LibBalsaAddressBook * ab, 
@@ -63,7 +55,7 @@ static LibBalsaABErr libbalsa_address_book_gpe_load(LibBalsaAddressBook * ab,
                                                     gpointer closure);
 
 static void
-libbalsa_address_book_gpe_close_db(LibBalsaAddressBookGpe *ab);
+libbalsa_address_book_gpe_close_db(LibBalsaAddressBookGpe *ab_gpe);
 static LibBalsaABErr
 libbalsa_address_book_gpe_add_address(LibBalsaAddressBook *ab,
                                       LibBalsaAddress *address);
@@ -79,31 +71,21 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
 static GList *libbalsa_address_book_gpe_alias_complete(LibBalsaAddressBook *ab,
                                                        const gchar *prefix);
 
-GType libbalsa_address_book_gpe_get_type(void)
-{
-    static GType address_book_gpe_type = 0;
+struct _LibBalsaAddressBookGpe {
+    LibBalsaAddressBook parent;
+#ifdef HAVE_SQLITE3
+    sqlite3 *db;
+#else                           /* HAVE_SQLITE3 */
+    sqlite *db;
+#endif                          /* HAVE_SQLITE3 */
+};
 
-    if (!address_book_gpe_type) {
-	static const GTypeInfo address_book_gpe_info = {
-	    sizeof(LibBalsaAddressBookGpeClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) libbalsa_address_book_gpe_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(LibBalsaAddressBookGpe),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) libbalsa_address_book_gpe_init
-	};
+struct _LibBalsaAddressBookGpeClass {
+    LibBalsaAddressBookClass parent_class;
+};
 
-	address_book_gpe_type =
-            g_type_register_static(LIBBALSA_TYPE_ADDRESS_BOOK,
-	                           "LibBalsaAddressBookGpe",
-			           &address_book_gpe_info, 0);
-    }
-
-    return address_book_gpe_type;
-}
+G_DEFINE_TYPE(LibBalsaAddressBookGpe, libbalsa_address_book_gpe,
+        LIBBALSA_TYPE_ADDRESS_BOOK)
 
 static void
 libbalsa_address_book_gpe_class_init(LibBalsaAddressBookGpeClass * klass)
@@ -111,8 +93,6 @@ libbalsa_address_book_gpe_class_init(LibBalsaAddressBookGpeClass * klass)
     LibBalsaAddressBookClass *address_book_class;
     GObjectClass *object_class;
 
-    parent_class = g_type_class_peek_parent(klass);
-
     object_class = G_OBJECT_CLASS(klass);
     address_book_class = LIBBALSA_ADDRESS_BOOK_CLASS(klass);
 
@@ -130,10 +110,10 @@ libbalsa_address_book_gpe_class_init(LibBalsaAddressBookGpeClass * klass)
 }
 
 static void
-libbalsa_address_book_gpe_init(LibBalsaAddressBookGpe * ab)
+libbalsa_address_book_gpe_init(LibBalsaAddressBookGpe *ab_gpe)
 {
-    ab->db = NULL;
-    LIBBALSA_ADDRESS_BOOK(ab)->is_expensive = FALSE;
+    ab_gpe->db = NULL;
+    libbalsa_address_book_set_is_expensive(LIBBALSA_ADDRESS_BOOK(ab_gpe), FALSE);
 }
 
 static void
@@ -141,23 +121,23 @@ libbalsa_address_book_gpe_finalize(GObject * object)
 {
     libbalsa_address_book_gpe_close_db(LIBBALSA_ADDRESS_BOOK_GPE(object));
 
-    G_OBJECT_CLASS(parent_class)->finalize(object);
+    G_OBJECT_CLASS(libbalsa_address_book_gpe_parent_class)->finalize(object);
 }
 
 LibBalsaAddressBook *
 libbalsa_address_book_gpe_new(const gchar *name)
 {
-    LibBalsaAddressBookGpe *gpe;
+    LibBalsaAddressBookGpe *ab_gpe;
     LibBalsaAddressBook *ab;
 
-    gpe = LIBBALSA_ADDRESS_BOOK_GPE(g_object_new
+    ab_gpe = LIBBALSA_ADDRESS_BOOK_GPE(g_object_new
                                     (LIBBALSA_TYPE_ADDRESS_BOOK_GPE,
                                      NULL));
-    ab = LIBBALSA_ADDRESS_BOOK(gpe);
+    ab = LIBBALSA_ADDRESS_BOOK(ab_gpe);
 
-    ab->name = g_strdup(name);
+    libbalsa_address_book_set_name(LIBBALSA_ADDRESS_BOOK(ab_gpe), name);
     /* We open on demand... */
-    gpe->db = NULL;
+    ab_gpe->db = NULL;
     return ab;
 }
 
@@ -165,15 +145,15 @@ libbalsa_address_book_gpe_new(const gchar *name)
  * Close the SQLite db....
  */
 static void
-libbalsa_address_book_gpe_close_db(LibBalsaAddressBookGpe * ab)
+libbalsa_address_book_gpe_close_db(LibBalsaAddressBookGpe * ab_gpe)
 {
-    if (ab->db) {
+    if (ab_gpe->db) {
 #ifdef HAVE_SQLITE3
-	sqlite3_close(ab->db);
+	sqlite3_close(ab_gpe->db);
 #else                           /* HAVE_SQLITE3 */
-	sqlite_close(ab->db);
+	sqlite_close(ab_gpe->db);
 #endif                          /* HAVE_SQLITE3 */
-	ab->db = NULL;
+	ab_gpe->db = NULL;
     }
 }
 
@@ -191,7 +171,7 @@ static const char *schema2_str =
 "create table contacts_urn (urn INTEGER PRIMARY KEY)";
 
 static int
-libbalsa_address_book_gpe_open_db(LibBalsaAddressBookGpe * ab)
+libbalsa_address_book_gpe_open_db(LibBalsaAddressBookGpe * ab_gpe)
 {
 #ifdef HAVE_SQLITE3
     gchar *dir, *name;
@@ -201,31 +181,31 @@ libbalsa_address_book_gpe_open_db(LibBalsaAddressBookGpe * ab)
     name = g_build_filename(dir, "contacts", NULL);
     g_free(dir);
 
-    if (sqlite3_open(name, &ab->db) != SQLITE_OK) {
-        printf("Cannot open \"%s\": %s\n", name, sqlite3_errmsg(ab->db));
+    if (sqlite3_open(name, &ab_gpe->db) != SQLITE_OK) {
+        printf("Cannot open “%s”: %s\n", name, sqlite3_errmsg(ab_gpe->db));
         g_free(name);
-        sqlite3_close(ab->db);
-        ab->db = NULL;
+        sqlite3_close(ab_gpe->db);
+        ab_gpe->db = NULL;
         return 0;
     }
     g_free(name);
 
-    sqlite3_exec(ab->db, schema_str,  NULL, NULL, NULL);
-    sqlite3_exec(ab->db, schema2_str, NULL, NULL, NULL);
+    sqlite3_exec(ab_gpe->db, schema_str,  NULL, NULL, NULL);
+    sqlite3_exec(ab_gpe->db, schema2_str, NULL, NULL, NULL);
 #else                           /* HAVE_SQLITE3 */
     gchar *name, *errmsg = NULL;
 
     ASSURE_GPE_DIR;
     name = g_strconcat(g_get_home_dir(), DB_NAME, NULL);
-    ab->db = sqlite_open(name, 0, &errmsg);
+    ab_gpe->db = sqlite_open(name, 0, &errmsg);
     g_free(name);
-    if(ab->db == NULL) {
+    if(ab_gpe->db == NULL) {
         printf("Cannot open: %s\n", errmsg);
         free(errmsg);
         return 0;
     }
-    sqlite_exec (ab->db, schema_str,  NULL, NULL, NULL);
-    sqlite_exec (ab->db, schema2_str, NULL, NULL, NULL);
+    sqlite_exec (ab_gpe->db, schema_str,  NULL, NULL, NULL);
+    sqlite_exec (ab_gpe->db, schema2_str, NULL, NULL, NULL);
 #endif                          /* HAVE_SQLITE3 */
 
     return 1;
@@ -295,9 +275,9 @@ create_name(gchar * first, gchar * last)
 struct gpe_closure {
     LibBalsaAddressBookLoadFunc callback;
     gpointer closure;
-    LibBalsaAddressBookGpe *gpe;
+    LibBalsaAddressBookGpe *ab_gpe;
 };
-                                             
+
 static int
 gpe_read_address(void *arg, int argc, char **argv, char **names)
 {
@@ -310,10 +290,10 @@ gpe_read_address(void *arg, int argc, char **argv, char **names)
     gchar *sql =
         sqlite3_mprintf("select tag,value from contacts where urn=%d",
                         uid);
-    sqlite3_exec(gc->gpe->db, sql, gpe_read_attr, a, NULL);
+    sqlite3_exec(gc->ab_gpe->db, sql, gpe_read_attr, a, NULL);
     sqlite3_free(sql);
 #else                           /* HAVE_SQLITE3 */
-    sqlite_exec_printf (gc->gpe->db,
+    sqlite_exec_printf (gc->ab_gpe->db,
                         "select tag,value from contacts where urn=%d",
                         gpe_read_attr, a, NULL, uid);
 #endif                          /* HAVE_SQLITE3 */
@@ -324,7 +304,7 @@ gpe_read_address(void *arg, int argc, char **argv, char **names)
     if(!a->full_name)
         a->full_name = create_name(a->first_name, a->last_name);
     g_object_set_data(G_OBJECT(a), "urn", GUINT_TO_POINTER(uid));
-    gc->callback(LIBBALSA_ADDRESS_BOOK(gc->gpe), a, gc->closure);
+    gc->callback(LIBBALSA_ADDRESS_BOOK(gc->ab_gpe), a, gc->closure);
     return 0;
 }
 
@@ -339,7 +319,7 @@ libbalsa_address_book_gpe_load(LibBalsaAddressBook * ab,
                                LibBalsaAddressBookLoadFunc callback,
                                gpointer closure)
 {
-    LibBalsaAddressBookGpe *gpe_ab;
+    LibBalsaAddressBookGpe *ab_gpe;
     gchar *err = NULL;
     struct gpe_closure gc;
     int r;
@@ -350,18 +330,17 @@ libbalsa_address_book_gpe_load(LibBalsaAddressBook * ab,
     if (callback == NULL)
 	return LBABERR_OK;
 
-    gpe_ab = LIBBALSA_ADDRESS_BOOK_GPE(ab);
+    ab_gpe = LIBBALSA_ADDRESS_BOOK_GPE(ab);
     
-    if (gpe_ab->db == NULL)
-        if (!libbalsa_address_book_gpe_open_db(gpe_ab))
+    if (ab_gpe->db == NULL)
+        if (!libbalsa_address_book_gpe_open_db(ab_gpe))
             return LBABERR_CANNOT_CONNECT;
 
     gc.callback = callback;
     gc.closure  = closure;
-    gc.gpe      = gpe_ab;
+    gc.ab_gpe   = ab_gpe;
     /* FIXME: error reporting */
 #ifdef HAVE_SQLITE3
-{
     if (filter && *filter) {
         gchar *sql =
             sqlite3_mprintf("select distinct urn from contacts where "
@@ -372,18 +351,17 @@ libbalsa_address_book_gpe_load(LibBalsaAddressBook * ab,
                             " upper(tag)='HOME.EMAIL') "
                             "and value LIKE '%q%%'",
                             filter);
-        r = sqlite3_exec(gpe_ab->db, sql, gpe_read_address, &gc, &err);
+        r = sqlite3_exec(ab_gpe->db, sql, gpe_read_address, &gc, &err);
         sqlite3_free(sql);
     } else {
-        r = sqlite3_exec(gpe_ab->db,
+        r = sqlite3_exec(ab_gpe->db,
                          "select distinct urn from contacts_urn",
                          gpe_read_address, &gc, &err);
     }
-}
 #else                           /* HAVE_SQLITE3 */
     if(filter && *filter) {
         r = sqlite_exec_printf
-            (gpe_ab->db, 
+            (ab_gpe->db,
              "select distinct urn from contacts where "
              "(upper(tag)='FAMILY_NAME' or upper(tag)='GIVEN_NAME' or "
              "upper(tag)='NAME' or "
@@ -391,14 +369,14 @@ libbalsa_address_book_gpe_load(LibBalsaAddressBook * ab,
              "and value LIKE '%q%%'",
              gpe_read_address, &gc, &err, filter);
     } else {
-        r = sqlite_exec(gpe_ab->db, "select distinct urn from contacts_urn",
+        r = sqlite_exec(ab_gpe->db, "select distinct urn from contacts_urn",
                         gpe_read_address, &gc, &err);
     }
 #endif                          /* HAVE_SQLITE3 */
 
     if(r != SQLITE_OK) {
         printf("r=%d err=%s\n", r, err);
-        libbalsa_address_book_set_status(ab, g_strdup(err));
+        libbalsa_address_book_set_status(ab, err);
 #ifdef HAVE_SQLITE3
         sqlite3_free(err);
 #else                           /* HAVE_SQLITE3 */
@@ -432,7 +410,7 @@ static LibBalsaABErr
 libbalsa_address_book_gpe_add_address(LibBalsaAddressBook *ab,
                                        LibBalsaAddress *address)
 {
-    LibBalsaAddressBookGpe *gpe_ab = LIBBALSA_ADDRESS_BOOK_GPE(ab);
+    LibBalsaAddressBookGpe *ab_gpe = LIBBALSA_ADDRESS_BOOK_GPE(ab);
     int r;
     guint id;
     char *err = NULL;
@@ -443,21 +421,21 @@ libbalsa_address_book_gpe_add_address(LibBalsaAddressBook *ab,
     g_return_val_if_fail(address, LBABERR_CANNOT_WRITE);
     g_return_val_if_fail(address->address_list, LBABERR_CANNOT_WRITE);
 
-    if (gpe_ab->db == NULL) {
-        if(!libbalsa_address_book_gpe_open_db(gpe_ab))
+    if (ab_gpe->db == NULL) {
+        if(!libbalsa_address_book_gpe_open_db(ab_gpe))
 	    return LBABERR_CANNOT_CONNECT;
     }
 #ifdef HAVE_SQLITE3
-    r = sqlite3_exec(gpe_ab->db, "insert into contacts_urn values (NULL)",
+    r = sqlite3_exec(ab_gpe->db, "insert into contacts_urn values (NULL)",
                      NULL, NULL, &err);
     if (r != SQLITE_OK) {
-        libbalsa_address_book_set_status(ab, g_strdup(err));
+        libbalsa_address_book_set_status(ab, err);
         sqlite3_free(err);
 #else                           /* HAVE_SQLITE3 */
-    r = sqlite_exec(gpe_ab->db, "insert into contacts_urn values (NULL)",
+    r = sqlite_exec(ab_gpe->db, "insert into contacts_urn values (NULL)",
                     NULL, NULL, &err);
     if (r != SQLITE_OK) {
-        libbalsa_address_book_set_status(ab, g_strdup(err));
+        libbalsa_address_book_set_status(ab, err);
         free(err);
 #endif                          /* HAVE_SQLITE3 */
         return LBABERR_CANNOT_WRITE;
@@ -465,25 +443,25 @@ libbalsa_address_book_gpe_add_address(LibBalsaAddressBook *ab,
     /* FIXME: duplicate detection! */
 
 #ifdef HAVE_SQLITE3
-    id = sqlite3_last_insert_rowid(gpe_ab->db);
+    id = sqlite3_last_insert_rowid(ab_gpe->db);
 #else                           /* HAVE_SQLITE3 */
-    id = sqlite_last_insert_rowid(gpe_ab->db);
+    id = sqlite_last_insert_rowid(ab_gpe->db);
 #endif                          /* HAVE_SQLITE3 */
-    INSERT_ATTR(gpe_ab->db,id, "NAME",        address->full_name);
-    INSERT_ATTR(gpe_ab->db,id, "GIVEN_NAME",  address->first_name);
-    INSERT_ATTR(gpe_ab->db,id, "FAMILY_NAME", address->last_name);
-    INSERT_ATTR(gpe_ab->db,id, "NICKNAME",    address->nick_name);
-    INSERT_ATTR(gpe_ab->db,id, "WORK.ORGANIZATION", address->organization);
-    INSERT_ATTR(gpe_ab->db,id, "WORK.EMAIL",
+    INSERT_ATTR(ab_gpe->db,id, "NAME",        address->full_name);
+    INSERT_ATTR(ab_gpe->db,id, "GIVEN_NAME",  address->first_name);
+    INSERT_ATTR(ab_gpe->db,id, "FAMILY_NAME", address->last_name);
+    INSERT_ATTR(ab_gpe->db,id, "NICKNAME",    address->nick_name);
+    INSERT_ATTR(ab_gpe->db,id, "WORK.ORGANIZATION", address->organization);
+    INSERT_ATTR(ab_gpe->db,id, "WORK.EMAIL",
                 (char*)address->address_list->data);
 #ifdef HAVE_SQLITE3
     sql = sqlite3_mprintf("insert into contacts values "
                           "('%d', 'MODIFIED', %d)",
                           id, time(NULL));
-    sqlite3_exec(gpe_ab->db, sql, NULL, NULL, NULL);
+    sqlite3_exec(ab_gpe->db, sql, NULL, NULL, NULL);
     sqlite3_free(sql);
 #else                           /* HAVE_SQLITE3 */
-    sqlite_exec_printf(gpe_ab->db, "insert into contacts values "
+    sqlite_exec_printf(ab_gpe->db, "insert into contacts values "
                        "('%d', 'MODIFIED', %d)", NULL, NULL, NULL,
                        id, time(NULL));
 #endif                          /* HAVE_SQLITE3 */
@@ -552,23 +530,23 @@ static LibBalsaABErr
 libbalsa_address_book_gpe_remove_address(LibBalsaAddressBook *ab,
                                           LibBalsaAddress *address)
 {
-    LibBalsaAddressBookGpe *gpe_ab = LIBBALSA_ADDRESS_BOOK_GPE(ab);
+    LibBalsaAddressBookGpe *ab_gpe = LIBBALSA_ADDRESS_BOOK_GPE(ab);
     guint uid;
     char *err;
     g_return_val_if_fail(address, LBABERR_CANNOT_WRITE);
     g_return_val_if_fail(address->address_list, LBABERR_CANNOT_WRITE);
 
-    if (gpe_ab->db == NULL) {
-        if( !libbalsa_address_book_gpe_open_db(gpe_ab))
+    if (ab_gpe->db == NULL) {
+        if( !libbalsa_address_book_gpe_open_db(ab_gpe))
 	    return LBABERR_CANNOT_CONNECT;
     }
     uid = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(address), "urn"));
     if(!uid)/* safety check, perhaps unnecessary */
         return LBABERR_CANNOT_WRITE;
-        
-    err = db_delete_by_uid(gpe_ab->db, uid);
+
+    err = db_delete_by_uid(ab_gpe->db, uid);
     if(err) {
-        libbalsa_address_book_set_status(ab, g_strdup(err));
+        libbalsa_address_book_set_status(ab, err);
 #ifdef HAVE_SQLITE3
         sqlite3_free(err);
 #else                           /* HAVE_SQLITE3 */
@@ -604,7 +582,7 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
                                          LibBalsaAddress *address,
                                          LibBalsaAddress *newval)
 {
-    LibBalsaAddressBookGpe *gpe_ab = LIBBALSA_ADDRESS_BOOK_GPE(ab);
+    LibBalsaAddressBookGpe *ab_gpe = LIBBALSA_ADDRESS_BOOK_GPE(ab);
     guint uid;
     int r;
     char *err;
@@ -616,8 +594,8 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
     g_return_val_if_fail(address->address_list, LBABERR_CANNOT_WRITE);
     g_return_val_if_fail(newval->address_list, LBABERR_CANNOT_WRITE);
 
-    if (gpe_ab->db == NULL) {
-        if( !libbalsa_address_book_gpe_open_db(gpe_ab))
+    if (ab_gpe->db == NULL) {
+        if( !libbalsa_address_book_gpe_open_db(ab_gpe))
 	    return LBABERR_CANNOT_CONNECT;
     }
     uid = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(address), "urn"));
@@ -626,9 +604,9 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
 
     /* do the real work here */
 #ifdef HAVE_SQLITE3
-    if ((r = sqlite3_exec(gpe_ab->db, "begin transaction",
+    if ((r = sqlite3_exec(ab_gpe->db, "begin transaction",
                           NULL, NULL, &err)) != SQLITE_OK) {
-        libbalsa_address_book_set_status(ab, g_strdup(err));
+        libbalsa_address_book_set_status(ab, err);
         sqlite3_free(err);              /* failed, so soon!? */
         return LBABERR_CANNOT_WRITE;
     }
@@ -640,20 +618,20 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
                           " upper(tag)='WORK.ORGANIZATION' or"
                           " upper(tag)='WORK.EMAIL' or"
                           " upper(tag)='MODIFIED')", uid);
-    r = sqlite3_exec(gpe_ab->db, sql, NULL, NULL, &err);
+    r = sqlite3_exec(ab_gpe->db, sql, NULL, NULL, &err);
     sqlite3_free(sql);
     if (r != SQLITE_OK)
         goto rollback;
 #else                           /* HAVE_SQLITE3 */
-    if( (r=sqlite_exec(gpe_ab->db, "begin transaction",
+    if( (r=sqlite_exec(ab_gpe->db, "begin transaction",
                        NULL, NULL, &err)) != SQLITE_OK) {
-        libbalsa_address_book_set_status(ab, g_strdup(err));
+        libbalsa_address_book_set_status(ab, err);
         free(err); /* failed, so soon!? */
         return LBABERR_CANNOT_WRITE;
     }
 
     if( (r=sqlite_exec_printf
-         (gpe_ab->db,
+         (ab_gpe->db,
           "delete from contacts where urn='%d' and "
           "(upper(tag)='NAME' or upper(tag)='GIVEN_NAME' or "
           "upper(tag)='NICKNAME' or upper(tag)='WORK.ORGANIZATION' or "
@@ -661,42 +639,42 @@ libbalsa_address_book_gpe_modify_address(LibBalsaAddressBook *ab,
           NULL, NULL, &err, uid)) != SQLITE_OK)
         goto rollback;
 #endif                          /* HAVE_SQLITE3 */
-    INSERT_ATTR_R(gpe_ab->db,uid, "NAME",        newval->full_name);
-    INSERT_ATTR_R(gpe_ab->db,uid, "GIVEN_NAME",  newval->first_name);
-    INSERT_ATTR_R(gpe_ab->db,uid, "FAMILY_NAME", newval->last_name);
-    INSERT_ATTR_R(gpe_ab->db,uid, "NICKNAME",    newval->nick_name);
-    INSERT_ATTR_R(gpe_ab->db,uid, "WORK.ORGANIZATION", newval->organization);
-    INSERT_ATTR_R(gpe_ab->db,uid, "WORK.EMAIL",
+    INSERT_ATTR_R(ab_gpe->db,uid, "NAME",        newval->full_name);
+    INSERT_ATTR_R(ab_gpe->db,uid, "GIVEN_NAME",  newval->first_name);
+    INSERT_ATTR_R(ab_gpe->db,uid, "FAMILY_NAME", newval->last_name);
+    INSERT_ATTR_R(ab_gpe->db,uid, "NICKNAME",    newval->nick_name);
+    INSERT_ATTR_R(ab_gpe->db,uid, "WORK.ORGANIZATION", newval->organization);
+    INSERT_ATTR_R(ab_gpe->db,uid, "WORK.EMAIL",
                 (char*)newval->address_list->data);
 #ifdef HAVE_SQLITE3
     sql = sqlite3_mprintf("insert into contacts values "
                           "('%d', 'MODIFIED', %d)", uid, time(NULL));
-    r = sqlite3_exec(gpe_ab->db, sql, NULL, NULL, &err);
+    r = sqlite3_exec(ab_gpe->db, sql, NULL, NULL, &err);
     sqlite3_free(sql);
     if (r != SQLITE_OK)
         goto rollback;
 
-    if (sqlite3_exec(gpe_ab->db, "commit transaction", NULL, NULL, &err) ==
+    if (sqlite3_exec(ab_gpe->db, "commit transaction", NULL, NULL, &err) ==
         SQLITE_OK)
         return LBABERR_OK;
 #else                           /* HAVE_SQLITE3 */
-    if( (r=sqlite_exec_printf(gpe_ab->db, "insert into contacts values "
+    if( (r=sqlite_exec_printf(ab_gpe->db, "insert into contacts values "
                               "('%d', 'MODIFIED', %d)", NULL, NULL, &err,
                               uid, time(NULL))) != SQLITE_OK)
         goto rollback;
     
-    if( (r=sqlite_exec(gpe_ab->db, "commit transaction", NULL,
+    if( (r=sqlite_exec(ab_gpe->db, "commit transaction", NULL,
                        NULL, &err)) == SQLITE_OK) return LBABERR_OK;
 #endif                          /* HAVE_SQLITE3 */
 
  rollback:
-    libbalsa_address_book_set_status(ab, g_strdup(err));
+    libbalsa_address_book_set_status(ab, err);
 #ifdef HAVE_SQLITE3
     sqlite3_free(err);
-    sqlite3_exec(gpe_ab->db, "rollback transaction", NULL, NULL, NULL);
+    sqlite3_exec(ab_gpe->db, "rollback transaction", NULL, NULL, NULL);
 #else                           /* HAVE_SQLITE3 */
     free(err);
-    sqlite_exec(gpe_ab->db, "rollback transaction", NULL, NULL, NULL);
+    sqlite_exec(ab_gpe->db, "rollback transaction", NULL, NULL, NULL);
 #endif                          /* HAVE_SQLITE3 */
 
     return LBABERR_CANNOT_WRITE;
@@ -760,30 +738,32 @@ libbalsa_address_book_gpe_alias_complete(LibBalsaAddressBook * ab,
         "upper(tag)='WORK.EMAIL' or upper(tag)='HOME.EMAIL') "
         "and upper(value) LIKE '%q%%'";
     struct gpe_completion_closure gcc;
-    LibBalsaAddressBookGpe *gpe_ab;
+    LibBalsaAddressBookGpe *ab_gpe;
     char *err = NULL;
     int r;
 
     g_return_val_if_fail ( LIBBALSA_ADDRESS_BOOK_GPE(ab), NULL);
 
-    gpe_ab = LIBBALSA_ADDRESS_BOOK_GPE(ab);
+    ab_gpe = LIBBALSA_ADDRESS_BOOK_GPE(ab);
 
-    if (!ab->expand_aliases) return NULL;
-    if (gpe_ab->db == NULL) {
-        if( !libbalsa_address_book_gpe_open_db(gpe_ab))
+    if (!libbalsa_address_book_get_expand_aliases(ab))
+        return NULL;
+
+    if (ab_gpe->db == NULL) {
+        if( !libbalsa_address_book_gpe_open_db(ab_gpe))
 	    return NULL;
     }
 
-    gcc.db = gpe_ab->db;
+    gcc.db = ab_gpe->db;
     gcc.prefix = prefix;
     gcc.res = NULL;
 #ifdef HAVE_SQLITE3
     if (prefix) {
         gchar *sql = sqlite3_mprintf(query, prefix);
-        r = sqlite3_exec(gpe_ab->db, sql, gpe_read_completion, &gcc, &err);
+        r = sqlite3_exec(ab_gpe->db, sql, gpe_read_completion, &gcc, &err);
         sqlite3_free(sql);
     } else
-        r = sqlite3_exec(gpe_ab->db,
+        r = sqlite3_exec(ab_gpe->db,
                          "select distinct urn from contacts_urn",
                          gpe_read_completion, &gcc, &err);
     if(err) {
@@ -792,11 +772,11 @@ libbalsa_address_book_gpe_alias_complete(LibBalsaAddressBook * ab,
     }
 #else                           /* HAVE_SQLITE3 */
     if(prefix)
-        r = sqlite_exec_printf(gpe_ab->db, query,
+        r = sqlite_exec_printf(ab_gpe->db, query,
                                gpe_read_completion, &gcc, &err,
                                prefix);
     else
-        r = sqlite_exec(gpe_ab->db, "select distinct urn from contacts_urn",
+        r = sqlite_exec(ab_gpe->db, "select distinct urn from contacts_urn",
                         gpe_read_completion, &gcc, &err);
     if(err) {
         printf("r=%d err=%s\n", r, err);
