@@ -105,8 +105,8 @@ libbalsa_address_book_vcard_new(const gchar * name, const gchar * path)
                                      NULL));
     ab = LIBBALSA_ADDRESS_BOOK(abvc);
 
-    ab->name = g_strdup(name);
-    LIBBALSA_ADDRESS_BOOK_TEXT(ab)->path = g_strdup(path);
+    libbalsa_address_book_set_name(ab, name);
+    libbalsa_address_book_text_set_path(LIBBALSA_ADDRESS_BOOK_TEXT(ab), path);
 
     return ab;
 }
