@@ -131,7 +131,7 @@ libbalsa_address_view_class_init(LibBalsaAddressViewClass * klass)
         g_signal_new("open-address-book",
                      G_OBJECT_CLASS_TYPE(object_class),
                      0, 0, NULL, NULL,
-                     g_cclosure_marshal_VOID__POINTER,
+                     NULL,
                      G_TYPE_NONE, 1, G_TYPE_POINTER);
 }
 
@@ -190,7 +190,8 @@ lbav_get_matching_addresses(LibBalsaAddressView * address_view,
 
         ab = LIBBALSA_ADDRESS_BOOK(list->data);
         if (type == LIBBALSA_ADDRESS_VIEW_MATCH_FAST
-            && (!ab->expand_aliases || ab->is_expensive))
+            && (!libbalsa_address_book_get_expand_aliases(ab) ||
+                libbalsa_address_book_get_is_expensive(ab)))
             continue;
 
         match =
@@ -628,22 +629,10 @@ lbav_insert_text_cb(GtkEditable * editable,
                     gint length,
                     gint * position, LibBalsaAddressView * address_view)
 {
-    gchar *p;
     gchar *ins_text = g_strndup(text, length);
 
     /* replace non-printable chars by spaces */
-    p = ins_text;
-    while (*p != '\0') {
-        gchar *next = g_utf8_next_char(p);
-
-        if (g_unichar_isprint(g_utf8_get_char(p)))
-            p = next;
-        else {
-            *p++ = ' ';
-            if (p != next)
-                memmove(p, next, strlen(next) + 1);
-        }
-    }
+    lbav_clean_text(ins_text);
 
     /* insert */
     g_signal_handlers_block_by_func(editable,
