@@ -22,27 +22,17 @@
 #endif                          /* HAVE_CONFIG_H */
 #include "folder-scanners.h"
 
-#include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <dirent.h>
-#include <pwd.h>
-#include <ctype.h>
-#include <gtk/gtk.h>
-#include <limits.h>
+#include <glib/gstdio.h>
+#include <glib/gi18n.h>
 
 #include "libbalsa.h"
 #include "libimap.h"
+#include "server.h"
 #include "imap-handle.h"
 #include "imap-commands.h"
 #include "imap-server.h"
 
-#ifndef PATH_MAX
-#define PATH_MAX _POSIX_PATH_MAX
-#endif
-
 typedef void (*local_scanner_helper) (gpointer rnode,
                                       const gchar * prefix,
                                       LocalCheck check_local_path,
@@ -57,53 +47,53 @@ libbalsa_scanner_mdir(gpointer rnode, const gchar * prefix,
                       LocalHandler folder_handler,
                       LocalHandler mailbox_handler, guint * depth)
 {
-    DIR *dpc;
-    struct dirent *de;
-    char filename[PATH_MAX];
-    struct stat st;
-    gpointer parent_node = NULL;
+    GDir *dpc;
+    GError *error = NULL;
+    const gchar *entry;
 
     if (!check_local_path(prefix, *depth)
         || !mark_local_path(rnode))
         return;
 
-    dpc = opendir(prefix);
-    if (!dpc)
+    dpc = g_dir_open(prefix, 0U, &error);
+    if (dpc == NULL) {
+    	g_warning("error reading Maildir folder %s: %s", prefix, (error != NULL) ? error->message : "unknown");
+    	g_clear_error(&error);
         return;
+    }
 
     /*
      * if we don't find any subdirectories inside, we'll go
      * and ignore this one too...
      */
-    while ((de = readdir(dpc)) != NULL) {
-        if (de->d_name[0] == '.')
-            continue;
-        snprintf(filename, PATH_MAX, "%s/%s", prefix, de->d_name);
-        /* ignore file if it can't be read. */
-        if (stat(filename, &st) == -1 || access(filename, R_OK) == -1)
-            continue;
+    while ((entry = g_dir_read_name(dpc)) != NULL) {
+    	if (entry[0] != '.') {
+    		gchar *filename;
 
-        if (S_ISDIR(st.st_mode)) {
-            /*
-             * if we think that this looks like a mailbox, include it as such.
-             * otherwise we'll lose the mail in this folder
-             */
-            GType foo = libbalsa_mailbox_type_from_path(filename);
-            if ((foo == LIBBALSA_TYPE_MAILBOX_MH) ||
-                (foo == LIBBALSA_TYPE_MAILBOX_MAILDIR)) {
-                parent_node =
-		    mailbox_handler(rnode, de->d_name, filename, foo);
-                ++*depth;
-                libbalsa_scanner_mdir(parent_node, filename,
-                                      check_local_path, mark_local_path,
-                                      folder_handler, mailbox_handler,
-                                      depth);
-                --*depth;
-            }
-        }
-        /* ignore regular files */
+    		filename = g_build_filename(prefix, entry, NULL);
+
+    		/* ignore regular file, or if it can't be read. */
+    		if (g_file_test(filename, G_FILE_TEST_IS_DIR) && (g_access(filename, R_OK) == 0)) {
+    			/*
+    			 * if we think that this looks like a mailbox, include it as such.
+    			 * otherwise we'll lose the mail in this folder
+    			 */
+    			GType foo = libbalsa_mailbox_type_from_path(filename);
+    			if ((foo == LIBBALSA_TYPE_MAILBOX_MH) ||
+    				(foo == LIBBALSA_TYPE_MAILBOX_MAILDIR)) {
+    				gpointer parent_node;
+
+    				parent_node = mailbox_handler(rnode, entry, filename, foo);
+    				++*depth;
+    				libbalsa_scanner_mdir(parent_node, filename, check_local_path, mark_local_path, folder_handler,
+    					mailbox_handler, depth);
+    				--*depth;
+    			}
+    		}
+    		g_free(filename);
+    	}
     }
-    closedir(dpc);
+    g_dir_close(dpc);
 }
 
 static void
@@ -114,61 +104,64 @@ libbalsa_scanner_local_dir_helper(gpointer rnode, const gchar * prefix,
                                   LocalHandler mailbox_handler,
                                   guint * depth)
 {
-    DIR *dpc;
-    struct dirent *de;
-    char filename[PATH_MAX];
-    struct stat st;
-    GType mailbox_type;
-    gpointer current_node;
+    GDir *dpc;
+    GError *error = NULL;
+    const gchar *entry;
 
     if (!check_local_path(prefix, *depth)
         || !mark_local_path(rnode))
         return;
 
-    dpc = opendir(prefix);
-    if (!dpc)
+
+    dpc = g_dir_open(prefix, 0U, &error);
+    if (dpc == NULL) {
+    	g_warning("error reading mail folder %s: %s", prefix, (error != NULL) ? error->message : "unknown");
+    	g_clear_error(&error);
         return;
-
-    while ((de = readdir(dpc)) != NULL) {
-        if (de->d_name[0] == '.')
-            continue;
-        snprintf(filename, PATH_MAX, "%s/%s", prefix, de->d_name);
-
-        /* ignore file if it can't be read. */
-        if (stat(filename, &st) == -1 || access(filename, R_OK) == -1)
-            continue;
-
-        if (S_ISDIR(st.st_mode)) {
-	    local_scanner_helper helper;
-            mailbox_type = libbalsa_mailbox_type_from_path(filename);
-
-            if ((mailbox_type == LIBBALSA_TYPE_MAILBOX_MH) ||
-                (mailbox_type == LIBBALSA_TYPE_MAILBOX_MAILDIR)) {
-                current_node =
-                    mailbox_handler(rnode, de->d_name, filename, mailbox_type);
-		helper = libbalsa_scanner_mdir;
-            } else {
-                gchar *name = g_path_get_basename(prefix);
-                current_node = folder_handler(rnode, name, filename, 0);
-                g_free(name);
-		helper = libbalsa_scanner_local_dir_helper;
-            }
-
-            ++*depth;
-            helper(current_node, filename, check_local_path,
-                   mark_local_path, folder_handler, mailbox_handler,
-                   depth);
-            --*depth;
-        } else {
-            mailbox_type = libbalsa_mailbox_type_from_path(filename);
-            if (mailbox_type != G_TYPE_OBJECT) {
-                mark_local_path(mailbox_handler
-                                (rnode, de->d_name, filename,
-                                 mailbox_type));
-            }
-        }
     }
-    closedir(dpc);
+
+    while ((entry = g_dir_read_name(dpc)) != NULL) {
+    	if (entry[0] != '.') {
+    		gchar *filename;
+
+    		filename = g_build_filename(prefix, entry, NULL);
+
+    		/* ignore file if it can't be read. */
+    		if (g_access(filename, R_OK) == 0) {
+    		    GType mailbox_type;
+
+    			if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
+    				local_scanner_helper helper;
+    			    gpointer current_node;
+
+    				mailbox_type = libbalsa_mailbox_type_from_path(filename);
+
+    				if ((mailbox_type == LIBBALSA_TYPE_MAILBOX_MH) ||
+    					(mailbox_type == LIBBALSA_TYPE_MAILBOX_MAILDIR)) {
+    					current_node = mailbox_handler(rnode, entry, filename, mailbox_type);
+    					helper = libbalsa_scanner_mdir;
+    				} else {
+    					gchar *name = g_path_get_basename(prefix);
+
+    					current_node = folder_handler(rnode, name, filename, 0);
+    					g_free(name);
+    					helper = libbalsa_scanner_local_dir_helper;
+    				}
+
+    				++*depth;
+    				helper(current_node, filename, check_local_path, mark_local_path, folder_handler, mailbox_handler, depth);
+    				--*depth;
+    			} else {
+    				mailbox_type = libbalsa_mailbox_type_from_path(filename);
+    				if (mailbox_type != G_TYPE_OBJECT) {
+    					mark_local_path(mailbox_handler(rnode, entry, filename, mailbox_type));
+    				}
+    			}
+    		}
+    		g_free(filename);
+    	}
+    }
+    g_dir_close(dpc);
 }
 
 void
@@ -456,3 +449,174 @@ libbalsa_scanner_imap_dir(gpointer rnode, LibBalsaServer * server,
         g_signal_handler_disconnect(G_OBJECT(handle), lsub_handler_id);
     libbalsa_imap_server_release_handle(LIBBALSA_IMAP_SERVER(server), handle);
 }
+
+
+/* ---------------------------------------------------------------------
+ * IMAP folder tree functions (parent selection, subscription management)
+ * --------------------------------------------------------------------- */
+typedef struct {
+	gchar delim;
+	gboolean subscribed;
+	GHashTable *children;
+} folder_data_t;
+
+typedef struct {
+	GtkTreeStore *store;
+	GtkTreeIter *parent;
+} imap_tree_t;
+
+typedef struct {
+	ImapMboxHandle *handle;
+	gboolean subscriptions;
+} scan_data_t;
+
+static void imap_tree_scan(scan_data_t *scan_data,
+						   const gchar *list_path,
+						   GHashTable  *folders);
+static void imap_tree_to_store(gchar         *folder,
+							   folder_data_t *folder_data,
+							   imap_tree_t   *scan_data);
+
+GtkTreeStore *
+libbalsa_scanner_imap_tree(LibBalsaServer  *server,
+						   gboolean			subscriptions,
+						   GError         **error)
+{
+	scan_data_t scan_data;
+	imap_tree_t imap_store = { NULL, NULL };
+
+	g_return_val_if_fail(LIBBALSA_IS_IMAP_SERVER(server) && (error != NULL), NULL);
+
+	scan_data.handle = libbalsa_imap_server_get_handle(LIBBALSA_IMAP_SERVER(server), error);
+	if (scan_data.handle != NULL) {
+		GHashTable *folders;
+
+		scan_data.subscriptions = subscriptions;
+
+		/* scan the whole IMAP server tree */
+		folders = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+		imap_tree_scan(&scan_data, "", folders);
+		libbalsa_imap_server_release_handle(LIBBALSA_IMAP_SERVER(server), scan_data.handle);
+
+		/* create the resulting tree store */
+		imap_store.store = gtk_tree_store_new(LB_SCANNER_IMAP_N_COLS,
+			G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, PANGO_TYPE_STYLE);
+		g_hash_table_foreach(folders, (GHFunc) imap_tree_to_store, &imap_store);
+		g_hash_table_unref(folders);
+	} else {
+		if (*error == NULL) {
+			g_set_error(error, LIBBALSA_SCANNER_ERROR, LIBBALSA_SCANNER_ERROR_IMAP, _("Could not connect to “%s”"), server->host);
+		}
+	}
+
+	return imap_store.store;
+}
+
+
+static void
+imap_tree_scan_list_cb(ImapMboxHandle *handle,
+					   int             delim,
+					   ImapMboxFlags   flags,
+					   gchar          *folder,
+					   GHashTable     *folders)
+{
+	folder_data_t *folder_data;
+
+	folder_data = g_new0(folder_data_t, 1UL);
+	folder_data->delim = delim;
+	if (IMAP_MBOX_HAS_FLAG(flags, IMLIST_HASCHILDREN) != 0U) {
+		folder_data->children = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	}
+	g_hash_table_insert(folders, g_strdup(folder), folder_data);
+}
+
+
+static void
+imap_tree_scan_lsub_cb(ImapMboxHandle *handle,
+					   int             delim,
+					   ImapMboxFlags   flags,
+					   gchar          *folder,
+					   GHashTable     *folders)
+{
+	folder_data_t *folder_data;
+
+	folder_data = g_hash_table_lookup(folders, folder);
+	if (folder_data != NULL) {
+		folder_data->subscribed = IMAP_MBOX_HAS_FLAG(flags, IMLIST_NOSELECT) == 0U;
+	} else {
+		g_critical("%s: cannot identify folder %s", __func__, folder);
+	}
+}
+
+
+static void
+imap_tree_scan_children(gchar         *folder_name,
+						folder_data_t *folder_data,
+						scan_data_t   *scan_data)
+{
+	if (folder_data->children != NULL) {
+		gchar *scan_name = g_strdup_printf("%s%c", folder_name, folder_data->delim);
+
+		imap_tree_scan(scan_data, scan_name, folder_data->children);
+		g_free(scan_name);
+	}
+}
+
+
+static void
+imap_tree_scan(scan_data_t *scan_data,
+			   const gchar *list_path,
+			   GHashTable  *folders)
+{
+	gulong list_handler_id;
+
+	list_handler_id = g_signal_connect(G_OBJECT(scan_data->handle), "list-response", G_CALLBACK(imap_tree_scan_list_cb), folders);
+	imap_mbox_list(scan_data->handle, list_path);
+	g_signal_handler_disconnect(G_OBJECT(scan_data->handle), list_handler_id);
+
+	if (scan_data->subscriptions) {
+		list_handler_id =
+			g_signal_connect(G_OBJECT(scan_data->handle), "lsub-response", G_CALLBACK(imap_tree_scan_lsub_cb), folders);
+		imap_mbox_lsub(scan_data->handle, list_path);
+		g_signal_handler_disconnect(G_OBJECT(scan_data->handle), list_handler_id);
+	}
+
+	g_hash_table_foreach(folders, (GHFunc) imap_tree_scan_children, scan_data);
+}
+
+
+static void
+imap_tree_to_store(gchar 		 *folder,
+				   folder_data_t *folder_data,
+				   imap_tree_t   *tree_data)
+{
+	GtkTreeIter iter;
+	gchar *disp_name;
+
+	gtk_tree_store_append(tree_data->store, &iter, tree_data->parent);
+	if (tree_data->parent != NULL) {
+		gchar *parent_path;
+
+		gtk_tree_model_get(GTK_TREE_MODEL(tree_data->store), tree_data->parent, LB_SCANNER_IMAP_PATH, &parent_path, -1);
+		disp_name = g_strdup(&folder[strlen(parent_path) + 1UL]);
+		g_free(parent_path);
+	} else {
+		disp_name = g_strdup(folder);
+	}
+	gtk_tree_store_set(tree_data->store, &iter,
+		LB_SCANNER_IMAP_FOLDER, disp_name,
+		LB_SCANNER_IMAP_PATH, folder,
+		LB_SCANNER_IMAP_SUBS_NEW, folder_data->subscribed,
+		LB_SCANNER_IMAP_SUBS_OLD, folder_data->subscribed,
+		LB_SCANNER_IMAP_STYLE, PANGO_STYLE_NORMAL, -1);
+	g_free(disp_name);
+
+	if (folder_data->children != NULL) {
+		imap_tree_t child_data;
+
+		child_data.store = tree_data->store;
+		child_data.parent = &iter;
+		g_hash_table_foreach(folder_data->children, (GHFunc) imap_tree_to_store, &child_data);
+		g_hash_table_unref(folder_data->children);
+	}
+}
