@@ -117,6 +117,9 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
     ident->force_gpg_key_id = NULL;
     ident->force_smime_key_id = NULL;
+#ifdef ENABLE_AUTOCRYPT
+    ident->autocrypt_mode = AUTOCRYPT_DISABLE;
+#endif
     ident->request_mdn = FALSE;
     ident->request_dsn = FALSE;
     /*
@@ -455,7 +458,8 @@ libbalsa_identity_select_dialog(GtkWindow * parent,
     sdi = g_object_get_data(G_OBJECT(parent),
                             LIBBALSA_IDENTITY_SELECT_DIALOG_KEY);
     if (sdi) {
-        gtk_window_present(GTK_WINDOW(sdi->dialog));
+        gtk_window_present_with_time(GTK_WINDOW(sdi->dialog),
+                                     gtk_get_current_event_time());
         return;
     }
 
@@ -564,7 +568,6 @@ sd_response_ok(SelectDialogInfo * sdi)
  * The Manage Identities dialog; called from main window.
  */
 
-#define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
 typedef struct _IdentityDeleteInfo IdentityDeleteInfo;
 
 /* button actions */
@@ -636,6 +639,16 @@ static void ident_dialog_add_gpg_menu(GtkWidget * grid, gint row,
                                       GtkDialog * dialog,
                                       const gchar * label_name,
                                       const gchar * menu_key);
+#ifdef ENABLE_AUTOCRYPT
+static void ident_dialog_add_autocrypt_menu(GtkWidget   *grid,
+											gint         row,
+											GtkDialog   *dialog,
+											const gchar *label_name,
+											const gchar *menu_key);
+static void display_frame_set_autocrypt_mode(GObject       *dialog,
+											 const gchar   *key,
+											 AutocryptMode *value);
+#endif
 static void add_show_menu(const char *label, gpointer data,
                           GtkWidget * menu);
 static void ident_dialog_free_values(GPtrArray * values);
@@ -1068,6 +1081,11 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree,
                            	   	  _("use certificate with this id for signing S/MIME messages\n"
                            	   		"(leave empty for automatic selection)"),
 								  "identity-keyid-sm");
+#ifdef ENABLE_AUTOCRYPT
+    ident_dialog_add_autocrypt_menu(grid, row++, dialog,
+		 	 	 	 	 	 	 	_("Autocrypt mode"),
+									"identity-autocrypt");
+#endif
 #ifndef HAVE_GPGME
     gtk_widget_set_sensitive(grid, FALSE);
 #endif
@@ -1328,7 +1346,7 @@ md_sig_path_changed(gboolean active, GObject * dialog)
         "identity-sigpath",
     };
 
-    for (i = 0; i < ELEMENTS(button_key); i++) {
+    for (i = 0; i < G_N_ELEMENTS(button_key); i++) {
         GtkWidget *button = g_object_get_data(dialog, button_key[i]);
         gtk_widget_set_sensitive(button, active);
     }
@@ -1547,6 +1565,9 @@ ident_dialog_update(GObject * dlg)
     id->force_gpg_key_id = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid"));
     g_free(id->force_smime_key_id);
     id->force_smime_key_id = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid-sm"));
+#ifdef ENABLE_AUTOCRYPT
+    id->autocrypt_mode  = GPOINTER_TO_INT(ident_dialog_get_value(dlg, "identity-autocrypt"));
+#endif
 
     return TRUE;
 }
@@ -1760,7 +1781,8 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
 
     /* Show only one dialog at a time. */
     if (dialog) {
-        gtk_window_present(GTK_WINDOW(dialog));
+        gtk_window_present_with_time(GTK_WINDOW(dialog),
+                                     gtk_get_current_event_time());
         return;
     }
 
@@ -1786,7 +1808,7 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
     g_signal_connect(dialog, "response",
                      G_CALLBACK(md_response_cb), tree);
     g_object_set_data(G_OBJECT(dialog), "tree", tree);
-    g_object_add_weak_pointer(G_OBJECT(dialog), (gpointer) & dialog);
+    g_object_add_weak_pointer(G_OBJECT(dialog), (gpointer *) & dialog);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog),
                                     IDENTITY_RESPONSE_CLOSE);
 
@@ -1932,6 +1954,9 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
 			   &ident->crypt_protocol);
     display_frame_set_field(dialog, "identity-keyid", ident->force_gpg_key_id);
     display_frame_set_field(dialog, "identity-keyid-sm", ident->force_smime_key_id);
+#ifdef ENABLE_AUTOCRYPT
+    display_frame_set_autocrypt_mode(dialog, "identity-autocrypt", &ident->autocrypt_mode);
+#endif
 }
 
 
@@ -2034,6 +2059,9 @@ libbalsa_identity_new_config(const gchar* name)
     ident->crypt_protocol = libbalsa_conf_get_int("CryptProtocol=16");
     ident->force_gpg_key_id = libbalsa_conf_get_string("ForceKeyID");
     ident->force_smime_key_id = libbalsa_conf_get_string("ForceKeyIDSMime");
+#ifdef ENABLE_AUTOCRYPT
+    ident->autocrypt_mode = libbalsa_conf_get_int("Autocrypt=0");
+#endif
 
     return ident;
 }
@@ -2080,6 +2108,9 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     libbalsa_conf_set_int("CryptProtocol", ident->crypt_protocol);
     libbalsa_conf_set_string("ForceKeyID", ident->force_gpg_key_id);
     libbalsa_conf_set_string("ForceKeyIDSMime", ident->force_smime_key_id);
+#ifdef ENABLE_AUTOCRYPT
+    libbalsa_conf_set_int("Autocrypt", ident->autocrypt_mode);
+#endif
 
     libbalsa_conf_pop_group();
 }
@@ -2132,6 +2163,54 @@ display_frame_set_gpg_mode(GObject * dialog, const gchar* key, gint * value)
         }
 }
 
+#ifdef ENABLE_AUTOCRYPT
+static void
+display_frame_set_autocrypt_mode(GObject * dialog, const gchar* key, AutocryptMode * value)
+{
+    GtkComboBox *opt_menu = g_object_get_data(G_OBJECT(dialog), key);
+
+    switch (*value)
+        {
+        case AUTOCRYPT_NOPREFERENCE:
+	    gtk_combo_box_set_active(opt_menu, 1);
+            break;
+        case AUTOCRYPT_PREFER_ENCRYPT:
+	    gtk_combo_box_set_active(opt_menu, 2);
+            break;
+        default:
+	    gtk_combo_box_set_active(opt_menu, 0);
+            *value = AUTOCRYPT_DISABLE;
+        }
+}
+
+static void
+ident_dialog_add_autocrypt_menu(GtkWidget * grid, gint row, GtkDialog * dialog,
+                          	    const gchar * label_name, const gchar * menu_key)
+{
+    GtkWidget *label;
+    GtkWidget *opt_menu;
+    GPtrArray *values;
+
+    label = gtk_label_new_with_mnemonic(label_name);
+    gtk_widget_set_halign(label, GTK_ALIGN_START);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
+
+    opt_menu = gtk_combo_box_text_new();
+    values = g_ptr_array_sized_new(3);
+    g_object_set_data_full(G_OBJECT(opt_menu), "identity-value", values,
+                           (GDestroyNotify) ident_dialog_free_values);
+    gtk_grid_attach(GTK_GRID(grid), opt_menu, 1, row, 1, 1);
+    g_object_set_data(G_OBJECT(dialog), menu_key, opt_menu);
+
+    add_show_menu(_("disabled"),
+                  GINT_TO_POINTER(AUTOCRYPT_DISABLE), opt_menu);
+    add_show_menu(_("enabled, no preference"),
+                  GINT_TO_POINTER(AUTOCRYPT_NOPREFERENCE), opt_menu);
+    add_show_menu(_("enabled, prefer encryption"),
+                  GINT_TO_POINTER(AUTOCRYPT_PREFER_ENCRYPT), opt_menu);
+}
+#endif
+
 /*
  * Add an option menu to the given dialog with a label next to it
  * explaining the contents.  A reference to the entry is stored as
