@@ -30,10 +30,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#if defined(HAVE_LIBSECRET)
-#include <libsecret/secret.h>
-#endif                          /* defined(HAVE_LIBSECRET) */
-
 #include "libbalsa.h"
 #include "libbalsa-conf.h"
 #include "server.h"
@@ -42,8 +38,6 @@
 #include "imap-commands.h"
 #include <glib/gi18n.h>
 
-#define REQ_SSL(s) (LIBBALSA_SERVER(s)->use_ssl)
-
 /** wait 60 seconds for packets */
 #define IMAP_CMD_TIMEOUT (60*1000)
 
@@ -61,13 +55,13 @@ struct LibBalsaImapServer_ {
     guint used_connections;
     GList *used_handles;
     GList *free_handles;
-    unsigned persistent_cache:1; /* if TRUE, messages will be cached in
+    gboolean persistent_cache; /* if TRUE, messages will be cached in
                                     $HOME and preserved between
                                     sessions. If FALSE, messages will be
                                     kept in /tmp and cleaned on exit. */
-    unsigned has_fetch_bug:1;
-    unsigned use_status:1; /**< server has fast STATUS command */
-    unsigned use_idle:1;  /**< IDLE will work: no dummy firewall on the way */
+    gboolean has_fetch_bug;
+    gboolean use_status; /**< server has fast STATUS command */
+    gboolean use_idle;  /**< IDLE will work: no dummy firewall on the way */
 };
 
 typedef struct LibBalsaImapServerClass_ {
@@ -140,6 +134,9 @@ static void libbalsa_imap_server_set_username(LibBalsaServer * server,
     if(server->host && name) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
         
+        if (imap_server->key == NULL) {
+        	imap_server->key = g_strdup_printf("%s@%s", name, server->host);
+        }
         g_mutex_lock(&imap_servers_lock);
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
@@ -149,12 +146,18 @@ static void libbalsa_imap_server_set_username(LibBalsaServer * server,
     }
     (parent_class)->set_username(server, name);
 }
+
 static void
-libbalsa_imap_server_set_host(LibBalsaServer * server,
-                              const gchar * host, gboolean use_ssl)
+libbalsa_imap_server_set_host(LibBalsaServer     *server,
+                              const gchar        *host,
+							  NetClientCryptMode  security)
 {
     if(server->user && host) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
+
+        if (imap_server->key == NULL) {
+        	imap_server->key = g_strdup_printf("%s@%s", server->user, host);
+        }
         g_mutex_lock(&imap_servers_lock);
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
@@ -162,8 +165,9 @@ libbalsa_imap_server_set_host(LibBalsaServer * server,
         g_hash_table_insert(imap_servers, imap_server->key, imap_server);
         g_mutex_unlock(&imap_servers_lock);
     }
-    (parent_class)->set_host(server, host, use_ssl);
+    (parent_class)->set_host(server, host, security);
 }
+
 static void
 libbalsa_imap_server_class_init(LibBalsaImapServerClass * klass)
 {
@@ -224,55 +228,6 @@ libbalsa_imap_server_finalize(GObject * object)
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
-gint ImapDebug = 0;
-#define BALSA_TEST_IMAP 1
-
-static void
-monitor_cb(const char *buffer, int length, int direction, void *arg)
-{
-#if BALSA_TEST_IMAP
-  if (ImapDebug) {
-    const gchar *passwd = NULL;
-    int i;
-
-    if (direction) {
-      const gchar *login;
-      int login_length;
-          
-      login = g_strstr_len(buffer, length, "LOGIN ");
-      if (login) {
-        login_length = 6;
-      } else {
-        login = g_strstr_len(buffer, length, "AUTHENTICATE PLAIN");
-        login_length = 18;
-      }
-
-      if (login) {
-        const gchar *user = login + login_length;
-        passwd = g_strstr_len(user, length - (user - buffer), " ");
-        if (passwd) {
-          int new_len = ++passwd - buffer;
-          if (new_len < length)
-            length = new_len;
-          else
-            passwd = NULL;
-        }
-      }
-    }
-
-    printf("IMAP %c: ", direction ? 'C' : 'S');
-    for (i = 0; i < length; i++)
-      putchar(buffer[i]);
-
-    if (passwd)
-      puts("(password hidden)");
-
-    fflush(NULL);
-  }
-#endif                          /* BALSA_TEST_IMAP */
-  if (direction)
-    ((struct handle_info *) arg)->last_used = time(NULL);
-}
 
 static void
 is_info_cb(ImapMboxHandle *h, ImapResponse rc, const gchar* str, void *arg)
@@ -307,7 +262,6 @@ lb_imap_server_info_new(LibBalsaServer *server)
 {
     ImapMboxHandle *handle;
     struct handle_info *info;
-    ImapTlsMode mode;
 
     /* We do not ask for password now since the authentication might
      * not require it. Instead, we handle authentication requiests in
@@ -317,16 +271,10 @@ lb_imap_server_info_new(LibBalsaServer *server)
     imap_handle_set_timeout(handle, IMAP_CMD_TIMEOUT);
     info = g_new0(struct handle_info, 1);
     info->handle = handle;
-    imap_handle_set_monitorcb(handle, monitor_cb, info);
     imap_handle_set_infocb(handle,    is_info_cb, server);
-    imap_handle_set_usercb(handle,    libbalsa_server_user_cb, server);
-    switch(server->tls_mode) {
-    case LIBBALSA_TLS_DISABLED: mode = IMAP_TLS_DISABLED; break;
-    default:
-    case LIBBALSA_TLS_ENABLED : mode = IMAP_TLS_ENABLED;  break;
-    case LIBBALSA_TLS_REQUIRED: mode = IMAP_TLS_REQUIRED; break;
-    }
-    imap_handle_set_tls_mode(handle, mode);
+    imap_handle_set_authcb(handle, G_CALLBACK(libbalsa_server_get_auth), server);
+    imap_handle_set_certcb(handle, G_CALLBACK(libbalsa_server_check_cert));
+    imap_handle_set_tls_mode(handle, server->security);
     imap_handle_set_option(handle, IMAP_OPT_ANONYMOUS, server->try_anonymous);
     imap_handle_set_option(handle, IMAP_OPT_CLIENT_SORT, TRUE);
 #ifdef HAVE_GPGME
@@ -347,9 +295,10 @@ lb_imap_server_info_new(LibBalsaServer *server)
 static void
 lb_imap_server_info_free(struct handle_info *info)
 {
-    g_assert(info != NULL);
+    if (info == NULL)
+        return;
 
-    imap_handle_force_disconnect(info->handle);
+    //imap_handle_force_disconnect(info->handle); -- FIXME unref'ing handle will disconnect?
     g_object_unref(info->handle);
     g_free(info);
 }
@@ -375,7 +324,7 @@ lb_imap_server_cleanup(LibBalsaImapServer * imap_server)
         GList *next = list->next;
         struct handle_info *info = list->data;
 
-        if (info->last_used < idle_marker) {
+        if (info == NULL || info->last_used < idle_marker) {
             imap_server->free_handles =
                 g_list_delete_link(imap_server->free_handles, list);
             lb_imap_server_info_free(info);
@@ -461,110 +410,64 @@ LibBalsaImapServer* libbalsa_imap_server_new(const gchar *username,
     return get_or_create(username, host);
 }
 
+static inline void
+set_bool_if_defined(const gchar *path, gboolean *dest)
+{
+	gboolean dflt;
+	gboolean val;
+
+    val = libbalsa_conf_get_bool_with_default(path, &dflt);
+    if (!dflt) {
+    	*dest = val;
+    }
+}
+
 LibBalsaImapServer*
 libbalsa_imap_server_new_from_config(void)
 {
-    LibBalsaServer tmp_server;
+	gchar *host;
+	gchar *user;
     LibBalsaImapServer *imap_server;
     LibBalsaServer *server;
-    gboolean d, d1;
-    gint tls_mode, conn_limit;
+    gboolean d;
 
-    tmp_server.host = libbalsa_conf_get_string("Server");
-    if(strrchr(tmp_server.host, ':') == NULL) {
+    host = libbalsa_conf_get_string("Server");
+    if (strrchr(host, ':') == NULL) {
         gint port;
+
         port = libbalsa_conf_get_int_with_default("Port", &d);
         if (!d) {
-            gchar *newhost = g_strdup_printf("%s:%d", tmp_server.host, port);
-            g_free(tmp_server.host);
-            tmp_server.host = newhost;
+            gchar *newhost = g_strdup_printf("%s:%d", host, port);
+            g_free(host);
+            host = newhost;
         }
     }       
-    tmp_server.user = libbalsa_conf_private_get_string("Username");
-    if (!tmp_server.user)
-        tmp_server.user = g_strdup(getenv("USER"));
+    user = libbalsa_conf_private_get_string("Username", FALSE);
+    if (user == NULL) {
+        user = g_strdup(g_get_user_name());
+    }
 
-    imap_server = get_or_create(tmp_server.user, tmp_server.host);
+    imap_server = get_or_create(user, host);
+    g_free(user);
+    g_free(host);
     server = LIBBALSA_SERVER(imap_server);
-    if (server->user) {
-        g_free(tmp_server.user);
-        g_free(tmp_server.host);
-    } else {
-        server->user = tmp_server.user;
-        server->host = tmp_server.host;
-    }
-    d1 = libbalsa_conf_get_bool_with_default("Anonymous", &d);
-    if(!d) server->try_anonymous = !!d1;
-    server->use_ssl |= libbalsa_conf_get_bool("SSL=false");
-    tls_mode = libbalsa_conf_get_int_with_default("TLSMode", &d);
-    if(!d) server->tls_mode = tls_mode;
-    conn_limit = libbalsa_conf_get_int_with_default("ConnectionLimit", &d);
-    if(!d) imap_server->max_connections = conn_limit;
-    d1 = libbalsa_conf_get_bool_with_default("PersistentCache", &d);
-    if(!d) imap_server->persistent_cache = !!d1;
-    d1 = libbalsa_conf_get_bool_with_default("HasFetchBug", &d);
-    if(!d) imap_server->has_fetch_bug = !!d1;
-    d1 = libbalsa_conf_get_bool_with_default("UseStatus", &d);
-    if(!d) imap_server->use_status = !!d1;
-    d1 = libbalsa_conf_get_bool_with_default("UseIdle", &d);
-    if(!d) imap_server->use_idle = !!d1;
-    if (!server->passwd) {
-        server->remember_passwd = libbalsa_conf_get_bool("RememberPasswd=false");
-        if(server->remember_passwd) {
-#if defined(HAVE_LIBSECRET)
-            GError *err = NULL;
+    if (server->host == NULL) {
+        gint conn_limit;
 
-            server->passwd =
-                secret_password_lookup_sync(LIBBALSA_SERVER_SECRET_SCHEMA,
-                                            NULL, &err,
-                                            "protocol", server->protocol,
-                                            "server",   server->host,
-                                            "user",     server->user,
-                                            NULL);
-            if (err) {
-                libbalsa_free_password(server->passwd);
-                server->passwd = NULL;
-                printf(_("Error looking up password for %s@%s: %s\n"),
-                       server->user, server->host, err->message);
-                printf(_("Falling back\n"));
-                g_clear_error(&err);
-                server->passwd =
-                    libbalsa_conf_private_get_string("Password");
-                if (server->passwd != NULL) {
-                    gchar *buff = libbalsa_rot(server->passwd);
-                    libbalsa_free_password(server->passwd);
-                    server->passwd = buff;
-                    secret_password_store_sync
-                        (LIBBALSA_SERVER_SECRET_SCHEMA, NULL,
-                         _("Balsa passwords"), server->passwd, NULL, &err,
-                         "protocol", server->protocol,
-                         "server",   server->host,
-                         "user",     server->user,
-                         NULL);
-                    /* We could in principle clear the password in the
-                     * config file here but we do not for the backward
-                     * compatibility. */
-                    if (err) {
-                        printf(_("Error storing password for %s@%s: %s\n"),
-                               server->user, server->host, err->message);
-                        g_error_free(err);
-                    }
-                }
-            }
-#else
-            server->passwd = libbalsa_conf_private_get_string("Password");
-	    if (server->passwd != NULL) {
-		gchar *buff = libbalsa_rot(server->passwd);
-		libbalsa_free_password(server->passwd);
-		server->passwd = buff;
-	    }
-#endif                          /* defined(HAVE_LIBSECRET) */
-	}
-        if(server->passwd && server->passwd[0] == '\0') {
-            libbalsa_free_password(server->passwd);
-            server->passwd = NULL;
+        /* common server configs */
+    	libbalsa_server_load_config(server);
+
+    	/* imap specials */
+        conn_limit = libbalsa_conf_get_int_with_default("ConnectionLimit", &d);
+        if (!d) {
+        	imap_server->max_connections = conn_limit;
         }
+        set_bool_if_defined("PersistentCache", &imap_server->persistent_cache);
+        set_bool_if_defined("HasFetchBug", &imap_server->has_fetch_bug);
+        set_bool_if_defined("UseStatus", &imap_server->use_status);
+        set_bool_if_defined("UseIdle", &imap_server->use_idle);
     }
+
     return imap_server;
 }
 
@@ -588,7 +491,7 @@ handle_connection_error(int rc, struct handle_info *info,
     gchar *msg = imap_mbox_handle_get_last_msg(info->handle);
     switch(rc) {
     case IMAP_AUTH_FAILURE:
-        libbalsa_server_set_password(server, NULL); break;
+        libbalsa_server_set_password(server, NULL, FALSE); break;
     case IMAP_CONNECT_FAILED:
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_NETWORK_ERROR,
@@ -662,8 +565,7 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
         if(imap_mbox_is_disconnected(info->handle)) {
             ImapResult rc;
 
-            rc=imap_mbox_handle_connect(info->handle, server->host,
-                                        REQ_SSL(server));
+            rc=imap_mbox_handle_connect(info->handle, server->host);
             if(rc != IMAP_SUCCESS) {
                 handle_connection_error(rc, info, server, err);
                 g_mutex_unlock(&imap_server->lock);
@@ -760,8 +662,7 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
     if (imap_mbox_is_disconnected(info->handle)) {
         ImapResult rc;
 
-        rc=imap_mbox_handle_connect(info->handle, server->host,
-                                    REQ_SSL(server));
+        rc=imap_mbox_handle_connect(info->handle, server->host);
         if(rc != IMAP_SUCCESS) {
             handle_connection_error(rc, info, server, err);
             g_mutex_unlock(&imap_server->lock);
@@ -826,7 +727,7 @@ libbalsa_imap_server_set_max_connections(LibBalsaImapServer *server,
                                          int max)
 {
     server->max_connections = max;
-    printf("set_max_connections: set to %d\n", max);
+    g_debug("set_max_connections: set to %d", max);
 }
 
 int
@@ -839,7 +740,7 @@ void
 libbalsa_imap_server_enable_persistent_cache(LibBalsaImapServer *server,
                                              gboolean enable)
 {
-    server->persistent_cache = !!enable;
+    server->persistent_cache = enable;
 }
 gboolean
 libbalsa_imap_server_has_persistent_cache(LibBalsaImapServer *srv)
@@ -944,7 +845,7 @@ void
 libbalsa_imap_server_set_bug(LibBalsaImapServer *server,
                              LibBalsaImapServerBug bug, gboolean hasp)
 {
-    server->has_fetch_bug = !! hasp;
+    server->has_fetch_bug = hasp;
 }
 
 gboolean
@@ -958,7 +859,7 @@ void
 libbalsa_imap_server_set_use_status(LibBalsaImapServer *server, 
                                     gboolean use_status)
 {
-    server->use_status = !!use_status;
+    server->use_status = use_status;
 }
 gboolean
 libbalsa_imap_server_get_use_status(LibBalsaImapServer *server)
@@ -972,9 +873,8 @@ libbalsa_imap_server_set_use_idle(LibBalsaImapServer *server,
 {
     GList *list;
 
-    server->use_idle = !!use_idle;
-    printf("Server will%s use IDLE\n",
-           server->use_idle ? "" : " NOT");
+    server->use_idle = use_idle;
+    g_debug("Server will%s use IDLE", server->use_idle ? "" : " NOT");
     for (list = server->used_handles; list; list = list->next) {
         struct handle_info *info = list->data;
         imap_handle_set_option(info->handle, IMAP_OPT_IDLE,
@@ -987,3 +887,55 @@ libbalsa_imap_server_get_use_idle(LibBalsaImapServer *server)
 {
     return server->use_idle;
 }
+
+gboolean
+libbalsa_imap_server_subscriptions(LibBalsaImapServer  *server,
+								   GPtrArray           *subscribe,
+								   GPtrArray 		   *unsubscribe,
+								   GError 			  **error)
+{
+	ImapMboxHandle *handle;
+	gboolean result;
+
+	g_return_val_if_fail(LIBBALSA_IS_IMAP_SERVER(server), FALSE);
+
+	handle = libbalsa_imap_server_get_handle(server, error);
+	if (handle != NULL) {
+		guint n;
+
+		result = TRUE;
+		if (subscribe != NULL) {
+			for (n = 0U; result && (n < subscribe->len); n++) {
+				const gchar *mailbox = (const gchar *) g_ptr_array_index(subscribe, n);
+
+				if (imap_mbox_subscribe(handle, mailbox, TRUE) != IMR_OK) {
+					g_set_error(error, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+						_("subscribing to “%s” failed"), mailbox);
+					result = FALSE;
+				} else {
+					g_debug("subscribed to %s, '%s'", LIBBALSA_SERVER(server)->host, mailbox);
+				}
+			}
+		}
+
+		if (result && (unsubscribe != NULL)) {
+			for (n = 0U; result && (n < unsubscribe->len); n++) {
+				const gchar *mailbox = (const gchar *) g_ptr_array_index(unsubscribe, n);
+
+				if (imap_mbox_subscribe(handle, mailbox, FALSE) != IMR_OK) {
+					g_set_error(error, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+						_("unsubscribing from “%s” failed"), mailbox);
+					result = FALSE;
+				} else {
+					g_debug("unsubscribed from %s, '%s'", LIBBALSA_SERVER(server)->host, mailbox);
+				}
+			}
+		}
+
+		libbalsa_imap_server_release_handle(server, handle);
+	} else {
+		result = FALSE;
+	}
+
+	return result;
+}
