@@ -19,15 +19,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <glib/gi18n.h>
 
 #include "imap-handle.h"
 #include "imap-commands.h"
 #include "imap_private.h"
-#include "siobuf.h"
+#include "siobuf-nc.h"
 #include "util.h"
 
-#define ELEMENTS(x) (sizeof (x) / sizeof(x[0]))
-
 struct fetch_data {
   ImapMboxHandle* h;
   ImapFetchType ift;
@@ -59,7 +58,7 @@ need_fetch(unsigned seqno, struct fetch_data* fd)
       && fd->h->msg_cache[seqno-1]->rfc822size <0) return seqno;
 
   available_headers = fd->h->msg_cache[seqno-1]->available_headers;
-  for(i=0; i<ELEMENTS(header); i++) {
+  for(i=0; i<G_N_ELEMENTS(header); i++) {
     if( (fd->ift & header[i]) &&
         !(available_headers & (header[i]|IMFETCH_RFC822HEADERS|
                                IMFETCH_RFC822HEADERS_SELECTED))) {
@@ -122,7 +121,7 @@ imap_check_capability(ImapMboxHandle* handle)
 
   if (!(imap_mbox_handle_can_do(handle, IMCAP_IMAP4) ||
         imap_mbox_handle_can_do(handle, IMCAP_IMAP4REV1))) {
-    g_warning("IMAP4rev1 required but not provided.\n");
+    g_warning("IMAP4rev1 required but not provided.");
     return FALSE;
   }  
   return TRUE;
@@ -406,13 +405,13 @@ ImapResponse
 imap_mbox_status(ImapMboxHandle *r, const char*what,
                  struct ImapStatusResult *res)
 {
-  const char *item_arr[ELEMENTS(imap_status_item_names)+1];
+  const char *item_arr[G_N_ELEMENTS(imap_status_item_names)+1];
   ImapResponse rc = IMR_OK;
   unsigned i, ipos;
   
   for(ipos = i= 0; res[i].item != IMSTAT_NONE; i++) {
     /* repeated items? */
-    g_return_val_if_fail(i<ELEMENTS(imap_status_item_names), IMR_BAD);
+    g_return_val_if_fail(i<G_N_ELEMENTS(imap_status_item_names), IMR_BAD);
     /* invalid item? */
     g_return_val_if_fail(res[i].item>=IMSTAT_MESSAGES &&
                          res[i].item<=IMSTAT_UNSEEN, IMR_BAD);
@@ -440,7 +439,7 @@ enum_flag_to_str(ImapMsgFlags flg)
   GString *flags_str = g_string_new("");
   unsigned idx;
 
-  for(idx=0; idx < ELEMENTS(imap_msg_flags); idx++) {
+  for(idx=0; idx < G_N_ELEMENTS(imap_msg_flags); idx++) {
     if((flg & (1<<idx)) == 0) continue;
     if(*flags_str->str) g_string_append_c(flags_str, ' ');
     g_string_append_c(flags_str, '\\');
@@ -495,11 +494,8 @@ append_commit(ImapMboxHandle *handle, unsigned cmdno, ImapSequence *uid_seq)
     handle->uidplus.dst = uid_seq->ranges;
     handle->uidplus.store_response = 1;
   }
-  sio_write(handle->sio, "\r\n", 2);
-  sio_flush(handle->sio);
-  do {
-    rc = imap_cmd_step (handle, cmdno);
-  } while (rc == IMR_UNTAGGED);
+  net_client_siobuf_flush(handle->sio, NULL);
+  rc = imap_cmd_process_untagged(handle, cmdno);
 
   if(uid_seq) {
     if(uid_seq->uid_validity == 0) {
@@ -579,27 +575,21 @@ imap_mbox_append_multi_real(ImapMboxHandle *handle,
       /* MULTIAPPEND continuation */
       if(flags) {
 	gchar *str = enum_flag_to_str(flags);
-	sio_printf(handle->sio, " (%s) {%lu%s}\r\n", str,
+	sio_printf(handle->sio, " (%s) {%lu%s}", str,
 		   (unsigned long)msg_size, litstr);
 	g_free(str);
       } else 
-	sio_printf(handle->sio, " {%lu%s}\r\n",
+	sio_printf(handle->sio, " {%lu%s}",
 		   (unsigned long)msg_size, litstr);
     }
 
     if(use_literal)
       rc = IMR_RESPOND; /* we do it without flushing */
     else {
-      sio_flush(handle->sio);
-      do {
-	rc = imap_cmd_step (handle, cmdno);
-      } while (rc == IMR_UNTAGGED);
+    	net_client_siobuf_flush(handle->sio, NULL);
+    	rc = imap_cmd_process_untagged(handle, cmdno);
       if(rc != IMR_RESPOND) return rc;
-      EAT_LINE(handle, c);
-      if(c == -1) {
-	imap_handle_disconnect(handle);
-	return IMR_SEVERED;
-      }
+      net_client_siobuf_discard_line(handle->sio, NULL);
     }
 
     for(s=0; s<msg_size; s+= delta) {
@@ -940,12 +930,9 @@ imap_assure_needed_flags(ImapMboxHandle *h, ImapMsgFlag needed_flags)
     }
   }
   if(cmd) { /* was ever altered */
-    sio_flush(h->sio);
     for(i=0; i<issued_cmd; i++) {
       h->search_arg = &flag[i];
-      do {
-        rc = imap_cmd_step(h, cmdno[i]);
-      } while (rc == IMR_UNTAGGED);
+      rc = imap_cmd_process_untagged(h, cmdno[i]);
     }
     imap_handle_idle_enable(h, 30);
   }
@@ -1040,7 +1027,7 @@ set_avail_headers(ImapMboxHandle *h, const char *seq, ImapFetchType ift)
         }
       
       break;
-    default: g_warning("unexpected sequence %s\n", seq);
+    default: g_warning("unexpected sequence %s", seq);
     }
     if(*tmp==',') tmp++;
     s = tmp;
@@ -1227,7 +1214,7 @@ imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
     g_free(seq);
     if(passthrough_data.pipeline_error){
       rc = IMR_NO;
-      imap_mbox_handle_set_msg(handle, "Unordered data received from server");
+      imap_mbox_handle_set_msg(handle, _("Unordered data received from server"));
     }
   }
   g_mutex_unlock(&handle->mutex);
@@ -1395,7 +1382,7 @@ imap_mbox_handle_fetch_body(ImapMboxHandle* handle,
                             ImapFetchBodyOptions options,
                             ImapFetchBodyCb body_cb, void *arg)
 {
-  char cmd[160];
+  char cmd[200];
   ImapFetchBodyInternalCb fcb;
   void          *farg;
   ImapResponse rc;
@@ -1445,7 +1432,7 @@ imap_mbox_handle_fetch_body(ImapMboxHandle* handle,
       /* We have to strip last section part and replace it with HEADER */
       unsigned sz;
       char *last_dot = strrchr(section, '.');
-      strncpy(prefix, section, sizeof(prefix));
+      strncpy(prefix, section, sizeof(prefix) - 1);
       
       if(last_dot) {
         sz = last_dot-section+1;
@@ -1541,7 +1528,9 @@ imap_mbox_store_flag(ImapMboxHandle *h, unsigned msgcnt, unsigned*seqno,
   if(cmd) {
     res = imap_cmd_exec(h, cmd);
     g_free(cmd);
-  } res = IMR_OK;
+  } else {
+    res = IMR_OK; /* No action needs to be taken */
+  }
   g_mutex_unlock(&h->mutex);
   return res;
 }
@@ -1681,11 +1670,8 @@ imap_mbox_thread(ImapMboxHandle *h, const char *how, ImapSearchKey *filter)
         return rc;
     }
 
-    sio_write(h->sio, "\r\n", 2);
-    imap_handle_flush(h);
-    do
-      rc = imap_cmd_step(h, cmdno);
-    while(rc == IMR_UNTAGGED);
+    net_client_siobuf_flush(h->sio, NULL);
+    rc = imap_cmd_process_untagged(h, cmdno);
     imap_handle_idle_enable(h, 30);
   }
  exit_cleanup:
@@ -1763,8 +1749,8 @@ imap_mbox_sort_msgno_srv(ImapMboxHandle *handle, ImapSortKey key,
       for(i=0; i<cnt; i++)
         msgno[i] = i + 1;
       imap_mbox_handle_set_msg(handle,
-                               "bug in implementation of SORT command on "
-                               "IMAP server exposed.");
+                               _("Bug in implementation of SORT command on "
+                               "IMAP server exposed."));
       rc = IMR_NO;
     }
   }
@@ -1884,6 +1870,10 @@ imap_mbox_sort_msgno_client(ImapMboxHandle *handle, ImapSortKey key,
   sort_items = g_new(struct SortItem, cnt);
   for(i=0; i<cnt; i++) {
     sort_items[i].msg = imap_mbox_handle_get_msg(handle, msgno[i]);
+    if ((sort_items[i].msg == NULL) || (sort_items[i].msg->envelope == NULL)) {
+      g_free(sort_items);
+      return IMR_BAD;
+    }
     sort_items[i].no  = msgno[i];
   }
   switch(key) {
@@ -1998,12 +1988,9 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
             != IMR_OK)
           return rc;
       }
-      sio_write(handle->sio, "\r\n", 2);
       imap_handle_idle_enable(handle, 30);
-      imap_handle_flush(handle);
-      do
-        rc = imap_cmd_step(handle, cmdno);
-      while(rc == IMR_UNTAGGED);
+      net_client_siobuf_flush(handle->sio, NULL);
+      rc = imap_cmd_process_untagged(handle, cmdno);
     } else {                                           /* CASE 2b */
       /* try client-side sorting... */
       if(handle->enable_client_sort) {
