@@ -27,17 +27,9 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <sys/utsname.h>
 #include <sys/stat.h>
 #include <stdarg.h>
 #include <unistd.h>
-#include <openssl/ssl.h>
-#include <openssl/x509.h>
-#include <openssl/err.h>
-
-#ifdef HAVE_NOTIFY
-#include <libnotify/notify.h>
-#endif
 
 #if ENABLE_LDAP
 #include <ldap.h>
@@ -54,6 +46,8 @@
 #if HAVE_GCR
 #define GCR_API_SUBJECT_TO_CHANGE
 #include <gcr/gcr.h>
+#else
+#include <gnutls/x509.h>
 #endif
 
 #include "misc.h"
@@ -75,20 +69,10 @@ libbalsa_message(const char *fmt, ...)
 }
 
 void
-libbalsa_init(LibBalsaInformationFunc information_callback)
+libbalsa_init(void)
 {
-    struct utsname utsname;
-
-#ifdef HAVE_NOTIFY
-    notify_init("Basics");
-#endif
-
     main_thread_id = g_thread_self();
 
-    uname(&utsname);
-
-    libbalsa_real_information_func = information_callback;
-
     g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 
     GMIME_TYPE_DATA_WRAPPER;
@@ -113,7 +97,7 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
     LIBBALSA_TYPE_MESSAGE;
 
     LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
-    LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN;
+    LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ;
     LIBBALSA_TYPE_ADDRESS_BOOK_LDIF;
 #if ENABLE_LDAP
     LIBBALSA_TYPE_ADDRESS_BOOK_LDAP;
@@ -130,32 +114,6 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 }
 
 
-/* libbalsa_rot:
-   return rot13'ed string.
-*/
-gchar *
-libbalsa_rot(const gchar * pass)
-{
-    gchar *buff;
-    gint len = 0, i = 0;
-
-    /*PKGW: let's do the assert() BEFORE we coredump... */
-
-    len = strlen(pass);
-    buff = g_strdup(pass);
-
-    for (i = 0; i < len; i++) {
-	if ((buff[i] <= 'M' && buff[i] >= 'A')
-	    || (buff[i] <= 'm' && buff[i] >= 'a'))
-	    buff[i] += 13;
-	else if ((buff[i] <= 'Z' && buff[i] >= 'N')
-		 || (buff[i] <= 'z' && buff[i] >= 'n'))
-	    buff[i] -= 13;
-    }
-    return buff;
-}
-
-
 /* libbalsa_guess_email_address:
    Email address can be determined in four ways:
    1. Using the environment variable 'EMAIL'
@@ -173,26 +131,29 @@ libbalsa_guess_email_address(void)
 {
     /* Q: Find this location with configure? or at run-time? */
     static const gchar* MAILNAME_FILE = "/etc/mailname";
-    char hostbuf[512];
-    FILE *mailname_in = NULL;
-
     gchar* preset, *domain;
+    gchar *mailname;
+
     if(g_getenv("EMAIL") != NULL){                  /* 1. */
         preset = g_strdup(g_getenv("EMAIL"));
-    } else if( (mailname_in = fopen(MAILNAME_FILE, "r")) != NULL
-              && fgets(hostbuf, sizeof(hostbuf)-1, mailname_in)){ /* 2. */
-        hostbuf[sizeof(hostbuf)-1] = '\0';
-        preset = g_strconcat(g_get_user_name(), "@", hostbuf, NULL);
-        
+    } else if (g_file_get_contents(MAILNAME_FILE, &mailname, NULL, NULL)) { /* 2. */
+    	gchar *newline;
+
+    	newline = strchr(mailname, '\n');
+    	if (newline != NULL) {
+    		newline[0] = '\0';
+    	}
+        preset = g_strconcat(g_get_user_name(), "@", mailname, NULL);
+        g_free(mailname);
     }else if((domain = libbalsa_get_domainname())){ /* 3. */
         preset = g_strconcat(g_get_user_name(), "@", domain, NULL);
         g_free(domain);    
     } else {                                        /* 4. */
+        char hostbuf[512];
+
         gethostname(hostbuf, 511);
         preset = g_strconcat(g_get_user_name(), "@", hostbuf, NULL);
     }
-    if (mailname_in)
-        fclose(mailname_in);
     return preset;
 }
 
@@ -308,10 +269,8 @@ ask_idle(gpointer data)
 {
     AskData* ad = (AskData*)data;
     printf("ask_idle: ENTER %p\n", data);
-    gdk_threads_enter();
     ad->res = (ad->cb)(ad->arg);
     ad->done = TRUE;
-    gdk_threads_leave();
     g_cond_signal(&ad->condvar);
     printf("ask_idle: LEAVE %p\n", data);
     return FALSE;
@@ -329,9 +288,7 @@ libbalsa_ask(gboolean (*cb)(void *arg), void *arg)
     if (!libbalsa_am_i_subthread()) {
         int ret;
         printf("Main thread asks the following question.\n");
-        gdk_threads_enter();
         ret = cb(arg);
-        gdk_threads_leave();
         return ret;
     }
     printf("Side thread asks the following question.\n");
@@ -353,145 +310,87 @@ libbalsa_ask(gboolean (*cb)(void *arg), void *arg)
 }
 
 
-static int libbalsa_ask_for_cert_acceptance(X509 *cert,
-					    const char *explanation);
+static int libbalsa_ask_for_cert_acceptance(GTlsCertificate      *cert,
+											GTlsCertificateFlags  errors);
 
-#ifndef HAVE_GCR
-
-static char*
-asn1time_to_string(ASN1_UTCTIME *tm)
-{
-    char buf[64];
-    BIO *bio  = BIO_new(BIO_s_mem());
-    strncpy(buf, _("Invalid date"), sizeof(buf)); buf[sizeof(buf)-1]='\0';
-
-    if(ASN1_TIME_print(bio, tm)) {
-        int cnt;
-        cnt = BIO_read(bio, buf, sizeof(buf)-1);
-        buf[cnt] = '\0';
-    }
-    BIO_free(bio);
-    return g_strdup(buf);
-}
-
-static char*
-x509_get_part (char *line, const char *ndx)
-{
-    static char ret[256];
-    char *c;
-
-    strncpy (ret, _("Unknown"), sizeof (ret)); ret[sizeof(ret)-1]='\0';
-
-    c = strstr(line, ndx);
-    if (c) {
-        char *c2;
-
-        c += strlen (ndx);
-        c2 = strchr (c, '/');
-        if (c2)
-            *c2 = '\0';
-        strncpy (ret, c, sizeof (ret));
-        if (c2)
-            *c2 = '/';
-    }
-
-    return ret;
-}
-static void
-x509_fingerprint (char *s, unsigned len, X509 * cert)
-{
-    unsigned j, i, n, c;
-    unsigned char md[EVP_MAX_MD_SIZE];
-
-
-    X509_digest(cert, EVP_md5(), md, &n);
-    if(len<3*n) n = len/3;
-    for (j=i=0; j<n; j++) {
-        c = (md[j] >>4) & 0xF; s[i++] = c<10 ? c + '0' : c+'A'-10;
-        c = md[j] & 0xF;       s[i++] = c<10 ? c + '0' : c+'A'-10;
-        if(j<n-1) s[i++] = ':';
-    }
-    s[i] = '\0';
-}
-
-#endif  /* HAVE_GCR */
-
-static GList *accepted_certs = NULL; /* certs accepted for this session */
+static GList *accepted_certs = NULL; /* GTlsCertificate items accepted for this session */
 static GMutex certificate_lock;
 
 void
 libbalsa_certs_destroy(void)
 {
 	g_mutex_lock(&certificate_lock);
-    g_list_foreach(accepted_certs, (GFunc)X509_free, NULL);
-    g_list_free(accepted_certs);
+    g_list_free_full(accepted_certs, g_object_unref);
     accepted_certs = NULL;
     g_mutex_unlock(&certificate_lock);
 }
 
-/* compare Example 10-7 in the OpenSSL book */
+
+#define CERT_ACCEPT_NO			0
+#define CERT_ACCEPT_SESSION		1
+#define CERT_ACCEPT_PERMANENT	2
+
+
 gboolean
-libbalsa_is_cert_known(X509* cert, long vfy_result)
+libbalsa_is_cert_known(GTlsCertificate      *cert,
+					   GTlsCertificateFlags  errors)
 {
-    X509 *tmpcert = NULL;
-    FILE *fp;
-    gchar *cert_name;
-    gboolean res = FALSE;
-    GList *lst;
+	gchar *cert_file;
+	GList *cert_db;
+	gboolean cert_ok;
+	GList *lst;
 
-    g_mutex_lock(&certificate_lock);
-    for(lst = accepted_certs; lst; lst = lst->next) {
-        int res = X509_cmp(cert, lst->data);
-        if(res == 0) {
-        	g_mutex_unlock(&certificate_lock);
-            return TRUE;
-	}
-    }
-    
-    cert_name = g_strconcat(g_get_home_dir(), "/.balsa/certificates", NULL);
-
-    fp = fopen(cert_name, "rt");
-    g_free(cert_name);
-    if(fp) {
-        /* 
-        printf("Looking for cert: %s\n", 
-               X509_NAME_oneline(X509_get_subject_name (cert),
-                                 buf, sizeof (buf)));
-        */
-        res = FALSE;
-        while ((tmpcert = PEM_read_X509(fp, NULL, NULL, NULL)) != NULL) {
-            res = X509_cmp(cert, tmpcert)==0;
-            X509_free(tmpcert);
-            if(res) break;
-        }
-        ERR_clear_error();
-        fclose(fp);
-    }
-    g_mutex_unlock(&certificate_lock);
-    
-    if(!res) {
-	const char *reason = X509_verify_cert_error_string(vfy_result);
-	res = libbalsa_ask_for_cert_acceptance(cert, reason);
+	/* check the list of accepted certificates for this session */
 	g_mutex_lock(&certificate_lock);
-	if(res == 2) {
-	    cert_name = g_strconcat(g_get_home_dir(),
-				    "/.balsa/certificates", NULL);
-            libbalsa_assure_balsa_dir();
-	    fp = fopen(cert_name, "a");
-	    if (fp) {
-		if(PEM_write_X509 (fp, cert))
-		    res = TRUE;
-		fclose(fp);
-	    }
-	    g_free(cert_name);
+	for (lst = accepted_certs; lst; lst = lst->next) {
+		if (g_tls_certificate_is_same(cert, G_TLS_CERTIFICATE(lst->data))) {
+			g_mutex_unlock(&certificate_lock);
+			return TRUE;
+		}
 	}
-	if(res == 1)
-	    accepted_certs = 
-		g_list_prepend(accepted_certs, X509_dup(cert));
-	g_mutex_unlock(&certificate_lock);
-    }
 
-    return res;
+	/* check the database of accepted certificates */
+	cert_file = g_build_filename(g_get_home_dir(), ".balsa", "certificates", NULL);
+	cert_db = g_tls_certificate_list_new_from_file(cert_file, NULL);
+	g_mutex_unlock(&certificate_lock);
+
+	cert_ok = FALSE;
+	for (lst = cert_db; !cert_ok && (lst != NULL); lst = g_list_next(lst)) {
+		if (g_tls_certificate_is_same(cert, G_TLS_CERTIFICATE(lst->data))) {
+			cert_ok = TRUE;
+		}
+	}
+	g_list_free_full(cert_db, g_object_unref);
+
+	/* ask the user if the certificate is not in the data base */
+	if (!cert_ok) {
+		int accepted;
+
+		accepted = libbalsa_ask_for_cert_acceptance(cert, errors);
+		g_mutex_lock(&certificate_lock);
+		if (accepted == CERT_ACCEPT_SESSION) {
+			accepted_certs = g_list_prepend(accepted_certs, g_object_ref(cert));
+			cert_ok = TRUE;
+		} else if (accepted == CERT_ACCEPT_PERMANENT) {
+			gchar *pem_data;
+			FILE *fd;
+
+			g_object_get(G_OBJECT(cert), "certificate-pem", &pem_data, NULL);
+			fd = fopen(cert_file, "a");
+			if (fd != NULL) {
+				fputs(pem_data, fd);
+				fclose(fd);
+			}
+			g_free(pem_data);
+			cert_ok = TRUE;
+		} else {
+			/* nothing to do */
+		}
+		g_mutex_unlock(&certificate_lock);
+	}
+	g_free(cert_file);
+
+	return cert_ok;
 }
 
 /* libbalsa_ask_for_cert_acceptance():
@@ -503,7 +402,7 @@ libbalsa_is_cert_known(X509* cert, long vfy_result)
 
 */
 struct AskCertData {
-    X509 *certificate;
+    GTlsCertificate *certificate;
     const char *explanation;
 };
 
@@ -517,8 +416,7 @@ ask_cert_real(void *data)
     GtkWidget *cert_widget;
     GString *str;
     unsigned i;
-    unsigned char *der_buf = NULL;
-    int der_len;
+    GByteArray *cert_der;
     GcrCertificate *gcr_cert;
     GtkWidget *label;
 
@@ -531,9 +429,9 @@ ask_cert_real(void *data)
                                          _("_Reject"), GTK_RESPONSE_CANCEL,
                                          NULL);
     gtk_window_set_role(GTK_WINDOW(dialog), "tls_cert_dialog");
-    der_len = i2d_X509(acd->certificate, &der_buf);
-    gcr_cert = gcr_simple_certificate_new(der_buf, der_len);
-    OPENSSL_free(der_buf);
+    g_object_get(G_OBJECT(acd->certificate), "certificate", &cert_der, NULL);
+    gcr_cert = gcr_simple_certificate_new(cert_der->data, cert_der->len);
+    g_byte_array_unref(cert_der);
     cert_widget = GTK_WIDGET(gcr_certificate_widget_new(gcr_cert));
     g_object_unref(G_OBJECT(gcr_cert));
 
@@ -554,14 +452,14 @@ ask_cert_real(void *data)
 
     switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
     case 0:
-    	i = 1;
+    	i = CERT_ACCEPT_SESSION;
     	break;
     case 1:
-    	i = 2;
+    	i = CERT_ACCEPT_PERMANENT;
     	break;
     case GTK_RESPONSE_CANCEL:
     default:
-    	i = 0;
+    	i = CERT_ACCEPT_NO;
     	break;
     }
     gtk_widget_destroy(dialog);
@@ -570,52 +468,115 @@ ask_cert_real(void *data)
 
 #else
 
+static gnutls_x509_crt_t G_GNUC_WARN_UNUSED_RESULT
+get_gnutls_cert(GTlsCertificate *cert)
+{
+	gnutls_x509_crt_t res_crt;
+    int gnutls_res;
+
+    gnutls_res = gnutls_x509_crt_init(&res_crt);
+    if (gnutls_res == GNUTLS_E_SUCCESS) {
+    	GByteArray *cert_der;
+
+    	g_object_get(G_OBJECT(cert), "certificate", &cert_der, NULL);
+    	if (cert_der != NULL) {
+    		gnutls_datum_t data;
+
+    		data.data = cert_der->data;
+    		data.size = cert_der->len;
+    		gnutls_res = gnutls_x509_crt_import(res_crt, &data, GNUTLS_X509_FMT_DER);
+    		if (gnutls_res != GNUTLS_E_SUCCESS) {
+    			gnutls_x509_crt_deinit(res_crt);
+    			res_crt = NULL;
+    		}
+    		g_byte_array_unref(cert_der);
+    	}
+    } else {
+    	res_crt = NULL;
+    }
+    return res_crt;
+}
+
+static gchar * G_GNUC_WARN_UNUSED_RESULT
+gnutls_get_dn(gnutls_x509_crt_t cert, int (*load_fn)(gnutls_x509_crt_t cert, char *buf, size_t *buf_size))
+{
+    size_t buf_size;
+    gchar *str_buf;
+
+    buf_size = 0U;
+    (void) load_fn(cert, NULL, &buf_size);
+    str_buf = g_malloc0(buf_size + 1U);
+    if (load_fn(cert, str_buf, &buf_size) != GNUTLS_E_SUCCESS) {
+    	g_free(str_buf);
+    	str_buf = NULL;
+    } else {
+    	libbalsa_utf8_sanitize(&str_buf, TRUE, NULL);
+    }
+    return str_buf;
+}
+
+static gchar * G_GNUC_WARN_UNUSED_RESULT
+x509_fingerprint(gnutls_x509_crt_t cert)
+{
+    size_t buf_size;
+    guint8 sha1_buf[20];
+    gchar *str_buf;
+    gint n;
+
+    buf_size = 20U;
+    g_message("%d", gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_SHA1, sha1_buf, &buf_size));
+    str_buf = g_malloc0(61U);
+    for (n = 0; n < 20; n++) {
+    	sprintf(&str_buf[3 * n], "%02x:", sha1_buf[n]);
+    }
+    str_buf[59] = '\0';
+    return str_buf;
+}
+
 static int
 ask_cert_real(void *data)
 {
-    static const char *part[] =
-        {"/CN=", "/Email=", "/O=", "/OU=", "/L=", "/ST=", "/C="};
-
     struct AskCertData *acd = (struct AskCertData*)data;
-    X509 *cert = acd->certificate;
-    char buf[256]; /* fingerprint requires EVP_MAX_MD_SIZE*3 */
-    char *name = NULL, *c, *valid_from, *valid_until;
+    gnutls_x509_crt_t cert;
+    gchar *name, *c, *valid_from, *valid_until;
     GtkWidget* dialog, *label;
     unsigned i;
+    GString* str;
 
-    GString* str = g_string_new("");
+    cert = get_gnutls_cert(acd->certificate);
+    if (cert == NULL) {
+    	g_warning("%s: unable to create gnutls cert", __func__);
+    	return CERT_ACCEPT_NO;
+    }
 
+    str = g_string_new("");
     g_string_printf(str, _("Authenticity of this certificate "
                            "could not be verified.\n"
                            "<b>Reason:</b> %s\n"
                            "<b>This certificate belongs to:</b>\n"),
                     acd->explanation);
 
-    name = X509_NAME_oneline(X509_get_subject_name (cert), buf, sizeof (buf));
-    for (i = 0; i < ELEMENTS(part); i++) {
-        g_string_append(str, x509_get_part (name, part[i]));
-        g_string_append_c(str, '\n');
-    }
+    name = gnutls_get_dn(cert, gnutls_x509_crt_get_dn);
+    g_string_append(str, name);
+    g_free(name);
 
     g_string_append(str, _("\n<b>This certificate was issued by:</b>\n"));
-    name = X509_NAME_oneline(X509_get_issuer_name(cert), buf, sizeof (buf));
-    for (i = 0; i < ELEMENTS(part); i++) {
-        g_string_append(str, x509_get_part (name, part[i]));
-        g_string_append_c(str, '\n');
-    }
+    name = gnutls_get_dn(cert, gnutls_x509_crt_get_issuer_dn);
+    g_string_append_printf(str, "%s\n", name);
+    g_free(name);
 
-    buf[0] = '\0';
-    x509_fingerprint (buf, sizeof (buf), cert);
-    valid_from  = asn1time_to_string(X509_get_notBefore(cert));
-    valid_until = asn1time_to_string(X509_get_notAfter(cert)),
-    c = g_strdup_printf(_("<b>This certificate is valid</b>\n"
-                          "from %s\n"
-                          "to %s\n"
-                          "<b>Fingerprint:</b> %s"),
-                        valid_from, valid_until,
-                        buf);
-    g_string_append(str, c); g_free(c);
-    g_free(valid_from); g_free(valid_until);
+    name = x509_fingerprint(cert);
+    valid_from  = libbalsa_date_to_utf8(gnutls_x509_crt_get_activation_time(cert), "%x %X");
+    valid_until = libbalsa_date_to_utf8(gnutls_x509_crt_get_expiration_time(cert), "%x %X");
+    g_string_append_printf(str, _("<b>This certificate is valid</b>\n"
+    							  "from %s\n"
+    							  "to %s\n"
+                         		  "<b>Fingerprint:</b> %s"),
+                        	valid_from, valid_until, name);
+    g_free(name);
+    g_free(valid_from);
+    g_free(valid_until);
+    gnutls_x509_crt_deinit(cert);
 
     /* This string uses markup, so we must replace "&" with "&amp;" */
     c = str->str;
@@ -645,71 +606,44 @@ ask_cert_real(void *data)
     gtk_widget_show(label);
 
     switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
-    case 0: i = 1; break;
-    case 1: i = 2; break;
+    case 0: i = CERT_ACCEPT_SESSION; break;
+    case 1: i = CERT_ACCEPT_PERMANENT; break;
     case GTK_RESPONSE_CANCEL:
-    default: i=0; break;
+    default: i=CERT_ACCEPT_NO; break;
     }
     gtk_widget_destroy(dialog);
     /* Process some events to let the window disappear:
      * not really necessary but helps with debugging. */
    while(gtk_events_pending()) 
         gtk_main_iteration_do(FALSE);
-    printf("%s returns %d\n", __FUNCTION__, i);
+    g_debug("%s returns %u", __func__, i);
     return i;
 }
 
 #endif	/* HAVE_GCR */
 
 static int
-libbalsa_ask_for_cert_acceptance(X509 *cert, const char *explanation)
+libbalsa_ask_for_cert_acceptance(GTlsCertificate      *cert,
+								 GTlsCertificateFlags  errors)
 {
     struct AskCertData acd;
     acd.certificate = cert;
-    acd.explanation = explanation;
-    return libbalsa_ask(ask_cert_real, &acd);
-}
-
-
-static int
-ask_timeout_real(void *data)
-{
-    const char *host = (const char*)data;
-    GtkWidget* dialog;
-    int i;
-
-    dialog = gtk_message_dialog_new(NULL, /* FIXME: NULL parent */
-                                    GTK_DIALOG_MODAL,
-                                    GTK_MESSAGE_INFO,
-                                    GTK_BUTTONS_YES_NO,
-                                    _("Connection to %s timed out. Abort?"),
-                                    host);
-    gtk_window_set_role(GTK_WINDOW(dialog), "timeout_dialog");
-    switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
-    case GTK_RESPONSE_YES: i = 1; break;
-    case GTK_RESPONSE_NO: i = 0; break;
-    default: printf("Unknown response. Defaulting to 'yes'.\n");
-        i = 1;
+    if ((errors & G_TLS_CERTIFICATE_UNKNOWN_CA) == G_TLS_CERTIFICATE_UNKNOWN_CA) {
+    	acd.explanation = _("the signing certificate authority is not known");
+    } else if ((errors & G_TLS_CERTIFICATE_BAD_IDENTITY) == G_TLS_CERTIFICATE_BAD_IDENTITY) {
+    	acd.explanation = _("the certificate does not match the expected identity of the site that it was retrieved from");
+    } else if ((errors & G_TLS_CERTIFICATE_NOT_ACTIVATED) == G_TLS_CERTIFICATE_NOT_ACTIVATED) {
+    	acd.explanation = _("the certificate’s activation time is still in the future");
+    } else if ((errors & G_TLS_CERTIFICATE_EXPIRED) == G_TLS_CERTIFICATE_EXPIRED) {
+    	acd.explanation = _("the certificate has expired");
+    } else if ((errors & G_TLS_CERTIFICATE_REVOKED) == G_TLS_CERTIFICATE_REVOKED) {
+    	acd.explanation = _("the certificate has been revoked ");
+    } else if ((errors & G_TLS_CERTIFICATE_INSECURE) == G_TLS_CERTIFICATE_INSECURE) {
+    	acd.explanation = _("the certificate’s algorithm is considered insecure");
+    } else {
+    	acd.explanation = _("an error occurred validating the certificate");
     }
-    gtk_widget_destroy(dialog);
-    /* Process some events to let the window disappear:
-     * not really necessary but helps with debugging. */
-   while(gtk_events_pending()) 
-        gtk_main_iteration_do(FALSE);
-    printf("%s returns %d\n", __FUNCTION__, i);
-    return i;
-}
-
-gboolean
-libbalsa_abort_on_timeout(const char *host)
-{  /* It appears not to be entirely thread safe... Some locks do not
-      get released as they should be. */
-    char *hostname;
-
-    hostname = g_alloca (strlen (host) + 1);
-    strcpy (hostname, host);
-    
-    return libbalsa_ask(ask_timeout_real, hostname) != 0; 
+    return libbalsa_ask(ask_cert_real, &acd);
 }
 
 
@@ -952,7 +886,7 @@ libbalsa_source_view_new(gboolean highlight_phrases)
 	    if ((src_lang =
 		 gtk_source_language_manager_get_language(lm, "balsa"))) {
 		GtkSourceStyleSchemeManager *smgr =
-		    gtk_source_style_scheme_manager_new();
+		    gtk_source_style_scheme_manager_get_default();
 		gchar * sm_paths[] = {
 		    BALSA_DATA_PREFIX "/gtksourceview-3.0",
 		    NULL };
@@ -1012,11 +946,10 @@ libbalsa_image_error_quark(void)
     return quark;
 }
 
-#if GTK_CHECK_VERSION(3, 12, 0)
-GtkDialogFlags
-libbalsa_dialog_flags(void)
+gboolean
+libbalsa_use_headerbar(void)
 {
-	static GtkDialogFlags dialog_flags = GTK_DIALOG_USE_HEADER_BAR;
+	static gboolean use_headerbar = TRUE;
 	static gint check_done = 0;
 
 	if (g_atomic_int_get(&check_done) == 0) {
@@ -1024,10 +957,15 @@ libbalsa_dialog_flags(void)
 
 		dialog_env = g_getenv("BALSA_DIALOG_HEADERBAR");
 		if ((dialog_env != NULL) && (atoi(dialog_env) == 0)) {
-			dialog_flags = (GtkDialogFlags) 0;
+			use_headerbar = FALSE;
 		}
 		g_atomic_int_set(&check_done, 1);
 	}
-	return dialog_flags;
+	return use_headerbar;
+}
+
+GtkDialogFlags
+libbalsa_dialog_flags(void)
+{
+	return libbalsa_use_headerbar() ? GTK_DIALOG_USE_HEADER_BAR : (GtkDialogFlags) 0;
 }
-#endif
