@@ -82,6 +82,7 @@ typedef struct _SortTuple SortTuple;
 struct _SortTuple {
     guint offset;
     GNode *node;
+    time_t thread_date;
 };
 
 typedef enum {
@@ -100,7 +101,7 @@ typedef enum {
  * update pref-manager.c so the preferences work correctly */
 typedef enum {
     LB_MAILBOX_THREADING_FLAT,
-    LB_MAILBOX_THREADING_SIMPLE,
+    LB_MAILBOX_THREADING_SIMPLE, /* JWZ without the subject-gather step */
     LB_MAILBOX_THREADING_JWZ
 } LibBalsaMailboxThreadingType;
 
@@ -151,6 +152,7 @@ typedef struct _LibBalsaMailboxView LibBalsaMailboxView;
 struct _LibBalsaMailboxView {
     gchar *identity_name;
     LibBalsaMailboxThreadingType threading_type;
+    gboolean subject_gather;
     /** filter is a frontend-specific code determining used view
      * filter.  GUI usually allows to generate only a subset of all
      * possible LibBalsaCondition's and mapping from arbitary
@@ -182,6 +184,7 @@ struct _LibBalsaMailboxView {
     int unread;
     int total;
     time_t mtime;       /* Mailbox mtime when counts were cached. */
+    int position;       /* Position in the notebook */
 };
 
 struct _LibBalsaMailbox {
@@ -245,6 +248,7 @@ struct _LibBalsaMailbox {
     guint changed_idle_id;
     guint queue_check_idle_id;
     guint need_threading_idle_id;
+    guint run_filters_idle_id;
 };
 
 /* Search iter */
@@ -332,14 +336,14 @@ struct _LibBalsaMailboxClass {
 
 GType libbalsa_mailbox_get_type(void);
 
-LibBalsaMailbox *libbalsa_mailbox_new_from_config(const gchar * prefix);
+LibBalsaMailbox *libbalsa_mailbox_new_from_config(const gchar *prefix,
+												  gboolean     is_special);
 
 /* 
  * open and close a mailbox 
  */
 /* XXX these need to return a value if they failed */
 gboolean libbalsa_mailbox_open(LibBalsaMailbox * mailbox, GError **err);
-gboolean libbalsa_mailbox_is_valid(LibBalsaMailbox * mailbox);
 gboolean libbalsa_mailbox_is_open(LibBalsaMailbox *mailbox);
 void libbalsa_mailbox_close(LibBalsaMailbox * mailbox, gboolean expunge);
 
@@ -516,8 +520,7 @@ gboolean libbalsa_mailbox_can_do(LibBalsaMailbox *mailbox,
     libbalsa_mailbox_set_msg_tree and libbalsa_mailbox_unlink_and_prepend
     are helpers for the subclass methods.
 */
-void libbalsa_mailbox_set_threading(LibBalsaMailbox *mailbox,
-				    LibBalsaMailboxThreadingType thread_type);
+void libbalsa_mailbox_set_threading(LibBalsaMailbox *mailbox);
 void libbalsa_mailbox_set_msg_tree(LibBalsaMailbox * mailbox,
 				   GNode * msg_tree);
 void libbalsa_mailbox_unlink_and_prepend(LibBalsaMailbox * mailbox,
@@ -531,6 +534,8 @@ gboolean libbalsa_mailbox_set_identity_name(LibBalsaMailbox * mailbox,
 void libbalsa_mailbox_set_threading_type(LibBalsaMailbox * mailbox,
 					 LibBalsaMailboxThreadingType
 					 threading_type);
+void libbalsa_mailbox_set_subject_gather(LibBalsaMailbox * mailbox,
+                                         gboolean subject_gather);
 void libbalsa_mailbox_set_sort_type(LibBalsaMailbox * mailbox,
 				    LibBalsaMailboxSortType sort_type);
 void libbalsa_mailbox_set_sort_field(LibBalsaMailbox * mailbox,
@@ -551,10 +556,12 @@ gboolean libbalsa_mailbox_set_crypto_mode(LibBalsaMailbox * mailbox,
 void libbalsa_mailbox_set_unread(LibBalsaMailbox * mailbox, gint unread);
 void libbalsa_mailbox_set_total (LibBalsaMailbox * mailbox, gint total);
 void libbalsa_mailbox_set_mtime (LibBalsaMailbox * mailbox, time_t mtime);
+void libbalsa_mailbox_set_position(LibBalsaMailbox * mailbox, gint position);
 
 const gchar *libbalsa_mailbox_get_identity_name(LibBalsaMailbox * mailbox);
 LibBalsaMailboxThreadingType
 libbalsa_mailbox_get_threading_type(LibBalsaMailbox * mailbox);
+gboolean libbalsa_mailbox_get_subject_gather(LibBalsaMailbox * mailbox);
 LibBalsaMailboxSortType libbalsa_mailbox_get_sort_type(LibBalsaMailbox *
 						       mailbox);
 LibBalsaMailboxSortFields libbalsa_mailbox_get_sort_field(LibBalsaMailbox *
@@ -571,6 +578,7 @@ LibBalsaChkCryptoMode libbalsa_mailbox_get_crypto_mode(LibBalsaMailbox * mailbox
 gint libbalsa_mailbox_get_unread(LibBalsaMailbox * mailbox);
 gint libbalsa_mailbox_get_total (LibBalsaMailbox * mailbox);
 time_t libbalsa_mailbox_get_mtime(LibBalsaMailbox * mailbox);
+gint libbalsa_mailbox_get_position(LibBalsaMailbox * mailbox);
 
 /** force update of given msgno */
 void libbalsa_mailbox_msgno_changed(LibBalsaMailbox  *mailbox, guint seqno);
