@@ -278,26 +278,23 @@ libbalsa_mailbox_local_remove_files(LibBalsaMailboxLocal * local)
 */
 
 static void
-lbml_add_message_to_pool(LibBalsaMailboxLocal * local,
-                         LibBalsaMessage * message)
+lbml_message_pool_take_message(LibBalsaMailboxLocal * local,
+                               LibBalsaMessage * message)
 {
     LibBalsaMailboxLocalPool *item, *oldest;
 
     ++local->pool_seqno;
 
     for (item = oldest = &local->message_pool[0];
-         item < &local->message_pool[LBML_POOL_SIZE]; item++) {
-        if (item->message == message) {
-            item->pool_seqno = local->pool_seqno;
-            return;
-        }
+         item < &local->message_pool[LBML_POOL_SIZE];
+         item++) {
         if (item->pool_seqno < oldest->pool_seqno)
             oldest = item;
     }
 
-    if (oldest->message)
+    if (oldest->message != NULL)
         g_object_unref(oldest->message);
-    oldest->message = g_object_ref(message);
+    oldest->message = message; /* we take over the reference count */
     oldest->pool_seqno = local->pool_seqno;
 }
 
@@ -311,13 +308,13 @@ lbm_local_get_message_with_msg_info(LibBalsaMailboxLocal * local,
 
     msg_info->message = message = libbalsa_message_new();
     g_object_add_weak_pointer(G_OBJECT(message),
-                              (gpointer) & msg_info->message);
+                              (gpointer *) & msg_info->message);
 
     message->flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
     message->mailbox = LIBBALSA_MAILBOX(local);
     message->msgno = msgno;
     libbalsa_message_load_envelope(message);
-    lbml_add_message_to_pool(local, message);
+    lbml_message_pool_take_message(local, message);
 }
 
 static gboolean message_match_real(LibBalsaMailbox * mailbox, guint msgno,
@@ -756,12 +753,10 @@ libbalsa_mailbox_local_close_mailbox(LibBalsaMailbox * mailbox,
                                      mailbox->persistent_view_filter, TRUE);
 
     if (local->thread_id) {
-        /* Rethread immediately. */
-        LibBalsaMailboxThreadingType cur_type =
-            libbalsa_mailbox_get_threading_type(mailbox);
         g_source_remove(local->thread_id);
         local->thread_id = 0;
-        libbalsa_mailbox_set_threading(mailbox, cur_type);
+        /* Rethread immediately. */
+        libbalsa_mailbox_set_threading(mailbox);
     }
 
     if (local->save_tree_id) {
@@ -806,12 +801,10 @@ libbalsa_mailbox_local_get_message(LibBalsaMailbox * mailbox, guint msgno)
     LibBalsaMailboxLocalMessageInfo *msg_info =
         LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info(local, msgno);
 
-    if (msg_info->message)
-        return g_object_ref(msg_info->message);
+    if (msg_info->message == NULL)
+        lbm_local_get_message_with_msg_info(local, msgno, msg_info);
 
-    lbm_local_get_message_with_msg_info(local, msgno, msg_info);
-
-    return msg_info->message;
+    return g_object_ref(msg_info->message);
 }
 
 /* Search iters. We do not use the fallback version because it does
@@ -1102,28 +1095,33 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
  * Threading
  */
 
-static void lbml_threading_jwz(LibBalsaMailbox * mailbox);
-static void lbml_threading_simple(LibBalsaMailbox * mailbox,
-				  LibBalsaMailboxThreadingType th_type);
+static void lbml_thread_messages(LibBalsaMailbox *mailbox, gboolean subject_gather);
+static void lbml_threading_flat(LibBalsaMailbox * mailbox);
 
 void 
 libbalsa_mailbox_local_set_threading_info(LibBalsaMailboxLocal * local)
 {
-    if (!local->threading_info)
-        local->threading_info = g_ptr_array_new();
+    if (local->threading_info == NULL)
+        local->threading_info =
+            g_ptr_array_new_with_free_func((GDestroyNotify) lbm_local_free_info);
 }
 
 static void
 lbml_set_threading(LibBalsaMailbox * mailbox,
                    LibBalsaMailboxThreadingType thread_type)
 {
+    if (mailbox->msg_tree == NULL)
+        return;
+
     switch (thread_type) {
     case LB_MAILBOX_THREADING_JWZ:
-        lbml_threading_jwz(mailbox);
+        lbml_thread_messages(mailbox, TRUE);
+        break;
+    case LB_MAILBOX_THREADING_SIMPLE:
+        lbml_thread_messages(mailbox, FALSE);
         break;
     case LB_MAILBOX_THREADING_FLAT:
-    case LB_MAILBOX_THREADING_SIMPLE:
-        lbml_threading_simple(mailbox, thread_type);
+        lbml_threading_flat(mailbox);
         break;
     }
 }
@@ -1138,8 +1136,9 @@ lbml_set_threading_idle_cb(LbmlSetThreadingInfo * info)
 {
     lbml_set_threading(info->mailbox, info->thread_type);
     g_object_unref(info->mailbox);
-    g_slice_free(LbmlSetThreadingInfo, info);
-    return FALSE;
+    g_free(info);
+
+    return G_SOURCE_REMOVE;
 }
 
 static void
@@ -1187,7 +1186,7 @@ libbalsa_mailbox_local_set_threading(LibBalsaMailbox * mailbox,
     if (libbalsa_am_i_subthread()) {
         LbmlSetThreadingInfo *info;
 
-        info = g_slice_new(LbmlSetThreadingInfo);
+        info = g_new(LbmlSetThreadingInfo, 1);
         info->mailbox = g_object_ref(mailbox);
         info->thread_type = thread_type;
         g_idle_add((GSourceFunc) lbml_set_threading_idle_cb, info);
@@ -1209,11 +1208,9 @@ libbalsa_mailbox_local_msgno_removed(LibBalsaMailbox * mailbox,
 
     /* local might not have a threading-info array, and even if it does,
      * it might not be populated; we check both. */
-    if (local->threading_info && msgno <= local->threading_info->len) {
-	lbm_local_free_info(g_ptr_array_index(local->threading_info,
-			                      msgno - 1));
+    if (local->threading_info != NULL &&
+        msgno > 0 && msgno <= local->threading_info->len)
 	g_ptr_array_remove_index(local->threading_info, msgno - 1);
-    }
 
     libbalsa_mailbox_msgno_removed(mailbox, msgno);
 }
@@ -1291,12 +1288,8 @@ lbm_local_thread_idle(LibBalsaMailboxLocal * local)
 
     libbalsa_lock_mailbox(mailbox);
 
-    if (MAILBOX_OPEN(mailbox)) {
-        LibBalsaMailboxThreadingType cur_type =
-            libbalsa_mailbox_get_threading_type(mailbox);
-
-        libbalsa_mailbox_set_threading(mailbox, cur_type);
-    }
+    if (MAILBOX_OPEN(mailbox))
+        libbalsa_mailbox_set_threading(mailbox);
     local->thread_id = 0;
 
     libbalsa_unlock_mailbox(mailbox);
@@ -1421,12 +1414,11 @@ libbalsa_mailbox_local_get_msg_part(LibBalsaMessage *msg,
 /*  Start of threading functions  */
 /*--------------------------------*/
 /*
- * This code includes two message threading functions.
- * The first is the implementation of jwz's algorithm describled at
- * http://www.jwz.org/doc/threading.html . The another is very simple and 
- * trivial one. If you confirm that your mailbox includes every threaded 
- * messages, the later will be enough. Those functions are selectable on
- * each mailbox by setting the 'type' member in BalsaIndex. If you don't need
+ * Threading is implementated using jwz's algorithm describled at
+ * http://www.jwz.org/doc/threading.html. It is implemented with an option
+ * to avoid the "subject gather" step, which is needed only if some
+ * messages do not have a valid and complete "References" header, and
+ * can otherwise be annoying. If you don't need
  * message threading functionality, just specify 'LB_MAILBOX_THREADING_FLAT'. 
  *
  * ymnk@jcraft.com
@@ -1475,7 +1467,7 @@ lbml_info_free(ThreadingInfo * ti)
 }
 
 static void
-lbml_threading_jwz(LibBalsaMailbox * mailbox)
+lbml_thread_messages(LibBalsaMailbox * mailbox, gboolean subject_gather)
 {
     /* This implementation of JWZ's algorithm uses a second tree, rooted
      * at ti.root, for the message IDs.  Each node in the second tree
@@ -1495,12 +1487,14 @@ lbml_threading_jwz(LibBalsaMailbox * mailbox)
     g_node_traverse(ti.root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 		    (GNodeTraverseFunc) lbml_prune, &ti);
 
-    /* Do the evil subject gather and merge on the second tree. */
-    ti.subject_table = g_hash_table_new(g_str_hash, g_str_equal);
-    g_node_children_foreach(ti.root, G_TRAVERSE_ALL,
-			    (GNodeForeachFunc) lbml_subject_gather, &ti);
-    g_node_children_foreach(ti.root, G_TRAVERSE_ALL,
-			    (GNodeForeachFunc) lbml_subject_merge, &ti);
+    if (subject_gather) {
+        /* Do the evil subject gather and merge on the second tree. */
+        ti.subject_table = g_hash_table_new(g_str_hash, g_str_equal);
+        g_node_children_foreach(ti.root, G_TRAVERSE_ALL,
+                                (GNodeForeachFunc) lbml_subject_gather, &ti);
+        g_node_children_foreach(ti.root, G_TRAVERSE_ALL,
+                                (GNodeForeachFunc) lbml_subject_merge, &ti);
+    }
 
     /* Traverse the second tree and reparent corresponding nodes in the
      * mailbox's msg_tree. */
@@ -1991,86 +1985,26 @@ lbml_clear_empty(GNode * msg_tree)
 }
 #endif				/* MAKE_EMPTY_CONTAINER_FOR_MISSING_PARENT */
 
-/* yet another message threading function */
+/*------------------------------*/
+/*       Flat threading         */
+/*------------------------------*/
 
-static gboolean lbml_insert_message(GNode * node, ThreadingInfo * ti);
-static gboolean lbml_thread_message(GNode * node, ThreadingInfo * ti);
+static gboolean
+lbml_unthread_message(GNode * node, LibBalsaMailbox * mailbox)
+{
+    if (node->parent != NULL && node->parent != mailbox->msg_tree)
+        libbalsa_mailbox_unlink_and_prepend(mailbox, node, mailbox->msg_tree);
+
+    return FALSE;
+}
 
 static void
-lbml_threading_simple(LibBalsaMailbox * mailbox,
-		      LibBalsaMailboxThreadingType type)
+lbml_threading_flat(LibBalsaMailbox * mailbox)
 {
-    GNode *msg_tree = mailbox->msg_tree;
-    ThreadingInfo ti;
-
-    lbml_info_setup(mailbox, &ti);
-
-    if (type == LB_MAILBOX_THREADING_SIMPLE)
-	g_node_traverse(msg_tree, G_POST_ORDER, G_TRAVERSE_ALL,
-			-1, (GNodeTraverseFunc) lbml_insert_message, &ti);
-
-    ti.type = type;
-    g_node_traverse(msg_tree, G_POST_ORDER, G_TRAVERSE_ALL, -1,
-		    (GNodeTraverseFunc) lbml_thread_message, &ti);
-
-#ifdef MAKE_EMPTY_CONTAINER_FOR_MISSING_PARENT
-    lbml_clear_empty(msg_tree);
-#endif				/* MAKE_EMPTY_CONTAINER_FOR_MISSING_PARENT */
-
-    lbml_info_free(&ti);
+    g_node_traverse(mailbox->msg_tree, G_POST_ORDER, G_TRAVERSE_ALL, -1,
+		    (GNodeTraverseFunc) lbml_unthread_message, mailbox);
 }
 
-static gboolean
-lbml_insert_message(GNode * node, ThreadingInfo * ti)
-{
-    LibBalsaMailboxLocalInfo *info;
-
-    if (!node->parent)
-	return FALSE;
-
-    info = lbml_get_info(node, ti);
-    if (!info)
-	return FALSE;
-
-    if (info->message_id)
-	g_hash_table_insert(ti->id_table, info->message_id, node);
-
-    return FALSE;
-}
-
-static gboolean
-lbml_thread_message(GNode * node, ThreadingInfo * ti)
-{
-    if (!node->parent)
-        return FALSE;
-
-    if (ti->type == LB_MAILBOX_THREADING_FLAT) {
-        if (node->parent != ti->mailbox->msg_tree)
-            libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node,
-                                                ti->mailbox->msg_tree);
-    } else {
-        LibBalsaMailboxLocalInfo *info;
-        GList *refs;
-        GNode *parent = NULL;
-
-        info = lbml_get_info(node, ti);
-        if (!info)
-            return FALSE;
-
-        refs = info->refs_for_threading;
-        if (refs)
-            parent = g_hash_table_lookup(ti->id_table,
-                                         g_list_last(refs)->data);
-
-        if (!parent)
-            parent = ti->mailbox->msg_tree;
-        if (parent != node->parent && parent != node
-            && !g_node_is_ancestor(node, parent))
-            libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node, parent);
-    }
-
-    return FALSE;
-}
 /*------------------------------*/
 /*  End of threading functions  */
 /*------------------------------*/
@@ -2136,13 +2070,11 @@ lbm_local_sync_real(LibBalsaMailboxLocal * local)
 
     time(&tstart);
     libbalsa_lock_mailbox(mailbox);
-    if (local->sync_id &&                       /* request still pending */
-        MAILBOX_OPEN(mailbox) &&                   /* mailbox still open */
+    if (MAILBOX_OPEN(mailbox) &&                   /* mailbox still open */
         !libbalsa_mailbox_sync_storage(mailbox, FALSE))   /* cannot sync */
 	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
 			     _("Failed to sync mailbox “%s”"),
 			     mailbox->name);
-    local->sync_id = 0;
     local->sync_time += time(NULL)-tstart;
     local->sync_cnt++;
     libbalsa_unlock_mailbox(mailbox);
@@ -2154,11 +2086,12 @@ lbm_local_sync_idle(LibBalsaMailboxLocal * local)
 {
     GThread *sync_thread;
 
+    local->sync_id = 0;
     sync_thread =
     	g_thread_new("lbm_local_sync_real", (GThreadFunc) lbm_local_sync_real, local);
     g_thread_unref(sync_thread);
 
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 static void
