@@ -126,7 +126,7 @@ libbalsa_mailbox_pop3_init(LibBalsaMailboxPop3 * mailbox)
     mailbox->msg_size_limit = -1;
 
     mailbox->filter = FALSE;
-    mailbox->filter_cmd = NULL;
+    mailbox->filter_cmd = g_strdup("procmail -f -");
     remote = LIBBALSA_MAILBOX_REMOTE(mailbox);
     remote->server = libbalsa_server_new();
 }
@@ -135,8 +135,10 @@ static void
 libbalsa_mailbox_pop3_finalize(GObject * object)
 {
     LibBalsaMailboxRemote *remote = LIBBALSA_MAILBOX_REMOTE(object);
+    LibBalsaMailboxPop3 *mailbox = LIBBALSA_MAILBOX_POP3(object);
 
     g_object_unref(G_OBJECT(remote->server));
+    g_free(mailbox->filter_cmd);
 
     if (G_OBJECT_CLASS(parent_class)->finalize)
 	G_OBJECT_CLASS(parent_class)->finalize(object);
@@ -494,6 +496,10 @@ libbalsa_mailbox_pop3_startup(LibBalsaServer            *server,
 		if (!net_client_set_cert_from_file(NET_CLIENT(pop), server->cert_file, &error)) {
 			libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("Cannot load certificate file %s: %s"), server->cert_file,
 				error->message);
+			/* bad certificate private key password: clear it */
+			if (error->code == NET_CLIENT_ERROR_CERT_KEY_PASS) {
+				libbalsa_server_set_password(server, NULL, TRUE);
+			}
 			g_error_free(error);
 			g_object_unref(G_OBJECT(pop));
 			return NULL;
@@ -509,6 +515,10 @@ libbalsa_mailbox_pop3_startup(LibBalsaServer            *server,
 	if (!net_client_pop_connect(pop, NULL, &error)) {
 		libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("POP3 mailbox %s: cannot connect %s: %s"), name, server->host,
 			error->message);
+		/* authentication failed: clear password */
+		if (error->code == NET_CLIENT_ERROR_POP_AUTHFAIL) {
+			libbalsa_server_set_password(server, NULL, FALSE);
+		}
 		g_error_free(error);
 		net_client_shutdown(NET_CLIENT(pop));
 		g_object_unref(G_OBJECT(pop));
@@ -742,6 +752,7 @@ libbalsa_mailbox_pop3_load_config(LibBalsaMailbox * mailbox,
     pop->disable_apop = libbalsa_conf_get_bool("DisableApop=false");
     pop->enable_pipe = libbalsa_conf_get_bool("EnablePipe=false");
     pop->filter = libbalsa_conf_get_bool("Filter=false");
+    g_free(pop->filter_cmd);
     pop->filter_cmd = libbalsa_conf_get_string("FilterCmd");
     if(pop->filter_cmd && *pop->filter_cmd == '\0') {
 	g_free(pop->filter_cmd); pop->filter_cmd = NULL;
