@@ -25,21 +25,18 @@
 
 #include <string.h>
 #include <gpgme.h>
-#include <pwd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdlib.h>
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
+#include "libbalsa-gpgme-widgets.h"
+#include "libbalsa-gpgme-keys.h"
+#include "libbalsa-gpgme.h"
 
 #include "gmime-multipart-crypt.h"
 #include "gmime-gpgme-signature.h"
 #include "gmime-part-rfc2440.h"
 
-#ifdef HAVE_SMIME
-#  include "gmime-application-pkcs7.h"
-#endif
+#include "gmime-application-pkcs7.h"
 
 #if HAVE_MACOSX_DESKTOP
 #  include "macosx-helpers.h"
@@ -48,6 +45,12 @@
 #include <glib/gi18n.h>
 
 
+#ifdef G_LOG_DOMAIN
+#  undef G_LOG_DOMAIN
+#endif
+#define G_LOG_DOMAIN "crypto"
+
+
 /* local prototypes */
 static gboolean gpg_updates_trustdb(void);
 static gboolean have_pub_key_for(gpgme_ctx_t gpgme_ctx,
@@ -88,27 +91,21 @@ libbalsa_can_encrypt_for_all(InternetAddressList * recipients,
     /* silent paranoia checks */
     if (!recipients)
 	return TRUE;  /* we can of course encrypt for nobody... */
-#ifndef HAVE_SMIME
-    if (protocol == GPGME_PROTOCOL_OpenPGP)
-	return FALSE;
-#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
 	return FALSE;
 
     /* create the gpgme context and set the protocol */
-    if (gpgme_new(&gpgme_ctx) != GPG_ERR_NO_ERROR)
-	return FALSE;
-    if (gpgme_set_protocol(gpgme_ctx, protocol) != GPG_ERR_NO_ERROR) {
-	gpgme_release(gpgme_ctx);
-	return FALSE;
+    gpgme_ctx = libbalsa_gpgme_new_with_proto(protocol, NULL, NULL, NULL);
+    if (gpgme_ctx == NULL) {
+    	result = FALSE;
+    } else {
+    	/* loop over all recipients and try to find valid keys */
+    	result = have_pub_key_for(gpgme_ctx, recipients);
+    	gpgme_release(gpgme_ctx);
     }
 
-    /* loop over all recipients and try to find valid keys */
-    result = have_pub_key_for(gpgme_ctx, recipients);
-    gpgme_release(gpgme_ctx);
-
     return result;
 }
 
@@ -208,9 +205,6 @@ libbalsa_sign_mime_object(GMimeObject ** content, const gchar * rfc822_for,
     /* paranoia checks */
     g_return_val_if_fail(rfc822_for != NULL, FALSE);
     g_return_val_if_fail(content != NULL, FALSE);
-#ifndef HAVE_SMIME
-    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
-#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -249,9 +243,6 @@ libbalsa_encrypt_mime_object(GMimeObject ** content, GList * rfc822_for,
     /* paranoia checks */
     g_return_val_if_fail(rfc822_for != NULL, FALSE);
     g_return_val_if_fail(content != NULL, FALSE);
-#ifndef HAVE_SMIME
-    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
-#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -271,17 +262,13 @@ libbalsa_encrypt_mime_object(GMimeObject ** content, GList * rfc822_for,
 
 	encrypted_obj = GMIME_OBJECT(mpe);
 	result = g_mime_gpgme_mpe_encrypt(mpe, *content, recipients, always_trust, parent, error);
-    }
-#ifdef HAVE_SMIME
-    else {
-	GMimePart *pkcs7 =
-	    g_mime_part_new_with_type("application", "pkcs7-mime");
-	encrypted_obj = GMIME_OBJECT(pkcs7);
+    } else {
+    	GMimePart *pkcs7 = g_mime_part_new_with_type("application", "pkcs7-mime");
+    	encrypted_obj = GMIME_OBJECT(pkcs7);
 
-	result = g_mime_application_pkcs7_encrypt(pkcs7, *content, recipients, always_trust, parent, error);
+    	result = g_mime_application_pkcs7_encrypt(pkcs7, *content, recipients, always_trust, parent, error);
     }
-#endif
-    g_ptr_array_free(recipients, FALSE);
+    g_ptr_array_unref(recipients);
 
     /* error checking */
     if (!result) {
@@ -315,9 +302,6 @@ libbalsa_sign_encrypt_mime_object(GMimeObject ** content,
     g_return_val_if_fail(rfc822_signer != NULL, FALSE);
     g_return_val_if_fail(rfc822_for != NULL, FALSE);
     g_return_val_if_fail(content != NULL, FALSE);
-#ifndef HAVE_SMIME
-    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
-#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -363,9 +347,6 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
     g_return_val_if_fail(body, FALSE);
     g_return_val_if_fail(body->mime_part != NULL, FALSE);
     g_return_val_if_fail(body->message, FALSE);
-#ifndef HAVE_SMIME
-    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
-#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -411,17 +392,12 @@ libbalsa_body_decrypt(LibBalsaMessageBody *body, gpgme_protocol_t protocol, GtkW
     GError *error = NULL;
     LibBalsaMessage *message;
     GMimeGpgmeSigstat *sig_state = NULL;
-#ifdef HAVE_SMIME
     gboolean smime_encrypted = FALSE;
-#endif
 
     /* paranoia checks */
     g_return_val_if_fail(body != NULL, body);
     g_return_val_if_fail(body->mime_part != NULL, body);
     g_return_val_if_fail(body->message != NULL, body);
-#ifndef HAVE_SMIME
-    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
-#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -431,33 +407,29 @@ libbalsa_body_decrypt(LibBalsaMessageBody *body, gpgme_protocol_t protocol, GtkW
     if (protocol == GPGME_PROTOCOL_OpenPGP) {
 	if (!GMIME_IS_MULTIPART_ENCRYPTED(body->mime_part))
 	    return body;
-    }
-#ifdef HAVE_SMIME
-    else {
-	const char * smime_type = 
-	    g_mime_object_get_content_type_parameter(body->mime_part,
-						     "smime-type");
+    } else {
+    	const char * smime_type =
+    		g_mime_object_get_content_type_parameter(body->mime_part,
+    			"smime-type");
 
-	if (!smime_type || !GMIME_IS_PART(body->mime_part))
-	    return body;
-	if (!g_ascii_strcasecmp(smime_type, "enveloped-data"))
-	    smime_encrypted = TRUE;
-	else
-            smime_encrypted = body->was_encrypted;
+    	if (!smime_type || !GMIME_IS_PART(body->mime_part))
+    		return body;
+    	if (!g_ascii_strcasecmp(smime_type, "enveloped-data"))
+    		smime_encrypted = TRUE;
+    	else
+    		smime_encrypted = body->was_encrypted;
     }
-#endif
 
     libbalsa_mailbox_lock_store(body->message->mailbox);
-    if (protocol == GPGME_PROTOCOL_OpenPGP)
-	mime_obj =
-	    g_mime_gpgme_mpe_decrypt(GMIME_MULTIPART_ENCRYPTED(body->mime_part),
-				     &sig_state, parent, &error);
-#ifdef HAVE_SMIME
-    else
-	mime_obj =
-	    g_mime_application_pkcs7_decrypt_verify(GMIME_PART(body->mime_part),
-						    &sig_state, parent, &error);
-#endif
+    if (protocol == GPGME_PROTOCOL_OpenPGP) {
+    	mime_obj =
+    		g_mime_gpgme_mpe_decrypt(GMIME_MULTIPART_ENCRYPTED(body->mime_part),
+    			&sig_state, parent, &error);
+    } else {
+    	mime_obj =
+    		g_mime_application_pkcs7_decrypt_verify(GMIME_PART(body->mime_part),
+    			&sig_state, parent, &error);
+    }
     libbalsa_mailbox_unlock_store(body->message->mailbox);
 
     /* check the result */
@@ -478,12 +450,11 @@ libbalsa_body_decrypt(LibBalsaMessageBody *body, gpgme_protocol_t protocol, GtkW
     body = libbalsa_message_body_new(message);
 
     /* remember that is was encrypted */
-    if (protocol == GPGME_PROTOCOL_OpenPGP)
-	body->was_encrypted = TRUE;
-#ifdef HAVE_SMIME
-    else
-	body->was_encrypted = smime_encrypted;
-#endif
+    if (protocol == GPGME_PROTOCOL_OpenPGP) {
+    	body->was_encrypted = TRUE;
+    } else {
+    	body->was_encrypted = smime_encrypted;
+    }
     if (body->was_encrypted)
         body->message->prot_state = LIBBALSA_MSG_PROTECT_CRYPT;
 
@@ -532,7 +503,7 @@ libbalsa_rfc2440_sign_encrypt(GMimePart *part, const gchar *sign_for,
 					      always_trust, parent, error);
     /* clean up */
     if (recipients)
-	g_ptr_array_free(recipients, FALSE);
+	g_ptr_array_unref(recipients);
     return result;
 }
 
@@ -649,42 +620,41 @@ libbalsa_rfc2440_decrypt(GMimePart * part, GMimeGpgmeSigstat ** sig_info,
 
 
 /* conversion of status values to human-readable messages */
-const gchar *
+gchar *
 libbalsa_gpgme_sig_stat_to_gchar(gpgme_error_t stat)
 {
-    switch (stat) {
-    case GPG_ERR_NO_ERROR:
-	return _("The signature is valid.");
-    case GPG_ERR_SIG_EXPIRED:
-	return _("The signature is valid but expired.");
-    case GPG_ERR_KEY_EXPIRED:
-	return _
-	    ("The signature is valid but the key used to verify the signature has expired.");
-    case GPG_ERR_CERT_REVOKED:
-	return _
-	    ("The signature is valid but the key used to verify the signature has been revoked.");
-    case GPG_ERR_BAD_SIGNATURE:
-	return _
-	    ("The signature is invalid.");
-    case GPG_ERR_NO_PUBKEY:
-	return
-	    _("The signature could not be verified due to a missing key.");
-    case GPG_ERR_NO_DATA:
-	return _("This part is not a real PGP signature.");
-    case GPG_ERR_INV_ENGINE:
-	return _
-	    ("The signature could not be verified due to an invalid crypto engine.");
-    case GPG_ERR_TRY_AGAIN:
-	return _
-	    ("GnuPG is rebuilding the trust database and is currently unavailable.");
-    default: {
-	gchar errbuf[4096];		/* should be large enough... */
+	switch (stat) {
+	case GPG_ERR_NO_ERROR:
+		return g_strdup(_("The signature is valid."));
+	case GPG_ERR_SIG_EXPIRED:
+		return g_strdup(_("The signature is valid but expired."));
+	case GPG_ERR_KEY_EXPIRED:
+		return g_strdup(_("The signature is valid but the key used to verify the signature has expired."));
+	case GPG_ERR_CERT_REVOKED:
+		return g_strdup(_("The signature is valid but the key used to verify the signature has been revoked."));
+	case GPG_ERR_BAD_SIGNATURE:
+		return g_strdup(_("The signature is invalid."));
+	case GPG_ERR_NO_PUBKEY:
+		return g_strdup(_("The signature could not be verified due to a missing key."));
+	case GPG_ERR_NO_DATA:
+		return g_strdup(_("This part is not a real signature."));
+	case GPG_ERR_INV_ENGINE:
+		return g_strdup(_("The signature could not be verified due to an invalid crypto engine."));
+	case GPG_ERR_TRY_AGAIN:
+		return g_strdup(_("GnuPG is rebuilding the trust database and is currently unavailable."));
+	case GPG_ERR_MULT_SIGNATURES:
+		return g_strdup(_("The signature contains multiple signers, this may be a forgery."));
+	default: {
+		gchar errbuf[4096];		/* should be large enough... */
 
-	gpgme_strerror_r(stat, errbuf, sizeof(errbuf));
-	g_message("stat %d: %s %s", stat, gpgme_strsource(stat), errbuf);
-	return _("An error prevented the signature verification.");
-    }
-    }
+		gpgme_strerror_r(stat, errbuf, sizeof(errbuf));
+		if (gpgme_err_source(stat) != GPG_ERR_SOURCE_UNKNOWN) {
+			return g_strdup_printf(_("An error prevented the signature verification: %s: %s"), gpgme_strsource(stat), errbuf);
+		} else {
+			return g_strdup_printf(_("An error prevented the signature verification: %s"), errbuf);
+		}
+	}
+	}
 }
 
 
@@ -732,295 +702,6 @@ libbalsa_gpgme_validity_to_gchar_short(gpgme_validity_t validity)
 }
 
 
-const gchar *
-libbalsa_gpgme_sig_protocol_name(gpgme_protocol_t protocol)
-{
-    switch (protocol) {
-    case GPGME_PROTOCOL_OpenPGP:
-	return _("PGP signature: ");
-    case GPGME_PROTOCOL_CMS:
-	return _("S/MIME signature: ");
-    default:
-	return _("(unknown protocol) ");
-    }
-}
-
-static inline void
-append_time_t(GString *str, const gchar *format, time_t when,
-              const gchar * date_string)
-{
-    if (when != (time_t) 0) {
-        gchar *tbuf = libbalsa_date_to_utf8(when, date_string);
-        g_string_append_printf(str, format, tbuf);
-        g_free(tbuf);
-    } else {
-        g_string_append_printf(str, format, _("never"));
-    }
-}
-
-gchar *
-libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
-				 const gchar * date_string)
-{
-    GString *msg;
-    gchar *retval;
-
-    g_return_val_if_fail(info != NULL, NULL);
-    g_return_val_if_fail(date_string != NULL, NULL);
-    msg = g_string_new(libbalsa_gpgme_sig_protocol_name(info->protocol));
-    msg =
-	g_string_append(msg,
-			libbalsa_gpgme_sig_stat_to_gchar(info->status));
-    g_string_append_printf(msg, _("\nSignature validity: %s"),
-			   libbalsa_gpgme_validity_to_gchar(info->
-							    validity));
-    append_time_t(msg, _("\nSigned on: %s"), info->sign_time, date_string);
-    if (info->protocol == GPGME_PROTOCOL_OpenPGP && info->key)
-	g_string_append_printf(msg, _("\nKey owner trust: %s"),
-			       libbalsa_gpgme_validity_to_gchar_short
-			       (info->key->owner_trust));
-    if (info->fingerprint)
-	g_string_append_printf(msg, _("\nKey fingerprint: %s"),
-			       info->fingerprint);
-
-    /* add key information */
-    if (info->key) {
-        gpgme_user_id_t uid;
-        gpgme_subkey_t subkey;
-
-        /* user ID's */
-        if ((uid = info->key->uids)) {
-            gchar *lead_text;
-
-            uid = info->key->uids;
-            if (uid->next) {
-        	msg = g_string_append(msg, _("\nUser IDs:"));
-        	lead_text = "\n\342\200\242";
-            } else {
-        	msg = g_string_append(msg, _("\nUser ID:"));
-        	lead_text = "";
-            }
-
-            /* Note: there is no way to determine which user id has been used
-             * to create the signature.  A broken client may even use an
-             * invalid and/or revoked one.  We therefore add all to the
-             * result. */
-            while (uid) {
-        	msg = g_string_append(msg, lead_text);
-        	if (uid->revoked)
-        	    msg = g_string_append(msg, _(" [Revoked]"));
-        	if (uid->invalid)
-        	    msg = g_string_append(msg, _(" [Invalid]"));
-
-        	if (uid->uid && *(uid->uid)) {
-        	    gchar *uid_readable =
-        	    	libbalsa_cert_subject_readable(uid->uid);
-        	    g_string_append_printf(msg, " %s", uid_readable);
-        	    g_free(uid_readable);
-        	} else {
-        	    if (uid->name && *(uid->name))
-        		g_string_append_printf(msg, " %s", uid->name);
-        	    if (uid->email && *(uid->email))
-        		g_string_append_printf(msg, " <%s>", uid->email);
-        	    if (uid->comment && *(uid->comment))
-        		g_string_append_printf(msg, " (%s)", uid->comment);
-        	}
-
-        	uid = uid->next;
-            }
-        }
-
-        /* subkey */
-        if ((subkey = info->key->subkeys)) {
-            /* find the one which can sign */
-            while (subkey && !subkey->can_sign)
-        	subkey = subkey->next;
-
-            if (subkey) {
-        	append_time_t(msg, _("\nSubkey created on: %s"),
-        		      subkey->timestamp, date_string);
-        	append_time_t(msg, _("\nSubkey expires on: %s"),
-        		      subkey->expires, date_string);
-        	if (subkey->revoked || subkey->expired || subkey->disabled ||
-        	    subkey->invalid) {
-       GString * attrs = g_string_new("");
-       int count = 0;
-
-        	    if (subkey->revoked) {
-           count++;
-           attrs = g_string_append(attrs, _(" revoked"));
-       }
-        	    if (subkey->expired) {
-           if (count++)
-               attrs = g_string_append_c(attrs, ',');
-           attrs = g_string_append(attrs, _(" expired"));
-       }
-        	    if (subkey->disabled) {
-           if (count)
-               attrs = g_string_append_c(attrs, ',');
-           attrs = g_string_append(attrs, _(" disabled"));
-       }
-        	    if (subkey->invalid) {
-           if (count)
-               attrs = g_string_append_c(attrs, ',');
-           attrs = g_string_append(attrs, _(" invalid"));
-       }
-        	    /* ngettext: string begins with a single space, so no space
-        	     * after the colon is correct punctuation (in English). */
-       g_string_append_printf(msg, ngettext("\nSubkey attribute:%s",
-                                            "\nSubkey attributes:%s",
-                                            count),
-                              attrs->str);
-       g_string_free(attrs, TRUE);
-    }
-            }
-        }
-
-        if (info->key->issuer_name) {
-            gchar *issuer_name =
-        	libbalsa_cert_subject_readable(info->key->issuer_name);
-            g_string_append_printf(msg, _("\nIssuer name: %s"), issuer_name);
-            g_free(issuer_name);
-    }
-        if (info->key->issuer_serial)
-	g_string_append_printf(msg, _("\nIssuer serial number: %s"),
-				   info->key->issuer_serial);
-        if (info->key->chain_id)
-            g_string_append_printf(msg, _("\nChain ID: %s"), info->key->chain_id);
-    }
-
-    retval = msg->str;
-    g_string_free(msg, FALSE);
-    return retval;
-}
-
-
-#ifdef HAVE_GPG
-
-#include <sys/wait.h>
-#include <fcntl.h>
-
-/* run gpg asynchronously to import or update a key */
-typedef struct _spawned_gpg_T {
-    gint child_pid;
-    gint standard_error;
-    GString *stderr_buf;
-    GtkWindow *parent;
-} spawned_gpg_T;
-
-static gboolean check_gpg_child(gpointer data);
-
-gboolean
-gpg_keyserver_op(const gchar * fingerprint, gpg_keyserver_action_t action,
-                 GtkWindow * parent)
-{
-    gchar **argv;
-    spawned_gpg_T *spawned_gpg;
-    gboolean spawnres;
-
-    /* launch gpg... */
-    argv = g_new(gchar *, 5);
-    argv[0] = g_strdup(GPG_PATH);
-    argv[1] = g_strdup("--no-greeting");
-    switch (action) {
-    case GPG_KEYSERVER_IMPORT:
-        argv[2] = g_strdup("--recv-keys");
-        break;
-    case GPG_KEYSERVER_UPDATE:
-        argv[2] = g_strdup("--refresh-keys");
-        break;
-    default:
-        g_assert_not_reached();
-    }
-    argv[3] = g_strdup(fingerprint);
-    argv[4] = NULL;
-    spawned_gpg = g_new0(spawned_gpg_T, 1);
-    spawnres =
-	g_spawn_async_with_pipes(NULL, argv, NULL,
-				 G_SPAWN_DO_NOT_REAP_CHILD |
-				 G_SPAWN_STDOUT_TO_DEV_NULL, NULL, NULL,
-				 &spawned_gpg->child_pid, NULL, NULL,
-				 &spawned_gpg->standard_error, NULL);
-    g_strfreev(argv);
-    if (spawnres == FALSE) {
-	libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-			     _
-			     ("Could not launch %s to query the public key %s."),
-			     GPG_PATH, fingerprint);
-	g_free(spawned_gpg);
-	return FALSE;
-    }
-
-    /* install an idle handler to check if the child returnd successfully. */
-    fcntl(spawned_gpg->standard_error, F_SETFL, O_NONBLOCK);
-    spawned_gpg->stderr_buf = g_string_new("");
-    spawned_gpg->parent = parent;
-    g_timeout_add(250, check_gpg_child, spawned_gpg);
-
-    return TRUE;
-}
-
-
-static gboolean
-check_gpg_child(gpointer data)
-{
-    spawned_gpg_T *spawned_gpg = (spawned_gpg_T *) data;
-    int status;
-    ssize_t bytes_read;
-    gchar buffer[1024], *gpg_message;
-    GtkWidget *dialog;
-
-    /* read input from the child and append it to the buffer */
-    while ((bytes_read =
-	    read(spawned_gpg->standard_error, buffer, 1023)) > 0) {
-	buffer[bytes_read] = '\0';
-	g_string_append(spawned_gpg->stderr_buf, buffer);
-    }
-
-    /* check if the child exited */
-    if (waitpid(spawned_gpg->child_pid, &status, WNOHANG) !=
-	spawned_gpg->child_pid)
-	return TRUE;
-
-    /* child exited, display some information... */
-    close(spawned_gpg->standard_error);
-
-    gpg_message =
-	g_locale_to_utf8(spawned_gpg->stderr_buf->str, -1, NULL,
-			 NULL, NULL);
-    gdk_threads_enter();
-    if (WEXITSTATUS(status) > 0)
-	dialog =
-	    gtk_message_dialog_new(spawned_gpg->parent,
-				   GTK_DIALOG_DESTROY_WITH_PARENT,
-				   GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
-				   _
-				   ("Running %s failed with return value %d:\n%s"),
-				   GPG_PATH, WEXITSTATUS(status), gpg_message);
-    else
-	dialog =
-	    gtk_message_dialog_new(spawned_gpg->parent,
-				   GTK_DIALOG_DESTROY_WITH_PARENT,
-				   GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
-				   _("Running %s successful:\n%s"),
-				   GPG_PATH, gpg_message);
-#if HAVE_MACOSX_DESKTOP
-    libbalsa_macosx_menu_for_parent(dialog, spawned_gpg->parent);
-#endif
-    g_free(gpg_message);
-    g_string_free(spawned_gpg->stderr_buf, TRUE);
-    g_free(spawned_gpg);
-
-    gtk_dialog_run(GTK_DIALOG(dialog));
-    gtk_widget_destroy(dialog);
-    gdk_threads_leave();
-
-    return FALSE;
-}
-
-#endif				/* HAVE_GPG */
-
-
 /* ==== local stuff ======================================================== */
 
 
@@ -1031,71 +712,54 @@ check_gpg_child(gpointer data)
 static gboolean
 gpg_updates_trustdb(void)
 {
-    static gchar *lockname = NULL;
-    struct passwd *pwent;
-    struct stat stat_buf;
+	static gchar *lockname = NULL;
+	gboolean result;
 
-    if (!lockname)
-	if ((pwent = getpwuid(getuid())))
-	    lockname =
-		g_strdup_printf("%s/.gnupg/trustdb.gpg.lock",
-				pwent->pw_dir);
-    if (!stat(lockname, &stat_buf)) {
-	libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s%s",
-			     _
-			     ("GnuPG is rebuilding the trust database and is currently unavailable."),
-			     _("Try again later."));
-	return TRUE;
-    } else
-	return FALSE;
+	if (lockname == NULL) {
+		lockname = g_build_filename(g_get_home_dir(), ".gnupg", "trustdb.gpg.lock", NULL);
+	}
+
+	if (g_file_test(lockname, G_FILE_TEST_EXISTS)) {
+		libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s %s",
+			_("GnuPG is rebuilding the trust database and is currently unavailable."),
+			_("Try again later."));
+		result = TRUE;
+	} else {
+		result = FALSE;
+	}
+	return result;
 }
 
 
-/* check if the context contains a public key for the passed recipients */
-#define KEY_IS_OK(k)   (!((k)->expired || (k)->revoked || \
-                          (k)->disabled || (k)->invalid))
+/* check if the local key ring contains a public key for the passed recipients */
 static gboolean
-have_pub_key_for(gpgme_ctx_t gpgme_ctx, InternetAddressList * recipients)
+have_pub_key_for(gpgme_ctx_t          gpgme_ctx,
+				 InternetAddressList *recipients)
 {
-    gpgme_key_t key;
     gboolean result = TRUE;
-    time_t now = time(NULL);
     gint i;
 
-    for (i = 0; result && i < internet_address_list_length(recipients);
-         i++) {
-        InternetAddress *ia =
-            internet_address_list_get_address(recipients, i);
+    for (i = 0; result && (i < internet_address_list_length(recipients)); i++) {
+    	InternetAddress *ia = internet_address_list_get_address(recipients, i);
 
-	/* check all entries in the list, handle groups recursively */
-	if (INTERNET_ADDRESS_IS_GROUP(ia))
-	    result =
-                have_pub_key_for(gpgme_ctx,
-                                 INTERNET_ADDRESS_GROUP(ia)->members);
-	else {
-	    if (gpgme_op_keylist_start(gpgme_ctx,
-                                       INTERNET_ADDRESS_MAILBOX(ia)->addr,
-                                       FALSE) != GPG_ERR_NO_ERROR)
-		return FALSE;
+    	/* check all entries in the list, handle groups recursively */
+    	if (INTERNET_ADDRESS_IS_GROUP(ia)) {
+    		result = have_pub_key_for(gpgme_ctx, INTERNET_ADDRESS_GROUP(ia)->members);
+    	} else {
+    		gchar *mail_name;
+    		GList *keys = NULL;
 
-	    result = FALSE;
-	    while (!result &&
-		   gpgme_op_keylist_next(gpgme_ctx, &key) == GPG_ERR_NO_ERROR) {
-		/* check if this key and the relevant subkey are usable */
-		if (KEY_IS_OK(key)) {
-		    gpgme_subkey_t subkey = key->subkeys;
-
-		    while (subkey && !subkey->can_encrypt)
-			subkey = subkey->next;
-
-		    if (subkey && KEY_IS_OK(subkey) && 
-			(subkey->expires == 0 || subkey->expires > now))
-			result = TRUE;
-		}
-		gpgme_key_unref(key);
-	    }
-	    gpgme_op_keylist_end(gpgme_ctx);
-	}
+    		result = FALSE;
+    		/* enclose the mail address into "<...>" to perform an exact search */
+    		mail_name = g_strconcat("<", INTERNET_ADDRESS_MAILBOX(ia)->addr, ">", NULL);
+    		if (libbalsa_gpgme_list_keys(gpgme_ctx, &keys, NULL, mail_name, FALSE, FALSE, FALSE, NULL)) {
+    			if (keys != NULL) {
+    				result = TRUE;
+    				g_list_free_full(keys, (GDestroyNotify) gpgme_key_unref);
+    			}
+    		}
+    		g_free(mail_name);
+    	}
     }
 
     return result;
