@@ -27,7 +27,7 @@
 #include "send.h"
 
 #include <math.h>
-
+#include <stdlib.h>
 #include <string.h>
 
 #include "libbalsa.h"
@@ -784,6 +784,10 @@ lbs_process_queue_init_session(LibBalsaServer* server)
 		session = net_client_smtp_new(server->host, 587U, server->security);
 	}
 
+	/* connect signals */
+	g_signal_connect(G_OBJECT(session), "cert-check", G_CALLBACK(libbalsa_server_check_cert), session);
+	g_signal_connect(G_OBJECT(session), "auth", G_CALLBACK(libbalsa_server_get_auth), server);
+
 	/* load client certificate if configured */
 	if (server->client_cert) {
 		GError* error = NULL;
@@ -792,14 +796,14 @@ lbs_process_queue_init_session(LibBalsaServer* server)
 		if (!net_client_set_cert_from_file(NET_CLIENT(session), server->cert_file, &error)) {
 			libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("Cannot load certificate file %s: %s"), server->cert_file,
 				error->message);
+			/* bad certificate private key password: clear it */
+			if (error->code == NET_CLIENT_ERROR_CERT_KEY_PASS) {
+				libbalsa_server_set_password(server, NULL, TRUE);
+			}
 			g_error_free(error);
 			g_object_unref(session);
 			session = NULL;
 		}
-	} else {
-		/* connect signals */
-		g_signal_connect(G_OBJECT(session), "cert-check", G_CALLBACK(libbalsa_server_check_cert), session);
-		g_signal_connect(G_OBJECT(session), "auth", G_CALLBACK(libbalsa_server_get_auth), server);
 	}
 
 	return session;
@@ -1048,7 +1052,7 @@ balsa_send_message_real(SendMessageInfo *info)
             g_mutex_unlock(&send_messages_lock);
         }
     } else {
-        if (ERROR_IS_TRANSIENT(error)) {
+        if (ERROR_IS_TRANSIENT(error) || (error->code == NET_CLIENT_ERROR_SMTP_AUTHFAIL)) {
             GList *this_msg;
 
             /* Mark all messages as neither flagged nor deleted, so they can be resent later
@@ -1063,6 +1067,10 @@ balsa_send_message_real(SendMessageInfo *info)
                                                   LIBBALSA_MESSAGE_FLAG_DELETED);
                 }
             }
+        	if (error->code == NET_CLIENT_ERROR_SMTP_AUTHFAIL) {
+        		/* authentication failed: clear password */
+        		libbalsa_server_set_password(LIBBALSA_SERVER(info->smtp_server), NULL, FALSE);
+        	}
         }
         libbalsa_information(LIBBALSA_INFORMATION_ERROR,
                              _("Connecting SMTP server %s (%s) failed: %s"),
@@ -1194,7 +1202,6 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
                                      gboolean         postponing,
                                      GError         **error)
 {
-    gchar **mime_type;
     GMimeObject *mime_root = NULL;
     GMimeMessage *mime_message;
     LibBalsaMessageBody *body;
@@ -1223,12 +1230,13 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
         mime_part = NULL;
 
         if ((body->file_uri != NULL) || (body->filename != NULL)) {
+            gchar **mime_type;
+
             if (body->content_type != NULL) {
                 mime_type = parse_content_type(body->content_type);
             } else {
-                gchar *mt = g_strdup(libbalsa_vfs_get_mime_type(body->file_uri));
+                const gchar *mt = libbalsa_vfs_get_mime_type(body->file_uri);
                 mime_type = g_strsplit(mt, "/", 2);
-                g_free(mt);
             }
 
             if (body->attach_mode == LIBBALSA_ATTACH_AS_EXTBODY) {
@@ -1273,6 +1281,7 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
                         g_clear_error(&err);
                         g_free(msg);
                     }
+                    g_strfreev(mime_type);
                     return LIBBALSA_MESSAGE_CREATE_ERROR;
                 }
                 parser = g_mime_parser_new_with_stream(stream);
@@ -1340,6 +1349,7 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
                         g_free(msg);
                     }
                     g_object_unref(G_OBJECT(mime_part));
+                    g_strfreev(mime_type);
                     return LIBBALSA_MESSAGE_CREATE_ERROR;
                 }
                 content = g_mime_data_wrapper_new_with_stream(stream,
@@ -1501,6 +1511,20 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
     g_mime_object_append_header(GMIME_OBJECT(mime_message), "X-Mailer", tmp);
     g_free(tmp);
 
+#ifdef ENABLE_AUTOCRYPT
+    /* add Autocrypt header if requested */
+    if (!postponing && (message->ident != NULL) && (message->ident->autocrypt_mode != AUTOCRYPT_DISABLE) &&
+    	!autocrypt_ignore(g_mime_object_get_content_type(mime_root))) {
+    	tmp = autocrypt_header(message->ident, error);
+    	if (tmp == NULL) {
+    		g_object_unref(G_OBJECT(mime_message));
+    		return LIBBALSA_MESSAGE_CREATE_ERROR;
+    	}
+    	g_mime_object_append_header(GMIME_OBJECT(mime_message), "Autocrypt", tmp);
+    	g_free(tmp);
+    }
+#endif
+
     message->mime_msg = mime_message;
 
     return LIBBALSA_MESSAGE_CREATE_OK;
@@ -1578,9 +1602,11 @@ libbalsa_set_message_id(GMimeMessage *mime_message)
         /* initialise some stuff on first-time use... */
         rand = g_rand_new_with_seed((guint32) time(NULL));
         strncpy(id_data.user_name, g_get_user_name(),
-                sizeof(id_data.user_name));
+                sizeof(id_data.user_name) - 1);
+        id_data.user_name[sizeof(id_data.user_name) - 1] = '\0';
         strncpy(id_data.host_name, g_get_host_name(),
-                sizeof(id_data.host_name));
+                sizeof(id_data.host_name) - 1);
+        id_data.host_name[sizeof(id_data.host_name) - 1] = '\0';
     }
 
     /* get some randomness... */
@@ -1798,7 +1824,8 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessage *message,
                                         message->headers->cc_list);
         encrypt_for = get_mailbox_names(encrypt_for,
                                         message->headers->from);
-        if (message->headers->bcc_list != NULL) {
+        if ((message->headers->bcc_list != NULL) &&
+        	(internet_address_list_length(message->headers->bcc_list) > 0)) {
             libbalsa_information
                 (LIBBALSA_INFORMATION_WARNING,
                 ngettext("This message will not be encrypted "
@@ -1894,9 +1921,14 @@ do_multipart_crypto(LibBalsaMessage *message,
         if (message->headers->bcc_list
             && (internet_address_list_length(message->headers->
                                              bcc_list) > 0)) {
-            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                                 _(
-                                     "This message will not be encrypted for the BCC: recipient(s)."));
+            libbalsa_information
+                (LIBBALSA_INFORMATION_WARNING,
+                ngettext("This message will not be encrypted "
+                         "for the BCC: recipient.",
+                         "This message will not be encrypted "
+                         "for the BCC: recipients.",
+                         internet_address_list_length
+                             (message->headers->bcc_list)));
         }
 
         if (message->gpg_mode & LIBBALSA_PROTECT_SIGN) {
