@@ -30,12 +30,10 @@
 #include <libsecret/secret.h>
 #endif                          /* defined(HAVE_LIBSECRET) */
 
-#include <openssl/err.h>
-
 #include "libbalsa.h"
 #include "libbalsa_private.h"
-#include "libbalsa-marshal.h"
 #include "libbalsa-conf.h"
+#include "net-client-utils.h"
 #include <glib/gi18n.h>
 
 #if defined(HAVE_LIBSECRET)
@@ -48,7 +46,16 @@ static const SecretSchema server_schema = {
 	{ NULL, 0 }
     }
 };
-const SecretSchema *LIBBALSA_SERVER_SECRET_SCHEMA = &server_schema;
+
+static const SecretSchema cert_pass_schema = {
+    "org.gnome.Balsa.CertificatePassword", SECRET_SCHEMA_NONE,
+    {
+	{ "protocol", SECRET_SCHEMA_ATTRIBUTE_STRING },
+	{ "server",   SECRET_SCHEMA_ATTRIBUTE_STRING },
+	{ "user",     SECRET_SCHEMA_ATTRIBUTE_STRING },
+	{ NULL, 0 }
+    }
+};
 #endif                          /* defined(HAVE_LIBSECRET) */
 
 static GObjectClass *parent_class = NULL;
@@ -58,10 +65,12 @@ static void libbalsa_server_finalize(GObject * object);
 
 static void libbalsa_server_real_set_username(LibBalsaServer * server,
 					      const gchar * username);
-static void libbalsa_server_real_set_host(LibBalsaServer * server,
-					  const gchar * host,
-                                          gboolean use_ssl);
-/* static gchar* libbalsa_server_real_get_password(LibBalsaServer *server); */
+static void libbalsa_server_real_set_host(LibBalsaServer     *server,
+					  	  	  	  	  	  const gchar        *host,
+										  NetClientCryptMode  security);
+static gchar *libbalsa_server_get_password(LibBalsaServer *server,
+										   gchar          *cert_subject);
+static gchar *libbalsa_free_password(gchar *password);
 
 enum {
     SET_USERNAME,
@@ -116,8 +125,7 @@ libbalsa_server_class_init(LibBalsaServerClass * klass)
                      G_SIGNAL_RUN_FIRST,
 		     G_STRUCT_OFFSET(LibBalsaServerClass,
 				     set_username),
-                     NULL, NULL,
-                     g_cclosure_marshal_VOID__STRING,
+                     NULL, NULL, NULL,
                      G_TYPE_NONE, 1,
 		     G_TYPE_STRING);
     libbalsa_server_signals[SET_HOST] =
@@ -126,8 +134,7 @@ libbalsa_server_class_init(LibBalsaServerClass * klass)
                      G_SIGNAL_RUN_FIRST,
 		     G_STRUCT_OFFSET(LibBalsaServerClass,
                                      set_host),
-                     NULL, NULL,
-                     libbalsa_VOID__POINTER_INT,
+                     NULL, NULL, NULL,
                      G_TYPE_NONE, 2,
                      G_TYPE_POINTER, G_TYPE_INT
 		     );
@@ -137,25 +144,20 @@ libbalsa_server_class_init(LibBalsaServerClass * klass)
                      G_SIGNAL_RUN_FIRST,
 		     G_STRUCT_OFFSET(LibBalsaServerClass,
 				     config_changed),
-                     NULL, NULL,
-                     g_cclosure_marshal_VOID__VOID,
+                     NULL, NULL, NULL,
                      G_TYPE_NONE, 0);
 
-
     libbalsa_server_signals[GET_PASSWORD] =
 	g_signal_new("get-password",
                      G_TYPE_FROM_CLASS(object_class),
                      G_SIGNAL_RUN_LAST,
-		     G_STRUCT_OFFSET(LibBalsaServerClass,
-			             get_password),
-                     NULL, NULL,
-		     libbalsa_POINTER__OBJECT,
-                     G_TYPE_POINTER, 1,
-                     LIBBALSA_TYPE_MAILBOX);
+					 0U,
+                     NULL, NULL, NULL,
+                     G_TYPE_STRING, 1,
+					 G_TYPE_BYTE_ARRAY);
 
     klass->set_username = libbalsa_server_real_set_username;
     klass->set_host = libbalsa_server_real_set_host;
-    klass->get_password = NULL;	/* libbalsa_server_real_get_password; */
 }
 
 static void
@@ -166,8 +168,6 @@ libbalsa_server_init(LibBalsaServer * server)
     server->user = NULL;
     server->passwd = NULL;
     server->remember_passwd = TRUE;
-    server->use_ssl         = FALSE;
-    server->tls_mode        = LIBBALSA_TLS_ENABLED;
     server->security		= NET_CLIENT_CRYPT_STARTTLS;
 }
 
@@ -183,18 +183,11 @@ libbalsa_server_finalize(GObject * object)
 
     g_free(server->host);   server->host = NULL;
     g_free(server->user);   server->user = NULL;
-    if (server->passwd != NULL) {
-    	memset(server->passwd, 0, strlen(server->passwd));
-    }
-    libbalsa_free_password(server->passwd); server->passwd = NULL;
+    server->passwd = libbalsa_free_password(server->passwd);
 
     g_free(server->cert_file);
     server->cert_file = NULL;
-    if (server->cert_passphrase != NULL) {
-    	memset(server->cert_passphrase, 0, strlen(server->cert_passphrase));
-    }
-    g_free(server->cert_passphrase);
-    server->cert_passphrase = NULL;
+    server->cert_passphrase = libbalsa_free_password(server->cert_passphrase);
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -221,26 +214,34 @@ libbalsa_server_set_username(LibBalsaServer * server,
 }
 
 void
-libbalsa_server_set_password(LibBalsaServer * server,
-                             const gchar * passwd)
+libbalsa_server_set_password(LibBalsaServer *server,
+                             const gchar    *passwd,
+							 gboolean        for_cert)
 {
+	gchar **target;
     g_return_if_fail(LIBBALSA_IS_SERVER(server));
 
-    libbalsa_free_password(server->passwd);
-    if(passwd && passwd[0])
-	server->passwd = g_strdup(passwd);
-    else server->passwd = NULL;
+    if (for_cert) {
+    	target = &server->cert_passphrase;
+    } else {
+    	target = &server->passwd;
+    }
+
+    *target = libbalsa_free_password(*target);
+    if ((passwd != NULL) && (passwd[0] != '\0')) {
+    	*target = g_strdup(passwd);
+    }
 }
 
 void
 libbalsa_server_set_host(LibBalsaServer * server, const gchar * host,
-                         gboolean use_ssl)
+                         NetClientCryptMode security)
 {
     g_return_if_fail(server != NULL);
     g_return_if_fail(LIBBALSA_IS_SERVER(server));
 
     g_signal_emit(G_OBJECT(server), libbalsa_server_signals[SET_HOST],
-		  0, host, use_ssl);
+		  0, host, security);
 
 }
 
@@ -254,17 +255,16 @@ libbalsa_server_config_changed(LibBalsaServer * server)
                   0);
 }
 
-gchar *
-libbalsa_server_get_password(LibBalsaServer * server,
-			     LibBalsaMailbox * mbox)
+static gchar *
+libbalsa_server_get_password(LibBalsaServer *server,
+			     	 	 	 gchar          *cert_subject)
 {
     gchar *retval = NULL;
 
     g_return_val_if_fail(server != NULL, NULL);
     g_return_val_if_fail(LIBBALSA_IS_SERVER(server), NULL);
 
-    g_signal_emit(G_OBJECT(server), libbalsa_server_signals[GET_PASSWORD],
-                  0, mbox, &retval);
+    g_signal_emit(G_OBJECT(server), libbalsa_server_signals[GET_PASSWORD], 0, cert_subject, &retval);
     return retval;
 }
 
@@ -280,128 +280,211 @@ libbalsa_server_real_set_username(LibBalsaServer * server,
 
 static void
 libbalsa_server_real_set_host(LibBalsaServer * server, const gchar * host,
-                              gboolean use_ssl)
+	NetClientCryptMode  security)
 {
     g_return_if_fail(LIBBALSA_IS_SERVER(server));
 
     g_free(server->host);
     server->host = g_strdup(host);
-    server->use_ssl = use_ssl;
+    server->security = security;
 }
 
-
-#if 0
-static gchar *
-libbalsa_server_real_get_password(LibBalsaServer * server)
+void
+libbalsa_server_load_security_config(LibBalsaServer *server)
 {
-    g_return_val_if_fail(LIBBALSA_IS_SERVER(server), NULL);
+	gboolean not_found;
+
+    g_return_if_fail(LIBBALSA_IS_SERVER(server));
+
+    server->security = libbalsa_conf_get_int_with_default("Security", &not_found);
+    if (not_found) {
+    	gboolean want_ssl;
+
+    	g_debug("server %s@%s: no security config, try to read old settings", server->protocol, server->host);
+    	want_ssl = libbalsa_conf_get_bool_with_default("SSL", &not_found);
+    	if (want_ssl && !not_found) {
+    		server->security = NET_CLIENT_CRYPT_ENCRYPTED;
+    	} else {
+    		int want_tls;
+
+    		want_tls = libbalsa_conf_get_int_with_default("TLSMode", &not_found);
+    		if (not_found) {
+    			server->security = NET_CLIENT_CRYPT_STARTTLS;
+    		} else {
+    			switch (want_tls) {
+    			case 0:
+    				server->security = NET_CLIENT_CRYPT_NONE;
+    				break;
+    			case 1:
+    				server->security = NET_CLIENT_CRYPT_STARTTLS_OPT;
+    				break;
+    			default:
+    				server->security = NET_CLIENT_CRYPT_STARTTLS;
+    			}
+    		}
+    	}
+    }
 
-    return g_strdup(server->passwd);
 }
+
+
+#if defined(HAVE_LIBSECRET)
+
+static gchar *
+libbalsa_free_password(gchar *password)
+{
+	secret_password_free(password);
+	return NULL;
+}
+
+static void
+store_password_libsecret(const LibBalsaServer *server,
+	    				 const gchar          *password,
+						 const SecretSchema   *schema)
+{
+    GError *err = NULL;
+
+    secret_password_store_sync(schema, NULL, _("Balsa passwords"), password, NULL, &err,
+    	"protocol", server->protocol,
+		"server",   server->host,
+		"user",     server->user,
+		NULL);
+    if (err != NULL) {
+    	g_info(_("Error storing password for %s@%s:%s: %s"), server->user, server->protocol, server->host, err->message);
+        g_error_free(err);
+    } else {
+    	g_debug("stored password for %s@%s:%s in secret service", server->user, server->protocol, server->host);
+    }
+}
+
+static gchar *
+load_password_libsecret(const LibBalsaServer *server,
+			  	  	    const gchar          *cfg_path,
+						const SecretSchema   *schema)
+{
+    GError *err = NULL;
+    gchar *password;
+
+    password = secret_password_lookup_sync(schema, NULL, &err,
+    	"protocol", server->protocol,
+		"server",   server->host,
+		"user",     server->user,
+		NULL);
+    if (err != NULL) {
+        g_info(_("Error looking up password for %s@%s:%s: %s"), server->user, server->protocol, server->host, err->message);
+        g_error_free(err);
+
+        /* fall back to the private config file */
+        password = libbalsa_conf_private_get_string(cfg_path, TRUE);
+        if ((password != NULL) && (password[0] != '\0')) {
+        	g_info(_("loaded fallback password from private config file"));
+        	/* store a fallback password in the key ring */
+        	store_password_libsecret(server, password, schema);
+            /* We could in principle clear the password in the config file here but we do not for the backward compatibility.
+             * FIXME - Is this really the proper approach, as we leave the obfuscated password in the config file, and the user
+             * relies on the message that the password is stored in the Secret Service only.  OTOH, there are better methods for
+             * protecting the user's secrets, like full disk encryption. */
+        }
+    } else {
+    	g_debug("loaded password for %s@%s:%s from secret service", server->user, server->protocol, server->host);
+    }
+
+    return password;
+}
+
+static void
+erase_password_libsecret(const LibBalsaServer *server,
+						 const SecretSchema   *schema)
+{
+    GError *err = NULL;
+    gboolean removed;
+
+    removed = secret_password_clear_sync(schema, NULL, &err,
+    	"protocol", server->protocol,
+		"server",   server->host,
+		"user",     server->user,
+		NULL);
+    if (err != NULL) {
+    	g_info("error erasing password for %s@%s:%s from secret service: %s", server->user, server->protocol, server->host,
+    		err->message);
+    	g_error_free(err);
+    } else if (removed) {
+    	g_debug("erased password for %s@%s:%s from secret service", server->user, server->protocol, server->host);
+    } else {
+    	g_debug("no password erased for %s@%s:%s from secret service", server->user, server->protocol, server->host);
+    }
+}
+
+#else
+
+static gchar *
+libbalsa_free_password(gchar *password)
+{
+	net_client_free_authstr(password);
+	return NULL;
+}
+
 #endif
 
 
 /* libbalsa_server_load_config:
    load the server configuration using gnome-config.
-   Try to use sensible defaults. 
+   Try to use sensible defaults.
    FIXME: Port field is kept here only for compatibility, drop after 1.4.x
    release.
 */
 void
 libbalsa_server_load_config(LibBalsaServer * server)
 {
-    gboolean d;
     server->host = libbalsa_conf_get_string("Server");
-    if(server->host && strrchr(server->host, ':') == NULL) {
+    if ((server->host != NULL) && (strrchr(server->host, ':') == NULL)) {
         gint port;
+        gboolean d;
+
         port = libbalsa_conf_get_int_with_default("Port", &d);
         if (!d) {
             gchar *newhost = g_strdup_printf("%s:%d", server->host, port);
+
             g_free(server->host);
             server->host = newhost;
         }
-    }       
-    server->use_ssl = libbalsa_conf_get_bool("SSL=false");
-    d = FALSE;
-    server->tls_mode = libbalsa_conf_get_int_with_default("TLSMode", &d);
-    if(d) server->tls_mode = LIBBALSA_TLS_ENABLED;
-    d = FALSE;
-    server->security = libbalsa_conf_get_int_with_default("Security", &d);
-    if (d) {
-    	server->security = NET_CLIENT_CRYPT_STARTTLS;
     }
-    server->user = libbalsa_conf_private_get_string("Username");
-    if (!server->user)
-	server->user = g_strdup(getenv("USER"));
+    libbalsa_server_load_security_config(server);
+    server->user = libbalsa_conf_private_get_string("Username", FALSE);
+    if (server->user == NULL) {
+    	server->user = g_strdup(g_get_user_name());
+    }
 
     server->try_anonymous = libbalsa_conf_get_bool("Anonymous=false");
     server->remember_passwd = libbalsa_conf_get_bool("RememberPasswd=false");
 
-    if(server->remember_passwd) {
+    server->passwd = libbalsa_free_password(server->passwd);
+    if (server->remember_passwd) {
 #if defined(HAVE_LIBSECRET)
-        GError *err = NULL;
-
-        server->passwd =
-            secret_password_lookup_sync(LIBBALSA_SERVER_SECRET_SCHEMA,
-                                        NULL, &err,
-                                        "protocol", server->protocol,
-                                        "server",   server->host,
-                                        "user",     server->user,
-                                        NULL);
-        if (err) {
-            libbalsa_free_password(server->passwd);
-            server->passwd = NULL;
-            printf(_("Error looking up password for %s@%s: %s\n"),
-                   server->user, server->host, err->message);
-            printf(_("Falling back\n"));
-            g_clear_error(&err);
-            server->passwd = libbalsa_conf_private_get_string("Password");
-            if (server->passwd != NULL) {
-                gchar *buff = libbalsa_rot(server->passwd);
-                libbalsa_free_password(server->passwd);
-                server->passwd = buff;
-                secret_password_store_sync
-                    (LIBBALSA_SERVER_SECRET_SCHEMA, NULL,
-                     _("Balsa passwords"), server->passwd, NULL, &err,
-                     "protocol", server->protocol,
-                     "server",   server->host,
-                     "user",     server->user,
-                     NULL);
-                /* We could in principle clear the password in the
-                 * config file here but we do not for the backward
-                 * compatibility. */
-                if (err) {
-                    printf(_("Error storing password for %s@%s: %s\n"),
-                           server->user, server->host, err->message);
-                    g_error_free(err);
-                }
-            }
-        }
+    	server->passwd = load_password_libsecret(server, "Password", &server_schema);
 #else
-	server->passwd = libbalsa_conf_private_get_string("Password");
-	if (server->passwd != NULL) {
-	    gchar *buff = libbalsa_rot(server->passwd);
-	    libbalsa_free_password(server->passwd);
-	    server->passwd = buff;
-	}
-#endif
-    }
-    if(server->passwd && server->passwd[0] == '\0') {
-	libbalsa_free_password(server->passwd);
-	server->passwd = NULL;
+    	server->passwd = libbalsa_conf_private_get_string("Password", TRUE);
+#endif		/* HAVE_LIBSECRET */
+    	if ((server->passwd != NULL) && (server->passwd[0] == '\0')) {
+    		server->passwd = libbalsa_free_password(server->passwd);
+    	}
     }
 
+    /* client certificate stuff */
     server->client_cert = libbalsa_conf_get_bool("NeedClientCert=false");
     server->cert_file = libbalsa_conf_get_string("UserCertificateFile");
-    server->cert_passphrase = libbalsa_conf_private_get_string("CertificatePassphrase");
-    if ((server->cert_passphrase != NULL) && (server->cert_passphrase[0] != '\0')) {
-        gchar *tmp = libbalsa_rot(server->cert_passphrase);
+    server->remember_cert_passphrase = libbalsa_conf_get_bool("RememberCertPasswd=false");
 
-        g_free(server->cert_passphrase);
-        server->cert_passphrase = tmp;
-    } else {
-    	g_free(server->cert_passphrase);
-    	server->cert_passphrase = NULL;
+    server->cert_passphrase = libbalsa_free_password(server->cert_passphrase);
+    if (server->client_cert && server->remember_cert_passphrase) {
+#if defined(HAVE_LIBSECRET)
+    	server->cert_passphrase = load_password_libsecret(server, "CertificatePassphrase", &cert_pass_schema);
+#else
+    	server->cert_passphrase = libbalsa_conf_private_get_string("CertificatePassphrase", TRUE);
+#endif		/* HAVE_LIBSECRET */
+    	if ((server->cert_passphrase != NULL) && (server->cert_passphrase[0] == '\0')) {
+    		server->cert_passphrase = libbalsa_free_password(server->cert_passphrase);
+    	}
     }
 }
 
@@ -415,125 +498,45 @@ void
 libbalsa_server_save_config(LibBalsaServer * server)
 {
     libbalsa_conf_set_string("Server", server->host);
-    libbalsa_conf_private_set_string("Username", server->user);
+    libbalsa_conf_private_set_string("Username", server->user, FALSE);
     libbalsa_conf_set_bool("Anonymous",          server->try_anonymous);
-    libbalsa_conf_set_bool("RememberPasswd", 
-                          server->remember_passwd && server->passwd != NULL);
 
-    if (server->remember_passwd && server->passwd != NULL) {
+    if (server->remember_passwd && (server->passwd != NULL)) {
+    	libbalsa_conf_set_bool("RememberPasswd", TRUE);
 #if defined(HAVE_LIBSECRET)
-        GError *err = NULL;
-
-        secret_password_store_sync(LIBBALSA_SERVER_SECRET_SCHEMA, NULL,
-                                   _("Balsa passwords"), server->passwd,
-                                   NULL, &err,
-                                   "protocol", server->protocol,
-                                   "server",   server->host,
-                                   "user",     server->user,
-                                   NULL);
-        if (err) {
-            printf(_("Error storing password for %s@%s: %s\n"),
-                   server->user, server->host, err->message);
-            g_error_free(err);
-        }
+    	store_password_libsecret(server, server->passwd, &server_schema);
 #else
-	gchar *buff = libbalsa_rot(server->passwd);
-	libbalsa_conf_private_set_string("Password", buff);
-	g_free(buff);
-#endif                          /* defined(HAVE_LIBSECRET) */
+    	libbalsa_conf_private_set_string("Password", server->passwd, TRUE);
+#endif
+    } else {
+    	libbalsa_conf_set_bool("RememberPasswd", FALSE);
+#if defined(HAVE_LIBSECRET)
+    	erase_password_libsecret(server, &server_schema);
+#endif		/* HAVE_LIBSECRET */
+    	libbalsa_conf_private_set_string("Password", NULL, TRUE);
     }
 
     libbalsa_conf_set_bool("NeedClientCert", server->client_cert);
     if (server->cert_file != NULL) {
     	libbalsa_conf_set_string("UserCertificateFile", server->cert_file);
-    }
-    if (server->cert_passphrase != NULL) {
-        gchar *tmp = libbalsa_rot(server->cert_passphrase);
-
-        libbalsa_conf_private_set_string("CertificatePassphrase", tmp);
-        g_free(tmp);
-    }
-
-    libbalsa_conf_set_bool("SSL", server->use_ssl);
-    libbalsa_conf_set_int("TLSMode", server->tls_mode);
-    libbalsa_conf_set_int("Security", server->security);
-}
-
-void
-libbalsa_server_user_cb(ImapUserEventType ue, void *arg, ...)
-{
-    va_list alist;
-    int *ok;
-    LibBalsaServer *is = LIBBALSA_SERVER(arg);
-
-    va_start(alist, arg);
-    switch(ue) {
-    case IME_GET_USER_PASS: {
-        gchar *method = va_arg(alist, gchar*);
-        gchar **user = va_arg(alist, gchar**);
-        gchar **pass = va_arg(alist, gchar**);
-        ok = va_arg(alist, int*);
-        if(!is->passwd) {
-            is->passwd = libbalsa_server_get_password(is, NULL);
-        }
-        *ok = is->passwd != NULL;
-        if(*ok) {
-            g_free(*user); *user = g_strdup(is->user);
-            g_free(*pass); *pass = g_strdup(is->passwd);
-            libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
-                                 /* host, authentication method */
-                                 _("Logging in to %s using %s"), 
-                                   is->host, method);
-        }
-        break;
-    }
-    case IME_GET_USER:  { /* for eg kerberos */
-        gchar **user;
-        va_arg(alist, gchar*); /* Ignore the method */
-        user = va_arg(alist, gchar**);
-        ok = va_arg(alist, int*);
-        *ok = 1; /* consider popping up a dialog window here */
-        g_free(*user); *user = g_strdup(is->user);
-        break;
-    }
-    case IME_TLS_VERIFY_ERROR:  {
-        long vfy_result;
-        SSL *ssl;
-        X509 *cert;
-        ok = va_arg(alist, int*);
-        vfy_result = va_arg(alist, long);
-        X509_verify_cert_error_string(vfy_result);
-#if 0
-        printf("IMAP:TLS: failed cert verification: %ld : %s.\n",
-               vfy_result, reason);
+    	if (server->remember_cert_passphrase && (server->cert_passphrase != NULL)) {
+    		libbalsa_conf_set_bool("RememberCertPasswd", TRUE);
+#if defined(HAVE_LIBSECRET)
+    		store_password_libsecret(server, server->cert_passphrase, &cert_pass_schema);
+#else
+    		libbalsa_conf_private_set_string("CertificatePassphrase", server->cert_passphrase, TRUE);
 #endif
-        ssl = va_arg(alist, SSL*);
-        cert = SSL_get_peer_certificate(ssl);
-	if(cert) {
-	    *ok = libbalsa_is_cert_known(cert, vfy_result);
-	    X509_free(cert);
-	}
-        break;
+
+    	} else {
+    		libbalsa_conf_set_bool("RememberCertPasswd", FALSE);
+#if defined(HAVE_LIBSECRET)
+    		erase_password_libsecret(server, &cert_pass_schema);
+#endif		/* HAVE_LIBSECRET */
+    		libbalsa_conf_private_set_string("CertificatePassphrase", NULL, TRUE);
+    	}
     }
-    case IME_TLS_NO_PEER_CERT: {
-        ok = va_arg(alist, int*); *ok = 0;
-        printf("IMAP:TLS: Server presented no cert!\n");
-        break;
-    }
-    case IME_TLS_WEAK_CIPHER: {
-        ok = va_arg(alist, int*); *ok = 1;
-        printf("IMAP:TLS: Weak cipher accepted.\n");
-        break;
-    }
-    case IME_TIMEOUT: {
-        ok = va_arg(alist, int*); *ok = 1;
-        /* *ok = libbalsa_abort_on_timeout(is->host); */
-        /* For now, always timeout. The UI needs some work. */
-        break;
-    }
-    default: g_warning("unhandled imap event type! Fix the code."); break;
-    }
-    va_end(alist);
+
+    libbalsa_conf_set_int("Security", server->security);
 }
 
 /* Connect the server's "get-password" signal to the callback; if the
@@ -561,7 +564,7 @@ libbalsa_server_get_auth(NetClient *client,
 
     g_debug("%s: %p %p: encrypted = %d", __func__, client, user_data,
             net_client_is_encrypted(client));
-    if (server->try_anonymous == 0U) {
+    if (!server->try_anonymous || (strcmp(server->protocol, "imap") == 0)) {
         result = g_new0(gchar *, 3U);
         result[0] = g_strdup(server->user);
         if (need_passwd) {
@@ -582,60 +585,24 @@ libbalsa_server_check_cert(NetClient           *client,
 						   GTlsCertificateFlags errors,
 						   gpointer             user_data)
 {
-    GByteArray *cert_der = NULL;
-    gboolean result = FALSE;
-
-    /* FIXME - this a hack, simulating the (OpenSSL based) input for libbalsa_is_cert_known().
-        If we switch completely to
-     * (GnuTLS based) GTlsCertificate/GTlsClientConnection, we can omit this... */
-    g_debug("%s: %p %p %u %p", __func__, client, peer_cert, errors, user_data);
-
-    /* create a OpenSSL X509 object from the certificate's DER data */
-    g_object_get(G_OBJECT(peer_cert), "certificate", &cert_der, NULL);
-    if (cert_der != NULL) {
-        X509 *ossl_cert;
-        const unsigned char *der_p;
-
-        der_p = (const unsigned char *) cert_der->data;
-        ossl_cert = d2i_X509(NULL, &der_p, cert_der->len);
-        g_byte_array_unref(cert_der);
-
-        if (ossl_cert != NULL) {
-            long vfy_result;
-
-            /* convert the GIO error flags into OpenSSL error flags */
-            if ((errors & G_TLS_CERTIFICATE_UNKNOWN_CA) == G_TLS_CERTIFICATE_UNKNOWN_CA) {
-                vfy_result = X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT;
-            } else if ((errors & G_TLS_CERTIFICATE_BAD_IDENTITY) ==
-                       G_TLS_CERTIFICATE_BAD_IDENTITY) {
-                vfy_result = X509_V_ERR_SUBJECT_ISSUER_MISMATCH;
-            } else if ((errors & G_TLS_CERTIFICATE_NOT_ACTIVATED) ==
-                       G_TLS_CERTIFICATE_NOT_ACTIVATED) {
-                vfy_result = X509_V_ERR_CERT_NOT_YET_VALID;
-            } else if ((errors & G_TLS_CERTIFICATE_EXPIRED) == G_TLS_CERTIFICATE_EXPIRED) {
-                vfy_result = X509_V_ERR_CERT_HAS_EXPIRED;
-            } else if ((errors & G_TLS_CERTIFICATE_REVOKED) == G_TLS_CERTIFICATE_REVOKED) {
-                vfy_result = X509_V_ERR_CERT_REVOKED;
-            } else {
-                vfy_result = X509_V_ERR_APPLICATION_VERIFICATION;
-            }
-
-            result = libbalsa_is_cert_known(ossl_cert, vfy_result);
-            X509_free(ossl_cert);
-        }
-    }
-
-    return result;
+	return libbalsa_is_cert_known(peer_cert, errors);
 }
 
 
 gchar *
-libbalsa_server_get_cert_pass(NetClient        *client,
-			  	  	  	  	  const GByteArray *cert_der,
-							  gpointer          user_data)
+libbalsa_server_get_cert_pass(NetClient *client,
+			  	  	  	  	  gchar     *cert_subject,
+							  gpointer   user_data)
 {
-	/* FIXME - we just return the passphrase from the config, but we may also want to show a dialogue here... */
-	return g_strdup(LIBBALSA_SERVER(user_data)->cert_passphrase);
+    LibBalsaServer *server = LIBBALSA_SERVER(user_data);
+    gchar *result;
+
+    if ((server->cert_passphrase != NULL) && (server->cert_passphrase[0] != '\0')) {
+    	result = g_strdup(server->cert_passphrase);
+    } else {
+    	result = libbalsa_server_get_password(server, cert_subject);
+    }
+	return result;
 }
 
 /* Test whether a server is reachable */
