@@ -40,11 +40,11 @@ struct _NetClientPopPrivate {
 #define IS_ML_TERM(str)				((str[0] == '.') && (str[1] == '\0'))
 /*lint -emacro(9079,POP_MSG_INFO) -emacro(9087,POP_MSG_INFO)
  * allow conversion of GList data pointer, MISRA C:2012, Rules 11.3, 11.5 */
-#define POP_MSG_INFO(list)				((NetClientPopMessageInfo *) ((list)->data))
+#define POP_MSG_INFO(list)			((NetClientPopMessageInfo *) ((list)->data))
 /*lint -restore */
 
 
-G_DEFINE_TYPE(NetClientPop, net_client_pop, NET_CLIENT_TYPE)
+G_DEFINE_TYPE_WITH_PRIVATE(NetClientPop, net_client_pop, NET_CLIENT_TYPE)
 
 
 static void net_client_pop_finalise(GObject *object);
@@ -171,16 +171,28 @@ net_client_pop_connect(NetClientPop *client, gchar **greeting, GError **error)
 
 		auth_data = NULL;
 		need_pwd = (auth_supported & NET_CLIENT_POP_AUTH_NO_PWD) == 0U;
-		g_debug("emit 'auth' signal for client %p", client);
+		g_debug("emit 'auth' signal for client %p, need pwd %d", client, need_pwd);
 		g_signal_emit_by_name(client, "auth", need_pwd, &auth_data);
 		if ((auth_data != NULL) && (auth_data[0] != NULL)) {
 			result = net_client_pop_auth(client, auth_data[0], auth_data[1], auth_supported, error);
-			memset(auth_data[0], 0, strlen(auth_data[0]));
-			if (auth_data[1] != NULL) {
-				memset(auth_data[1], 0, strlen(auth_data[1]));
+			net_client_free_authstr(auth_data[0]);
+			net_client_free_authstr(auth_data[1]);
+
+			/* if passwordless authentication failed, try again with password */
+			if (!result && !need_pwd) {
+				g_debug("passwordless authentication failed, retry w/ password: emit 'auth' signal for client %p", client);
+				g_clear_error(error);
+				g_free(auth_data);
+				g_signal_emit_by_name(client, "auth", TRUE, &auth_data);
+				if ((auth_data != NULL) && (auth_data[0] != NULL)) {
+					result = net_client_pop_auth(client, auth_data[0], auth_data[1],
+						auth_supported & ~NET_CLIENT_POP_AUTH_NO_PWD, error);
+					net_client_free_authstr(auth_data[0]);
+					net_client_free_authstr(auth_data[1]);
+				}
 			}
 		}
-		g_strfreev(auth_data);
+		g_free(auth_data);
 	}
 
 	return result;
@@ -382,7 +394,7 @@ net_client_pop_class_init(NetClientPopClass *klass)
 static void
 net_client_pop_init(NetClientPop *self)
 {
-	self->priv = g_new0(NetClientPopPrivate, 1U);
+	self->priv = net_client_pop_get_instance_private(self);		/*lint !e9079 (MISRA C:2012 Rule 11.5) intended use */
 	self->priv->auth_allowed[0] = NET_CLIENT_POP_AUTH_ALL;
 	self->priv->auth_allowed[1] = NET_CLIENT_POP_AUTH_SAFE;
 }
@@ -399,10 +411,7 @@ net_client_pop_finalise(GObject *object)
 		(void) net_client_execute(NET_CLIENT(client), NULL, "QUIT", NULL);
 	}
 
-	if (client->priv != NULL) {
-		g_free(client->priv->apop_banner);
-		g_free(client->priv);
-	}
+	g_free(client->priv->apop_banner);
 	(*parent_class->finalize)(object);
 }
 
@@ -417,8 +426,12 @@ net_client_pop_read_reply(NetClientPop *client, gchar **reply, GError **error)
 	result = net_client_read_line(NET_CLIENT(client), &reply_buf, error);
 	if (result) {
 		if (strncmp(reply_buf, "+OK", 3U) == 0) {
-			if ((strlen(reply_buf) > 3U) && (reply != NULL)) {
-				*reply = g_strdup(&reply_buf[4]);
+			if (reply != NULL) {
+				if (strlen(reply_buf) > 3U) {
+					*reply = g_strdup(&reply_buf[4]);
+				} else {
+					*reply = g_strdup("");
+				}
 			}
 		} else if (strncmp(reply_buf, "-ERR", 4U) == 0) {
 			if (strlen(reply_buf) > 4U) {
@@ -487,7 +500,10 @@ net_client_pop_auth(NetClientPop *client, const gchar *user, const gchar *passwd
 		auth_mask = client->priv->auth_allowed[1] & auth_supported;
 	}
 
-	if (((auth_mask & NET_CLIENT_POP_AUTH_NO_PWD) == 0U) && (passwd == NULL)) {
+	if (auth_mask == 0U) {
+		g_set_error(error, NET_CLIENT_POP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_POP_NO_AUTH,
+			_("no suitable authentication mechanism"));
+	} else if (((auth_mask & NET_CLIENT_POP_AUTH_NO_PWD) == 0U) && (passwd == NULL)) {
 		g_set_error(error, NET_CLIENT_POP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_POP_NO_AUTH, _("password required"));
 	} else {
 		/* first try authentication methods w/o password, then safe ones, and finally the plain-text methods */
@@ -506,8 +522,13 @@ net_client_pop_auth(NetClientPop *client, const gchar *user, const gchar *passwd
 		} else if ((auth_mask & NET_CLIENT_POP_AUTH_LOGIN) != 0U) {
 			result = net_client_pop_auth_login(client, user, passwd, error);
 		} else {
-			g_set_error(error, NET_CLIENT_POP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_POP_NO_AUTH,
-				_("no suitable authentication mechanism"));
+			g_assert_not_reached();
+		}
+
+		/* POP3 does not define a mechanism to indicate that the authentication failed due to a too weak mechanism or wrong
+		 * credentials, so we treat all server -ERR responses as authentication failures */
+		if (!result && (*error != NULL) && ((*error)->code == (gint) NET_CLIENT_ERROR_POP_SERVER_ERR)) {
+			(*error)->code = (gint) NET_CLIENT_ERROR_POP_AUTHFAIL;
 		}
 	}
 
@@ -527,8 +548,7 @@ net_client_pop_auth_plain(NetClientPop *client, const gchar *user, const gchar *
 		if (result) {
 			result = net_client_pop_execute(client, "%s", NULL, error, base64_buf);
 		}
-		memset(base64_buf, 0, strlen(base64_buf));
-		g_free(base64_buf);
+		net_client_free_authstr(base64_buf);
 	} else {
 		result = FALSE;
 	}
@@ -548,13 +568,11 @@ net_client_pop_auth_login(NetClientPop *client, const gchar *user, const gchar *
 
 		base64_buf = g_base64_encode((const guchar *) user, strlen(user));
 		result = net_client_pop_execute_sasl(client, "%s", NULL, error, base64_buf);
-		memset(base64_buf, 0, strlen(base64_buf));
-		g_free(base64_buf);
+		net_client_free_authstr(base64_buf);
 		if (result) {
 			base64_buf = g_base64_encode((const guchar *) passwd, strlen(passwd));
 			result = net_client_pop_execute(client, "%s", NULL, error, base64_buf);
-			memset(base64_buf, 0, strlen(base64_buf));
-			g_free(base64_buf);
+			net_client_free_authstr(base64_buf);
 		}
 	}
 
@@ -585,11 +603,9 @@ net_client_pop_auth_apop(NetClientPop *client, const gchar *user, const gchar *p
 
 	auth_buf = g_strconcat(client->priv->apop_banner, passwd, NULL);
 	md5_buf = g_compute_checksum_for_string(G_CHECKSUM_MD5, auth_buf, -1);
-	memset(auth_buf, 0, strlen(auth_buf));
-	g_free(auth_buf);
+	net_client_free_authstr(auth_buf);
 	result = net_client_pop_execute(client, "APOP %s %s", NULL, error, user, md5_buf);
-	memset(md5_buf, 0, strlen(md5_buf));
-	g_free(md5_buf);
+	net_client_free_authstr(md5_buf);
 
 	return result;
 }
@@ -607,8 +623,7 @@ net_client_pop_auth_cram(NetClientPop *client, GChecksumType chksum_type, const
 		auth_buf = net_client_cram_calc(challenge, chksum_type, user, passwd);
 		if (auth_buf != NULL) {
 			result = net_client_pop_execute(client, "%s", NULL, error, auth_buf);
-			memset(auth_buf, 0, strlen(auth_buf));
-			g_free(auth_buf);
+			net_client_free_authstr(auth_buf);
 		} else {
 			result = FALSE;
 		}
