@@ -49,7 +49,7 @@ typedef struct {
 #define SMTP_DATA_BUF_SIZE			8192U
 
 
-G_DEFINE_TYPE(NetClientSmtp, net_client_smtp, NET_CLIENT_TYPE)
+G_DEFINE_TYPE_WITH_PRIVATE(NetClientSmtp, net_client_smtp, NET_CLIENT_TYPE)
 
 
 static void net_client_smtp_finalise(GObject *object);
@@ -157,16 +157,28 @@ net_client_smtp_connect(NetClientSmtp *client, gchar **greeting, GError **error)
 
 		auth_data = NULL;
 		need_pwd = (auth_supported & NET_CLIENT_SMTP_AUTH_NO_PWD) == 0U;
-		g_debug("emit 'auth' signal for client %p", client);
+		g_debug("emit 'auth' signal for client %p, need pwd %d", client, need_pwd);
 		g_signal_emit_by_name(client, "auth", need_pwd, &auth_data);
 		if ((auth_data != NULL) && (auth_data[0] != NULL)) {
 			result = net_client_smtp_auth(client, auth_data[0], auth_data[1], auth_supported, error);
-			memset(auth_data[0], 0, strlen(auth_data[0]));
-			if (auth_data[1] != NULL) {
-				memset(auth_data[1], 0, strlen(auth_data[1]));
+			net_client_free_authstr(auth_data[0]);
+			net_client_free_authstr(auth_data[1]);
+
+			/* if passwordless authentication failed, try again with password */
+			if (!result && !need_pwd) {
+				g_debug("passwordless authentication failed, retry w/ password: emit 'auth' signal for client %p", client);
+				g_clear_error(error);
+				g_free(auth_data);
+				g_signal_emit_by_name(client, "auth", TRUE, &auth_data);
+				if ((auth_data != NULL) && (auth_data[0] != NULL)) {
+					result = net_client_smtp_auth(client, auth_data[0], auth_data[1],
+						auth_supported & ~NET_CLIENT_SMTP_AUTH_NO_PWD, error);
+					net_client_free_authstr(auth_data[0]);
+					net_client_free_authstr(auth_data[1]);
+				}
 			}
 		}
-		g_strfreev(auth_data);
+		g_free(auth_data);
 	}
 
 	return result;
@@ -344,7 +356,7 @@ net_client_smtp_class_init(NetClientSmtpClass *klass)
 static void
 net_client_smtp_init(NetClientSmtp *self)
 {
-	self->priv = g_new0(NetClientSmtpPrivate, 1U);
+	self->priv = net_client_smtp_get_instance_private(self);	/*lint !e9079 (MISRA C:2012 Rule 11.5) intended use */
 	self->priv->auth_allowed[0] = NET_CLIENT_SMTP_AUTH_ALL;
 	self->priv->auth_allowed[1] = NET_CLIENT_SMTP_AUTH_SAFE;
 }
@@ -362,7 +374,6 @@ net_client_smtp_finalise(GObject *object)
 		(void) net_client_execute(NET_CLIENT(client), NULL, "QUIT", NULL);
 	}
 
-	g_free(client->priv);
 	(*parent_class->finalize)(object);
 }
 
@@ -394,7 +405,10 @@ net_client_smtp_auth(NetClientSmtp *client, const gchar *user, const gchar *pass
 		auth_mask = client->priv->auth_allowed[1] & auth_supported;
 	}
 
-	if (((auth_mask & NET_CLIENT_SMTP_AUTH_NO_PWD) == 0U) && (passwd == NULL)) {
+	if (auth_mask == 0U) {
+		g_set_error(error, NET_CLIENT_SMTP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_SMTP_NO_AUTH,
+			_("no suitable authentication mechanism"));
+	} else if (((auth_mask & NET_CLIENT_SMTP_AUTH_NO_PWD) == 0U) && (passwd == NULL)) {
 		g_set_error(error, NET_CLIENT_SMTP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_SMTP_NO_AUTH, _("password required"));
 	} else {
 		/* first try authentication methods w/o password, then safe ones, and finally the plain-text methods */
@@ -409,8 +423,7 @@ net_client_smtp_auth(NetClientSmtp *client, const gchar *user, const gchar *pass
 		} else if ((auth_mask & NET_CLIENT_SMTP_AUTH_LOGIN) != 0U) {
 			result = net_client_smtp_auth_login(client, user, passwd, error);
 		} else {
-			g_set_error(error, NET_CLIENT_SMTP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_SMTP_NO_AUTH,
-				_("no suitable authentication mechanism"));
+			g_assert_not_reached();
 		}
 	}
 
@@ -427,8 +440,7 @@ net_client_smtp_auth_plain(NetClientSmtp *client, const gchar *user, const gchar
 	base64_buf = net_client_auth_plain_calc(user, passwd);
 	if (base64_buf != NULL) {
 		result = net_client_smtp_execute(client, "AUTH PLAIN %s", NULL, error, base64_buf);
-		memset(base64_buf, 0, strlen(base64_buf));
-		g_free(base64_buf);
+		net_client_free_authstr(base64_buf);
 	} else {
 		result = FALSE;
 	}
@@ -445,13 +457,11 @@ net_client_smtp_auth_login(NetClientSmtp *client, const gchar *user, const gchar
 
 	base64_buf = g_base64_encode((const guchar *) user, strlen(user));
 	result = net_client_smtp_execute(client, "AUTH LOGIN %s", NULL, error, base64_buf);
-	memset(base64_buf, 0, strlen(base64_buf));
-	g_free(base64_buf);
+	net_client_free_authstr(base64_buf);
 	if (result) {
 		base64_buf = g_base64_encode((const guchar *) passwd, strlen(passwd));
 		result = net_client_smtp_execute(client, "%s", NULL, error, base64_buf);
-		memset(base64_buf, 0, strlen(base64_buf));
-		g_free(base64_buf);
+		net_client_free_authstr(base64_buf);
 	}
 
 	return result;
@@ -471,8 +481,7 @@ net_client_smtp_auth_cram(NetClientSmtp *client, GChecksumType chksum_type, cons
 		auth_buf = net_client_cram_calc(challenge, chksum_type, user, passwd);
 		if (auth_buf != NULL) {
 			result = net_client_smtp_execute(client, "%s", NULL, error, auth_buf);
-			memset(auth_buf, 0, strlen(auth_buf));
-			g_free(auth_buf);
+			net_client_free_authstr(auth_buf);
 		} else {
 			result = FALSE;
 		}
@@ -692,8 +701,13 @@ net_client_smtp_eval_rescode(gint res_code, const gchar *reply, GError **error)
 		result = FALSE;
 		break;
 	case 5:
-		g_set_error(error, NET_CLIENT_SMTP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_SMTP_PERMANENT,
-			_("permanent error %d: %s"), res_code, reply);
+		if ((res_code == 534) || (res_code == 535)) {
+			g_set_error(error, NET_CLIENT_SMTP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_SMTP_AUTHFAIL,
+				_("authentication failure %d: %s"), res_code, reply);
+		} else {
+			g_set_error(error, NET_CLIENT_SMTP_ERROR_QUARK, (gint) NET_CLIENT_ERROR_SMTP_PERMANENT,
+				_("permanent error %d: %s"), res_code, reply);
+		}
 		result = FALSE;
 		break;
 	default:
