@@ -16,9 +16,25 @@
 #include <stdio.h>
 #include <glib/gi18n.h>
 #include <gnutls/x509.h>
+#include "net-client-utils.h"
 #include "net-client.h"
 
 
+/*
+ * Stacking of the streams:
+ * Input channel:
+ *   GSocketClient *sock
+ *      GSocketConnection *plain_conn
+ *         GIOStream *tls_conn -- optional
+ *            GInputStream *comp_istream -- optional
+ *               GDataInputStream *istream
+ *
+ * Output channel:
+ *   GSocketClient *sock
+ *      GSocketConnection *plain_conn
+ *         GIOStream *tls_conn -- optional
+ *            GOutputStream *ostream -- optionally compressed
+ */
 struct _NetClientPrivate {
 	gchar *host_and_port;
 	guint16 default_port;
@@ -30,13 +46,17 @@ struct _NetClientPrivate {
 	GDataInputStream *istream;
 	GOutputStream *ostream;
 	GTlsCertificate *certificate;
+
+	GZlibCompressor *comp;
+	GZlibDecompressor *decomp;
+	GInputStream *comp_istream;
 };
 
 
 static guint signals[3];
 
 
-G_DEFINE_TYPE(NetClient, net_client, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(NetClient, net_client, G_TYPE_OBJECT)
 
 
 static void net_client_finalise(GObject *object);
@@ -133,7 +153,27 @@ void
 net_client_shutdown(const NetClient *client)
 {
 	if (NET_IS_CLIENT(client)) {
-		/* note: we must unref the GDataInputStream, but *not* the GOutputStream! */
+		/* Note: we must unref the GDataInputStream, but the GOutputStream only if compression is active! */
+		if (client->priv->comp != NULL) {
+			/* Note: for some strange reason, GIO decides to send a 0x03 0x00 sequence when closing a compressed connection, before
+			 * sending the usual FIN, ACK TCP reply packet.  As the remote server does not expect the former (the connection has
+			 * already been closed on its side), it replies with with a RST TCP packet.  Unref'ing client->priv->ostream and
+			 * client->priv->comp /after/ all other components of the connection fixes the issue for unencrypted connections, but
+			 * throws a critical error for TLS.  Observed with gio 2.48.2 and 2.50.3, no idea how it can be fixed.
+			 * See also https://bugzilla.gnome.org/show_bug.cgi?id=795985. */
+			if (client->priv->ostream != NULL) {
+				g_object_unref(G_OBJECT(client->priv->ostream));
+			}
+			g_object_unref(G_OBJECT(client->priv->comp));
+		}
+		if (client->priv->decomp != NULL) {
+			g_object_unref(G_OBJECT(client->priv->decomp));
+			client->priv->decomp = NULL;
+		}
+		if (client->priv->comp_istream!= NULL) {
+			g_object_unref(G_OBJECT(client->priv->comp_istream));
+			client->priv->comp_istream = NULL;
+		}
 		if (client->priv->istream != NULL) {
 			g_object_unref(G_OBJECT(client->priv->istream));
 			client->priv->istream = NULL;
@@ -355,29 +395,33 @@ net_client_set_cert_from_pem(NetClient *client, const gchar *pem_data, GError **
 			} else {
 				res = gnutls_x509_privkey_import2(key, &data, GNUTLS_X509_FMT_PEM, NULL, 0);
 				if (res == GNUTLS_E_DECRYPTION_FAILED) {
-					size_t der_size;
-					guint8 *der_data;
+					size_t dn_size;
+					gchar *dn_str;
 
-					/* determine cert buffer size requirements */
-					der_size = 0U;
-					(void) gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, NULL, &der_size);
-					der_data = g_malloc(der_size);		/*lint !e9079 (MISRA C:2012 Rule 11.5) */
+					/* determine dn string buffer size requirements */
+					dn_size = 0U;
+					(void) gnutls_x509_crt_get_dn(cert, NULL, &dn_size);
+					dn_str = g_malloc0(dn_size + 1U);		/*lint !e9079 (MISRA C:2012 Rule 11.5) */
 
-					res = gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, der_data, &der_size);
+					res = gnutls_x509_crt_get_dn(cert, dn_str, &dn_size);
 					if (res == GNUTLS_E_SUCCESS) {
-						GByteArray *cert_der;
 						gchar *key_pass = NULL;
 
-						cert_der = g_byte_array_new_take(der_data, der_size);
+						if (!g_utf8_validate(dn_str, -1, NULL)) {
+							gchar *buf;
+
+							buf = g_locale_to_utf8(dn_str, -1, NULL, NULL, NULL);
+							g_free(dn_str);
+							dn_str = buf;
+						}
 						g_debug("emit 'cert-pass' signal for client %p", client);
-						g_signal_emit(client, signals[2], 0, cert_der, &key_pass);
-						g_byte_array_unref(cert_der);
+						g_signal_emit(client, signals[2], 0, dn_str, &key_pass);
 						if (key_pass != NULL) {
 							res = gnutls_x509_privkey_import2(key, &data, GNUTLS_X509_FMT_PEM, key_pass, 0);
-							memset(key_pass, 0, strlen(key_pass));
-							g_free(key_pass);
+							net_client_free_authstr(key_pass);
 						}
 					}
+					g_free(dn_str);
 				}
 
 				/* on success, set the certificate using the unencrypted key */
@@ -406,7 +450,7 @@ net_client_set_cert_from_pem(NetClient *client, const gchar *pem_data, GError **
 				gnutls_x509_privkey_deinit(key);
 
 				if (res != GNUTLS_E_SUCCESS) {
-					g_set_error(error, NET_CLIENT_ERROR_QUARK, (gint) NET_CLIENT_ERROR_GNUTLS, _("error loading key: %s"),
+					g_set_error(error, NET_CLIENT_ERROR_QUARK, (gint) NET_CLIENT_ERROR_CERT_KEY_PASS, _("error loading key: %s"),
 						gnutls_strerror(res));
 				}
 			}
@@ -475,6 +519,45 @@ net_client_start_tls(NetClient *client, GError **error)
 }
 
 
+gboolean
+net_client_start_compression(NetClient *client, GError **error)
+{
+	gboolean result = FALSE;
+
+	g_return_val_if_fail(NET_IS_CLIENT(client), FALSE);
+
+	if (client->priv->plain_conn == NULL) {
+		g_set_error(error, NET_CLIENT_ERROR_QUARK, (gint) NET_CLIENT_ERROR_NOT_CONNECTED, _("not connected"));
+	} else if (client->priv->comp != NULL) {
+		g_set_error(error, NET_CLIENT_ERROR_QUARK, (gint) NET_CLIENT_ERROR_COMP_ACTIVE, _("connection is already compressed"));
+	} else {
+		client->priv->comp = g_zlib_compressor_new(G_ZLIB_COMPRESSOR_FORMAT_RAW, -1);
+		client->priv->decomp = g_zlib_decompressor_new(G_ZLIB_COMPRESSOR_FORMAT_RAW);
+
+		g_filter_input_stream_set_close_base_stream(G_FILTER_INPUT_STREAM(client->priv->istream), FALSE);
+		g_object_unref(client->priv->istream);
+
+		if (client->priv->tls_conn != NULL) {
+			client->priv->comp_istream =
+				g_converter_input_stream_new(g_io_stream_get_input_stream(G_IO_STREAM(client->priv->tls_conn)),
+					G_CONVERTER(client->priv->decomp));
+		} else {
+			client->priv->comp_istream =
+				g_converter_input_stream_new(g_io_stream_get_input_stream(G_IO_STREAM(client->priv->plain_conn)),
+					G_CONVERTER(client->priv->decomp));
+		}
+		client->priv->istream = g_data_input_stream_new(client->priv->comp_istream);
+		g_data_input_stream_set_newline_type(client->priv->istream, G_DATA_STREAM_NEWLINE_TYPE_CR_LF);
+
+		client->priv->ostream = g_converter_output_stream_new(client->priv->ostream, G_CONVERTER(client->priv->comp));
+		result = TRUE;
+		g_debug("connection is compressed");
+	}
+
+	return result;
+}
+
+
 gboolean
 net_client_set_timeout(NetClient *client, guint timeout_secs)
 {
@@ -485,6 +568,25 @@ net_client_set_timeout(NetClient *client, guint timeout_secs)
 }
 
 
+GSocket *
+net_client_get_socket(NetClient *client)
+{
+	g_return_val_if_fail(NET_IS_CLIENT(client) && (client->priv->plain_conn != NULL), NULL);
+
+	return g_socket_connection_get_socket(client->priv->plain_conn);
+}
+
+
+gboolean
+net_client_can_read(NetClient *client)
+{
+	g_return_val_if_fail(NET_IS_CLIENT(client) && (client->priv->plain_conn != NULL) && (client->priv->istream != NULL), FALSE);
+
+	return (g_socket_condition_check(g_socket_connection_get_socket(client->priv->plain_conn), G_IO_IN) != 0) ||
+		(g_buffered_input_stream_get_available(G_BUFFERED_INPUT_STREAM(client->priv->istream)) > 0U);
+}
+
+
 /* == local functions =========================================================================================================== */
 
 static void
@@ -492,20 +594,19 @@ net_client_class_init(NetClientClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
-	g_type_class_add_private(klass, sizeof(NetClientPrivate));
 	gobject_class->finalize = net_client_finalise;
 	signals[0] = g_signal_new("cert-check", NET_CLIENT_TYPE, G_SIGNAL_RUN_LAST, 0U, NULL, NULL, NULL, G_TYPE_BOOLEAN, 2U,
 		G_TYPE_TLS_CERTIFICATE, G_TYPE_TLS_CERTIFICATE_FLAGS);
 	signals[1] = g_signal_new("auth", NET_CLIENT_TYPE, G_SIGNAL_RUN_LAST, 0U, NULL, NULL, NULL, G_TYPE_STRV, 1U, G_TYPE_BOOLEAN);
 	signals[2] = g_signal_new("cert-pass", NET_CLIENT_TYPE, G_SIGNAL_RUN_LAST, 0U, NULL, NULL, NULL, G_TYPE_STRING, 1U,
-		G_TYPE_BYTE_ARRAY);
+		G_TYPE_STRING);
 }
 
 
 static void
 net_client_init(NetClient *self)
 {
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, NET_CLIENT_TYPE, NetClientPrivate);
+	self->priv = net_client_get_instance_private(self);		/*lint !e9079 (MISRA C:2012 Rule 11.5) intended use of this function */
 	self->priv->sock = g_socket_client_new();
 	if (self->priv->sock != NULL) {
 		g_socket_client_set_timeout(self->priv->sock, 180U);
