@@ -92,8 +92,7 @@ bab_config_init(const gchar * group, const gchar * value, gpointer data)
         contacts_app.address_book_list =
             g_list_append(contacts_app.address_book_list, address_book);
 
-        if (contacts_app.default_address_book_prefix
-            && strcmp(group,
+        if (g_strcmp0(group,
                       contacts_app.default_address_book_prefix) == 0)
             contacts_app.default_address_book = address_book;
     }
@@ -176,7 +175,7 @@ bab_set_address_book(LibBalsaAddressBook * ab,
          libbalsa_address_book_load(ab, filter,
                                     (LibBalsaAddressBookLoadFunc)
                                     bab_load_cb, model)) != LBABERR_OK) {
-        printf("error loading address book from %s: %d\n", ab->name,
+        printf("error loading address book from %s: %d\n", libbalsa_address_book_get_name(ab),
                ab_err);
     }
 
@@ -191,7 +190,7 @@ bab_window_set_title(LibBalsaAddressBook * address_book)
 
     if (LIBBALSA_IS_ADDRESS_BOOK_VCARD(address_book))
         type = "vCard";
-    else if (LIBBALSA_IS_ADDRESS_BOOK_EXTERN(address_book))
+    else if (LIBBALSA_IS_ADDRESS_BOOK_EXTERNQ(address_book))
         type = "External query";
     else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book))
         type = "LDIF";
@@ -209,7 +208,7 @@ bab_window_set_title(LibBalsaAddressBook * address_book)
 #endif /* HAVE_RUBRICA */
 
     title =
-        g_strconcat(type, _(" address book: "), address_book->name, NULL);
+        g_strconcat(type, _(" address book: "), libbalsa_address_book_get_name(address_book), NULL);
     gtk_window_set_title(contacts_app.window, title);
     g_free(title);
 }
@@ -226,7 +225,7 @@ address_book_change_state(GSimpleAction * action,
     value = g_variant_get_string(state, NULL);
     for (l = contacts_app.address_book_list; l; l = l->next) {
         address_book = l->data;
-        if (address_book && strcmp(value, address_book->name) == 0)
+        if (address_book && strcmp(value, libbalsa_address_book_get_name(address_book)) == 0)
             break;
     }
 
@@ -277,7 +276,7 @@ set_address_book_menu_items(void)
 
         g_string_append(string, "<attribute name='label'>");
         g_string_append_printf(string, "_%d:%s", ++pos,
-                               address_book->name);
+                               libbalsa_address_book_get_name(address_book));
         g_string_append(string, "</attribute>");
 
         g_string_append(string, "<attribute name='action'>");
@@ -285,7 +284,7 @@ set_address_book_menu_items(void)
         g_string_append(string, "</attribute>");
 
         g_string_append(string, "<attribute name='target'>");
-        g_string_append(string, address_book->name);
+        g_string_append(string, libbalsa_address_book_get_name(address_book));
         g_string_append(string, "</attribute>");
 
         g_string_append(string, "<attribute name='accel'>");
@@ -338,17 +337,20 @@ get_unused_group(const gchar * prefix)
 static void
 address_book_change(LibBalsaAddressBook * address_book, gboolean append)
 {
+    const gchar *config_prefix;
     gchar *group;
 
-    if (append)
+    if (append) {
         contacts_app.address_book_list =
             g_list_append(contacts_app.address_book_list, address_book);
+    }
 
     set_address_book_menu_items();
     bab_window_set_title(address_book);
 
-    group = address_book->config_prefix ?
-        g_strdup(address_book->config_prefix) :
+    config_prefix = libbalsa_address_book_get_config_prefix(address_book);
+    group = config_prefix != NULL ?
+        g_strdup(config_prefix) :
         get_unused_group(ADDRESS_BOOK_SECTION_PREFIX);
     libbalsa_address_book_save_config(address_book, group);
     g_free(group);
@@ -367,12 +369,12 @@ file_new_vcard_activated(GSimpleAction * action,
 }
 
 static void
-file_new_extern_activated(GSimpleAction * action,
+file_new_externq_activated(GSimpleAction * action,
                           GVariant      * state,
                           gpointer        user_data)
 {
     balsa_address_book_config_new_from_type
-        (LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN, address_book_change,
+        (LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ, address_book_change,
          contacts_app.window);
 }
 
@@ -442,15 +444,17 @@ file_delete_activated(GSimpleAction * action,
                       gpointer        user_data)
 {
     LibBalsaAddressBook *address_book;
+    const gchar *config_prefix;
     GList *list;
 
     if (!(address_book = contacts_app.address_book)
         || !g_list_next(contacts_app.address_book_list))
         return;
 
-    if (address_book->config_prefix) {
-        libbalsa_conf_remove_group(address_book->config_prefix);
-        libbalsa_conf_private_remove_group(address_book->config_prefix);
+    config_prefix = libbalsa_address_book_get_config_prefix(address_book);
+    if (config_prefix != NULL) {
+        libbalsa_conf_remove_group(config_prefix);
+        libbalsa_conf_private_remove_group(config_prefix);
         libbalsa_conf_queue_sync();
     }
 
@@ -559,7 +563,7 @@ get_main_menu(GtkApplication * application)
 {
     static GActionEntry win_entries[] = {
         {"file-new-vcard",      file_new_vcard_activated},
-        {"file-new-external",   file_new_extern_activated},
+        {"file-new-external",   file_new_externq_activated},
         {"file-new-ldif",       file_new_ldif_activated},
 #if ENABLE_LDAP
         {"file-new-ldap",       file_new_ldap_activated},
@@ -580,13 +584,11 @@ get_main_menu(GtkApplication * application)
             address_book_change_state},
     };
     GtkBuilder *builder;
-    gchar *ui_file;
+    const gchar resource_path[] = "/org/desktop/BalsaAb/ab-main.ui";
     GError *err = NULL;
 
     builder = gtk_builder_new();
-    ui_file = g_build_filename(BALSA_DATA_PREFIX, "ui", "ab-main.ui",
-                               NULL);
-    if (gtk_builder_add_from_file(builder, ui_file, &err)) {
+    if (gtk_builder_add_from_resource(builder, resource_path, &err)) {
         gtk_application_set_menubar(application,
                                     G_MENU_MODEL(gtk_builder_get_object
                                                  (builder, "menubar")));
@@ -596,7 +598,6 @@ get_main_menu(GtkApplication * application)
         g_print("%s error: %s\n", __func__, err->message);
         g_error_free(err);
     }
-    g_free(ui_file);
     g_object_unref(builder);
 
     g_action_map_add_action_entries(G_ACTION_MAP(contacts_app.window),
@@ -1011,7 +1012,7 @@ static void
 bab_init(void)
 {
     LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
-    LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN;
+    LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ;
     LIBBALSA_TYPE_ADDRESS_BOOK_LDIF;
 #if ENABLE_LDAP
     LIBBALSA_TYPE_ADDRESS_BOOK_LDAP;
@@ -1025,12 +1026,6 @@ bab_init(void)
     memset(&contacts_app, 0, sizeof(contacts_app));
 }
 
-static void
-information_real(void)
-{
-    /* FIXME */
-}
-
 static void
 bab_set_intial_address_book(LibBalsaAddressBook * ab,
                             GtkWidget           * window)
@@ -1039,10 +1034,9 @@ bab_set_intial_address_book(LibBalsaAddressBook * ab,
 
     action =
         g_action_map_lookup_action(G_ACTION_MAP(window), "address-book");
-    g_action_change_state(action, g_variant_new_string(ab->name));
+    g_action_change_state(action, g_variant_new_string(libbalsa_address_book_get_name(ab)));
 }
 
-#if GTK_CHECK_VERSION(3, 12, 0)
 GtkDialogFlags
 libbalsa_dialog_flags(void)
 {
@@ -1060,7 +1054,44 @@ libbalsa_dialog_flags(void)
 	}
 	return dialog_flags;
 }
-#endif
+
+/*
+ * Set up GNotification for libbalsa
+ */
+
+#define BALSA_AB_NOTIFICATION "balsa-ab-notification"
+
+static void
+balsa_ab_notification_notify_cb(GNotification *notification, GApplication *application)
+{
+    gboolean send;
+
+    send = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(notification), "send"));
+    if (send) {
+        g_application_send_notification(application,
+                                        BALSA_AB_NOTIFICATION, notification);
+    } else {
+        g_application_withdraw_notification(application, BALSA_AB_NOTIFICATION);
+    }
+}
+
+static void
+balsa_ab_notification_shutdown_cb(GApplication *application, GNotification *notification)
+{
+    g_object_unref(notification);
+}
+
+static void
+balsa_ab_setup_libbalsa_notification(GApplication *application)
+{
+    GNotification *notification;
+
+    notification = libbalsa_notification_new("BalsaAb");
+    g_signal_connect(notification, "notify",
+                     G_CALLBACK(balsa_ab_notification_notify_cb), application);
+    g_signal_connect(application, "shutdown",
+                     G_CALLBACK(balsa_ab_notification_shutdown_cb), notification);
+}
 
 int
 main(int argc, char *argv[])
@@ -1088,7 +1119,7 @@ main(int argc, char *argv[])
 #endif
 
     bab_init();
-    libbalsa_real_information_func = (LibBalsaInformationFunc)information_real;
+    balsa_ab_setup_libbalsa_notification((GApplication *) application);
     g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 
     /* load address book data */
