@@ -23,6 +23,7 @@
 #include "ab-window.h"
 
 #include <string.h>
+#include <stdlib.h>
 #include <glib/gi18n.h>
 #include "address-view.h"
 #include "balsa-app.h"
@@ -116,7 +117,6 @@ balsa_ab_window_new(gboolean composing, GtkWindow* parent)
     GtkWidget *ret;
     BalsaAbWindow *ab;
     gint use_headerbar = 1;
-#if GTK_CHECK_VERSION(3, 12, 0)
 	const gchar *dialog_env;
 
     /* header bar configuration */
@@ -124,7 +124,6 @@ balsa_ab_window_new(gboolean composing, GtkWindow* parent)
 	if ((dialog_env != NULL) && (atoi(dialog_env) == 0)) {
 		use_headerbar = 0;
 	}
-#endif
 
     ret = g_object_new(BALSA_TYPE_AB_WINDOW,
                        "use-header-bar", use_headerbar,
@@ -216,7 +215,7 @@ balsa_ab_window_load_books(BalsaAbWindow * ab)
         LibBalsaAddressBook *address_book = ab_list->data;
 
         gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(ab->combo_box),
-                                  address_book->name);
+                                       libbalsa_address_book_get_name(address_book));
 
         if (ab->current_address_book == NULL)
             ab->current_address_book = address_book;
@@ -402,7 +401,7 @@ balsa_ab_window_init(BalsaAbWindow *ab)
     if(ab->current_address_book)
 	gtk_toggle_button_set_active(
 	    GTK_TOGGLE_BUTTON(ab->dist_address_mode_radio),
-	    ab->current_address_book->dist_list_mode);
+	    libbalsa_address_book_get_dist_list_mode(ab->current_address_book));
 
     /* Pack them into a box  */
     box2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 1);
@@ -686,7 +685,7 @@ balsa_ab_window_set_title(BalsaAbWindow *ab)
 
     if (LIBBALSA_IS_ADDRESS_BOOK_VCARD(address_book))
         type = "vCard";
-    else if (LIBBALSA_IS_ADDRESS_BOOK_EXTERN(address_book))
+    else if (LIBBALSA_IS_ADDRESS_BOOK_EXTERNQ(address_book))
         type = "External query";
     else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book))
         type = "LDIF";
@@ -704,7 +703,8 @@ balsa_ab_window_set_title(BalsaAbWindow *ab)
 #endif
 
     title =
-        g_strconcat(type, _(" address book: "), address_book->name, NULL);
+        g_strconcat(type, _(" address book: "),
+                    libbalsa_address_book_get_name(address_book), NULL);
     gtk_window_set_title(GTK_WINDOW(ab), title);
     g_free(title);
 }
@@ -733,7 +733,8 @@ balsa_ab_window_load(BalsaAbWindow *ab)
         balsa_information_parented(GTK_WINDOW(ab),
 				   LIBBALSA_INFORMATION_ERROR,
 				   _("Error opening address book “%s”:\n%s"),
-				   ab->current_address_book->name, desc);
+				   libbalsa_address_book_get_name(ab->current_address_book),
+                                   desc);
     }
     balsa_ab_window_set_title(ab);
 }
@@ -861,7 +862,7 @@ balsa_ab_window_dist_mode_toggled(GtkWidget * w, BalsaAbWindow *ab)
     active = gtk_toggle_button_get_active
 	(GTK_TOGGLE_BUTTON(ab->single_address_mode_radio));
 
-    ab->current_address_book->dist_list_mode = !active;
+    libbalsa_address_book_set_dist_list_mode(ab->current_address_book, !active);
 
     balsa_ab_window_load(ab);
 }
@@ -882,7 +883,7 @@ balsa_ab_window_menu_changed(GtkWidget * widget, BalsaAbWindow *ab)
 
     g_signal_handler_block(G_OBJECT(ab->single_address_mode_radio),
 			   ab->toggle_handler_id);
-    if ( ab->current_address_book->dist_list_mode )
+    if (libbalsa_address_book_get_dist_list_mode(ab->current_address_book))
 	gtk_toggle_button_set_active(
 	    GTK_TOGGLE_BUTTON(ab->dist_address_mode_radio), TRUE);
     else
