@@ -47,10 +47,10 @@ struct _AddressBookConfig {
 		GtkWidget *load;
 		GtkWidget *save;
 	} externq;
-	
+
 #ifdef ENABLE_LDAP
 	struct {
-	    GtkWidget *host_name;
+	    GtkWidget *host;
 	    GtkWidget *base_dn;
 	    GtkWidget *bind_dn;
 	    GtkWidget *book_dn;
@@ -111,7 +111,8 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book,
     abc = g_object_get_data(G_OBJECT(address_book), "balsa-abc");
     if (abc) {
         /* Only one dialog per address book. */
-        gtk_window_present(GTK_WINDOW(abc->window));
+        gtk_window_present_with_time(GTK_WINDOW(abc->window),
+                                     gtk_get_current_event_time());
         return;
     }
 
@@ -199,10 +200,10 @@ add_radio_buttons(GtkWidget * grid, gint row, AddressBookConfig * abc)
     gtk_grid_attach(GTK_GRID(grid), abc->never, 0, row, 2, 1);
 
     button = abc->as_i_type;
-    if (abc->address_book) {
-        if (!abc->address_book->expand_aliases)
+    if (abc->address_book != NULL) {
+        if (!libbalsa_address_book_get_expand_aliases(abc->address_book))
             button = abc->never;
-        else if (abc->address_book->is_expensive)
+        else if (libbalsa_address_book_get_is_expensive(abc->address_book))
             button = abc->on_request;
     }
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
@@ -221,10 +222,10 @@ create_local_dialog(AddressBookConfig * abc, const gchar * type)
     GtkSizeGroup *size_group;
 
     ab = abc->address_book;
-    if (ab) {
+    if (ab != NULL) {
         title = g_strdup_printf(_("Modify %s Address Book"), type);
         action = _("_Apply");
-        name = ab->name;
+        name = libbalsa_address_book_get_name(ab);
     } else {
         title = g_strdup_printf(_("Add %s Address Book"), type);
         action = _("_Add");
@@ -254,7 +255,8 @@ create_local_dialog(AddressBookConfig * abc, const gchar * type)
     add_radio_buttons(grid, 1, abc);
 
     if (ab) {
-        const gchar *path = LIBBALSA_ADDRESS_BOOK_TEXT(ab)->path;
+        LibBalsaAddressBookText *abt = (LibBalsaAddressBookText *) ab;
+        const gchar *path = libbalsa_address_book_text_get_path(abt);
         gchar *folder;
         gchar *utf8name;
 
@@ -290,7 +292,7 @@ create_dialog_from_type(AddressBookConfig * abc)
 {
     if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD) {
         return create_vcard_dialog(abc);
-    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ) {
         return create_externq_dialog(abc);
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
         return create_ldif_dialog(abc);
@@ -373,7 +375,7 @@ create_osmo_dialog(AddressBookConfig *abc)
     if (ab) {
         title = g_strdup_printf(_("Modify Osmo Address Book"));
         action = _("_Apply");
-        name = ab->name;
+        name = libbalsa_address_book_get_name(ab);
     } else {
         title = g_strdup_printf(_("Add Osmo Address Book"));
         action = _("_Add");
@@ -415,17 +417,19 @@ create_externq_dialog(AddressBookConfig * abc)
     GtkWidget *dialog;
     GtkWidget *grid;
     GtkWidget *label;
-    LibBalsaAddressBookExtern* ab;
+    LibBalsaAddressBookExternq* ab_externq;
 
-    ab = (LibBalsaAddressBookExtern*)abc->address_book; /* may be NULL */
+    ab_externq = (LibBalsaAddressBookExternq*)abc->address_book; /* may be NULL */
     grid = libbalsa_create_grid();
     gtk_container_set_border_width(GTK_CONTAINER(grid), 5);
 
     /* mailbox name */
 
     label = libbalsa_create_grid_label(_("A_ddress Book Name:"), grid, 0);
-    abc->name_entry = libbalsa_create_grid_entry(grid, NULL, NULL, 0, 
-				   ab ? abc->address_book->name : NULL, 
+    abc->name_entry =
+        libbalsa_create_grid_entry(grid, NULL, NULL, 0,
+				   ab_externq != NULL ?
+                                   libbalsa_address_book_get_name(LIBBALSA_ADDRESS_BOOK(ab_externq)) : NULL,
 				   label);
 
     label = gtk_label_new(_("Load program location:"));
@@ -456,13 +460,13 @@ create_externq_dialog(AddressBookConfig * abc)
 
     add_radio_buttons(grid, 3, abc);
 
-    if (ab) {
+    if (ab_externq != NULL) {
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER
                                       (abc->ab_specific.externq.load),
-                                      ab->load);
+                                      libbalsa_address_book_externq_get_load(ab_externq));
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER
                                       (abc->ab_specific.externq.save),
-                                      ab->save);
+                                      libbalsa_address_book_externq_get_save(ab_externq));
     }
 
     dialog = create_generic_dialog(abc, "Extern");
@@ -490,39 +494,41 @@ create_ldap_dialog(AddressBookConfig * abc)
     /* mailbox name */
 
     label = libbalsa_create_grid_label(_("A_ddress Book Name:"), grid, 0);
-    abc->name_entry = libbalsa_create_grid_entry(grid, NULL, NULL, 0, 
-				   ab ? abc->address_book->name : name, 
+    abc->name_entry =
+        libbalsa_create_grid_entry(grid, NULL, NULL, 0, 
+				   ab != NULL ?
+                                   libbalsa_address_book_get_name(LIBBALSA_ADDRESS_BOOK(ab)) : name,
 				   label);
 
     label = libbalsa_create_grid_label(_("_Host Name"), grid, 1);
-    abc->ab_specific.ldap.host_name = 
+    abc->ab_specific.ldap.host = 
 	libbalsa_create_grid_entry(grid, NULL, NULL, 1, 
-		     ab ? ab->host : host, label);
+		     ab ? libbalsa_address_book_ldap_get_host(ab) : host, label);
 
     label = libbalsa_create_grid_label(_("Base Domain _Name"), grid, 2);
     abc->ab_specific.ldap.base_dn = 
 	libbalsa_create_grid_entry(grid, NULL, NULL, 2, 
-		     ab ? ab->base_dn : base, label);
+		     ab ? libbalsa_address_book_ldap_get_base_dn(ab) : base, label);
 
     label = libbalsa_create_grid_label(_("_User Name (Bind DN)"), grid, 3);
     abc->ab_specific.ldap.bind_dn = 
 	libbalsa_create_grid_entry(grid, NULL, NULL, 3, 
-		     ab ? ab->bind_dn : "", label);
+		     ab ? libbalsa_address_book_ldap_get_bind_dn(ab) : "", label);
 
     label = libbalsa_create_grid_label(_("_Password"), grid, 4);
     abc->ab_specific.ldap.passwd = 
 	libbalsa_create_grid_entry(grid, NULL, NULL, 4, 
-		     ab ? ab->passwd : "", label);
+		     ab ? libbalsa_address_book_ldap_get_passwd(ab) : "", label);
     gtk_entry_set_visibility(GTK_ENTRY(abc->ab_specific.ldap.passwd), FALSE);
 
     label = libbalsa_create_grid_label(_("_User Address Book DN"), grid, 5);
     abc->ab_specific.ldap.book_dn = 
 	libbalsa_create_grid_entry(grid, NULL, NULL, 5,
-		     ab ? ab->priv_book_dn : "", label);
+		     ab ? libbalsa_address_book_ldap_get_book_dn(ab) : "", label);
 
     abc->ab_specific.ldap.enable_tls =
 	libbalsa_create_grid_check(_("Enable _TLS"), grid, 6,
-		     ab ? ab->enable_tls : FALSE);
+		     ab ? libbalsa_address_book_ldap_get_enable_tls(ab) : FALSE);
 
     add_radio_buttons(grid, 7, abc);
 
@@ -557,8 +563,10 @@ create_gpe_dialog(AddressBookConfig * abc)
     /* mailbox name */
 
     label = libbalsa_create_grid_label(_("A_ddress Book Name:"), grid, 0);
-    abc->name_entry = libbalsa_create_grid_entry(grid, NULL, NULL, 0, 
-				   ab ? ab->name : _("GPE Address Book"), 
+    abc->name_entry =
+        libbalsa_create_grid_entry(grid, NULL, NULL, 0,
+				   ab != NULL ?  libbalsa_address_book_get_name(ab) :
+                                   _("GPE Address Book"),
 				   label);
 
     add_radio_buttons(grid, 1, abc);
@@ -638,7 +646,7 @@ handle_close(AddressBookConfig * abc)
                      GTK_WINDOW(abc->window),
                      ADDRESS_BOOK_CONFIG_PATH_FILE))
             return FALSE;
-    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ) {
         if (chooser_bad_path(GTK_FILE_CHOOSER(abc->ab_specific.externq.load),
                      GTK_WINDOW(abc->window),
                      ADDRESS_BOOK_CONFIG_PATH_LOAD))
@@ -698,7 +706,7 @@ create_book(AddressBookConfig * abc)
         if (path != NULL)
             address_book = libbalsa_address_book_vcard_new(name, path);
         g_free(path);
-    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ) {
 #define GET_FILENAME(chooser) \
   gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser))
         gchar *load = GET_FILENAME(abc->ab_specific.externq.load);
@@ -716,8 +724,8 @@ create_book(AddressBookConfig * abc)
         g_free(path);
 #ifdef ENABLE_LDAP
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDAP) {
-        const gchar *host_name =
-            gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.host_name));
+        const gchar *host =
+            gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.host));
         const gchar *base_dn =
             gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.base_dn));
         const gchar *bind_dn =
@@ -730,7 +738,7 @@ create_book(AddressBookConfig * abc)
             gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                          (abc->ab_specific.ldap.enable_tls));
         address_book =
-            libbalsa_address_book_ldap_new(name, host_name, base_dn,
+            libbalsa_address_book_ldap_new(name, host, base_dn,
                                            bind_dn, passwd, book_dn,
                                            enable_tls);
 #endif
@@ -754,11 +762,11 @@ create_book(AddressBookConfig * abc)
     } else
         g_assert_not_reached();
 
-    if (address_book) {
-        address_book->expand_aliases =
-            !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->never));
-        address_book->is_expensive =
-            gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->on_request));
+    if (address_book != NULL) {
+        libbalsa_address_book_set_expand_aliases(address_book,
+            !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->never)));
+        libbalsa_address_book_set_is_expensive(address_book,
+            gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->on_request)));
         abc->callback(address_book, TRUE);
     }
 
@@ -770,9 +778,8 @@ modify_book(AddressBookConfig * abc)
 {
     LibBalsaAddressBook *address_book = abc->address_book;
 
-    g_free(address_book->name);
-    address_book->name =
-        g_strdup(gtk_entry_get_text(GTK_ENTRY(abc->name_entry)));
+    libbalsa_address_book_set_name(address_book,
+        gtk_entry_get_text(GTK_ENTRY(abc->name_entry)));
 
     if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD
         || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF
@@ -785,12 +792,10 @@ modify_book(AddressBookConfig * abc)
         gchar *path =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(abc->window));
 
-        if (path) {
-            g_free(ab_text->path);
-            ab_text->path = path;
-        }
-    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
-        LibBalsaAddressBookExtern *externq;
+        if (path != NULL)
+            libbalsa_address_book_text_set_path(ab_text, path);
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ) {
+        LibBalsaAddressBookExternq *ab_externq;
         gchar *load =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER
                                            (abc->ab_specific.externq.load));
@@ -798,20 +803,20 @@ modify_book(AddressBookConfig * abc)
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER
                                            (abc->ab_specific.externq.save));
 
-        externq = LIBBALSA_ADDRESS_BOOK_EXTERN(address_book);
+        ab_externq = LIBBALSA_ADDRESS_BOOK_EXTERNQ(address_book);
         if (load) {
-            g_free(externq->load);
-            externq->load = load;;
+            libbalsa_address_book_externq_set_load(ab_externq, load);
+            g_free(load);
         }
         if (save) {
-            g_free(externq->save);
-            externq->save = save;
+            libbalsa_address_book_externq_set_save(ab_externq, save);
+            g_free(save);
         }
 #ifdef ENABLE_LDAP
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDAP) {
         LibBalsaAddressBookLdap *ldap;
-        const gchar *host_name =
-            gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.host_name));
+        const gchar *host =
+            gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.host));
         const gchar *base_dn =
             gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.base_dn));
         const gchar *bind_dn =
@@ -826,13 +831,13 @@ modify_book(AddressBookConfig * abc)
 
         ldap = LIBBALSA_ADDRESS_BOOK_LDAP(address_book);
 
-        g_free(ldap->host);     ldap->host = g_strdup(host_name);
-        g_free(ldap->base_dn);  ldap->base_dn = g_strdup(base_dn);
-        g_free(ldap->bind_dn);  ldap->bind_dn = g_strdup(bind_dn);
-        g_free(ldap->passwd);   ldap->passwd  = g_strdup(passwd);
-        g_free(ldap->priv_book_dn);
-        ldap->priv_book_dn = g_strdup(book_dn && *book_dn ? book_dn : bind_dn);
-        ldap->enable_tls = enable_tls;
+        libbalsa_address_book_ldap_set_host(ldap, host);
+        libbalsa_address_book_ldap_set_base_dn(ldap, base_dn);
+        libbalsa_address_book_ldap_set_bind_dn(ldap, bind_dn);
+        libbalsa_address_book_ldap_set_passwd(ldap, passwd);
+        libbalsa_address_book_ldap_set_book_dn(ldap,
+                                               book_dn && *book_dn ? book_dn : bind_dn);
+        libbalsa_address_book_ldap_set_enable_tls(ldap, enable_tls);
         libbalsa_address_book_ldap_close_connection(ldap);
 #endif
 #if HAVE_SQLITE
@@ -845,10 +850,10 @@ modify_book(AddressBookConfig * abc)
     } else
         g_assert_not_reached();
 
-    address_book->expand_aliases =
-        !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->never));
-    address_book->is_expensive =
-        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->on_request));
+    libbalsa_address_book_set_expand_aliases(address_book,
+        !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->never)));
+    libbalsa_address_book_set_is_expensive(address_book,
+        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(abc->on_request)));
 
     abc->callback(address_book, FALSE);
 }
@@ -866,7 +871,7 @@ add_vcard_cb(GtkWidget * widget, AddressBookConfig * abc)
 static void
 add_externq_cb(GtkWidget * widget, AddressBookConfig * abc)
 {
-    abc->type = LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN;
+    abc->type = LIBBALSA_TYPE_ADDRESS_BOOK_EXTERNQ;
     abc->window = create_externq_dialog(abc);
     gtk_widget_show_all(abc->window);
 }
