@@ -34,11 +34,6 @@
 /* misc.h for LibBalsaCodeset */
 #include "misc.h"
 
-#if ENABLE_ESMTP
-#include <libesmtp.h>			/* part of libESMTP */
-#include <auth-client.h>		/* part of libESMTP */
-#endif
-
 /* Work around nonprivileged installs so we can find icons */
 #ifdef BALSA_LOCAL_INSTALL
 #define gnome_pixmap_file( s ) g_strconcat( BALSA_RESOURCE_PREFIX, "/pixmaps/", s, NULL ) 
@@ -144,9 +139,7 @@ extern struct BalsaApplication {
 
     gchar *local_mail_directory;
 
-#if ENABLE_ESMTP
     GSList *smtp_servers;
-#endif
 
     /* folder scanning */
     guint local_scan_depth;
@@ -196,17 +189,8 @@ extern struct BalsaApplication {
     gint quiet_background_check;
     gint msg_size_limit; /* for POP mailboxes; in kB */
 
-    /* GUI settings */
-    gint mw_width;
-    gint mw_height;
-    gboolean mw_maximized;
+    /* GUI settings (note: window sizes are tracked by the geometry-manager) */
     gint mblist_width;
-    gint sw_width; /* sendmsg window */
-    gint sw_height;
-    gboolean sw_maximized;
-    gint message_window_width;
-    gint message_window_height;
-    gboolean message_window_maximized;
 
     /* toolbars */
     int toolbar_wrap_button_text;
@@ -244,7 +228,8 @@ extern struct BalsaApplication {
     /* text color of URL's */
     GdkRGBA url_color;
 
-    guint pwindow_option;
+    gboolean send_progress_dialog;
+    gboolean recv_progress_dialog;
     gboolean wordwrap;
     gint wraplength;
     gboolean browse_wrap;
@@ -274,8 +259,6 @@ extern struct BalsaApplication {
 
     /* Source viewer */
     gboolean source_escape_specials;
-    gint source_width;
-    gint source_height;
 
     /* MRU mailbox tree */
     gint mru_tree_width;
@@ -335,6 +318,10 @@ extern struct BalsaApplication {
     gboolean always_queue_sent_mail;
     gboolean copy_to_sentbox;
 
+    /* timer for sending mail every xx minutes */
+    gboolean send_mail_auto;
+    guint send_mail_timer;
+
     /* mailbox indices */
     GtkWidget *notebook;
 
@@ -343,8 +330,8 @@ extern struct BalsaApplication {
     LibBalsaAddressBook *default_address_book;
 
     /* spell checking */
-#if HAVE_GSPELL || HAVE_GTKSPELL
     gchar   *spell_check_lang;
+#if HAVE_GSPELL || HAVE_GTKSPELL
     gboolean spell_check_active;
 #else                           /* HAVE_GSPELL */
     gboolean check_sig;
@@ -373,6 +360,7 @@ extern struct BalsaApplication {
     /* gpgme stuff */
     gboolean has_openpgp;
     gboolean has_smime;
+    gboolean warn_reply_decrypted;
 #endif 
 
     /* Most recently used lists */
@@ -384,6 +372,8 @@ extern struct BalsaApplication {
 
     /* use as default email client for GNOME */
     int default_client;
+
+    gboolean in_destruction;
 } balsa_app;
 
 #define BALSA_IS_MAILBOX_SPECIAL(a) ((a)==balsa_app.inbox || (a)==balsa_app.trash || (a)==balsa_app.outbox||(a)==balsa_app.draftbox || (a)==balsa_app.sentbox)
@@ -392,7 +382,9 @@ void balsa_app_init(void);
 void balsa_app_destroy(void);
 void update_timer(gboolean update, guint minutes);
 
-gchar *ask_password(LibBalsaServer * server, LibBalsaMailbox * mbox);
+gchar *ask_password(LibBalsaServer *server,
+	                const gchar    *cert_subject,
+					gpointer        user_data);
 void balsa_open_mailbox_list(gchar ** urls);
 
 /* Search functions */
