@@ -76,8 +76,12 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, const balsa_pixmap_t *bpixmap)
      * alternative name if not */
     if (!gtk_icon_theme_has_icon(icon_theme, bpixmap->stock_id)) {
 	pixmap_fallback_t *fb = fallback_id;
-	while (fb->def_id && strcmp(fb->def_id, bpixmap->stock_id))
+	while (fb->def_id && g_strcmp0(fb->def_id, bpixmap->stock_id) != 0)
 	    fb++;
+	if (!fb->def_id) {
+	    BICONS_LOG("No GTK or custom icon for %s\n", bpixmap);
+	    return;
+	}
 	if (fb->def_id) {
 	    use_id = fb->fb_id;
             BICONS_LOG("\t(%s not found, fall back to %s)",
@@ -229,6 +233,9 @@ balsa_register_pixbufs(GtkWidget * widget)
         gint width, height;
         const gchar *use_id = balsa_icon_id(icons[i].icon);
 
+        if (use_id == NULL) /* No icon table */
+            break;
+
         gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
         pixbuf =
             gtk_icon_theme_load_icon(icon_theme, use_id, width,
@@ -246,7 +253,13 @@ balsa_register_pixbufs(GtkWidget * widget)
 const gchar *
 balsa_icon_id(const gchar * name)
 {
-    const gchar *retval = g_hash_table_lookup(balsa_icon_table, name);
+    const gchar *retval = NULL;
 
-    return retval ? retval : name;
+    if (balsa_icon_table != NULL) {
+        retval = g_hash_table_lookup(balsa_icon_table, name);
+        if (retval == NULL)
+            retval = name;
+    }
+
+    return retval;
 }
