@@ -190,7 +190,7 @@ bndx_class_init(BalsaIndexClass * klass)
                      G_STRUCT_OFFSET(BalsaIndexClass,
                                      index_changed),
                      NULL, NULL,
-		     g_cclosure_marshal_VOID__VOID,
+		     NULL,
                      G_TYPE_NONE, 0);
 
     object_class->dispose = bndx_destroy;
@@ -453,7 +453,7 @@ bndx_instance_init(BalsaIndex * index)
 
     gtk_drag_source_set(GTK_WIDGET (index),
                         GDK_BUTTON1_MASK | GDK_SHIFT_MASK | GDK_CONTROL_MASK,
-                        index_drag_types, ELEMENTS(index_drag_types),
+                        index_drag_types, G_N_ELEMENTS(index_drag_types),
                         GDK_ACTION_DEFAULT | GDK_ACTION_COPY |
                         GDK_ACTION_MOVE);
     g_signal_connect(index, "drag-data-get",
@@ -472,10 +472,10 @@ bndx_clear_if_last_ref(gpointer data)
 {
     GObject **object = data;
 
-    g_object_add_weak_pointer(G_OBJECT(*object), data);
+    g_object_add_weak_pointer(G_OBJECT(*object), (gpointer *) data);
     g_object_unref(*object);
     if (*object) {
-        g_object_remove_weak_pointer(*object, data);
+        g_object_remove_weak_pointer(*object, (gpointer *) data);
         return FALSE;
     }
     return TRUE;
@@ -1466,9 +1466,7 @@ bndx_view_source(gpointer data)
             continue;
 	libbalsa_show_message_source(balsa_app.application,
                                      message, balsa_app.message_font,
-				     &balsa_app.source_escape_specials,
-                                     &balsa_app.source_width,
-                                     &balsa_app.source_height);
+				     &balsa_app.source_escape_specials);
         g_object_unref(message);
     }
     balsa_index_selected_msgnos_free(index, selected);
@@ -1840,7 +1838,7 @@ bndx_popup_menu_create(BalsaIndex * index)
 
     menu = gtk_menu_new();
 
-    for (i = 0; i < ELEMENTS(entries); i++)
+    for (i = 0; i < G_N_ELEMENTS(entries); i++)
         create_stock_menu_item(menu, _(entries[i].label),
                                entries[i].func, index);
 
@@ -2056,10 +2054,12 @@ balsa_index_update_tree(BalsaIndex * index, gboolean expand)
     bndx_changed_find_row(index);
 }
 
-/* balsa_index_set_threading_type: public method. */
+/* balsa_index_set_thread_messages: public method. */
 void
-balsa_index_set_threading_type(BalsaIndex * index, int thtype)
+balsa_index_set_thread_messages(BalsaIndex * index,
+                                gboolean thread_messages)
 {
+    LibBalsaMailboxThreadingType threading_type;
     LibBalsaMailbox *mailbox;
 
     g_return_if_fail(index != NULL);
@@ -2067,12 +2067,23 @@ balsa_index_set_threading_type(BalsaIndex * index, int thtype)
     mailbox = index->mailbox_node->mailbox;
     g_return_if_fail(mailbox != NULL);
 
-    if (thtype != LB_MAILBOX_THREADING_FLAT
-        && !libbalsa_mailbox_prepare_threading(mailbox, 0))
-        return;
-    libbalsa_mailbox_set_threading_type(mailbox, thtype);
+    if (thread_messages) {
+        if (libbalsa_mailbox_get_subject_gather(mailbox))
+            threading_type = LB_MAILBOX_THREADING_JWZ;
+        else
+            threading_type = LB_MAILBOX_THREADING_SIMPLE;
+    } else {
+        threading_type = LB_MAILBOX_THREADING_FLAT;
+    }
 
-    libbalsa_mailbox_set_threading(mailbox, thtype);
+    if (threading_type == libbalsa_mailbox_get_threading_type(mailbox))
+        return;
+
+    if (thread_messages && !libbalsa_mailbox_prepare_threading(mailbox, 0))
+        return;
+
+    libbalsa_mailbox_set_threading_type(mailbox, threading_type);
+    libbalsa_mailbox_set_threading(mailbox);
     balsa_index_update_tree(index, balsa_app.expand_tree);
 }
 
@@ -2595,7 +2606,7 @@ bndx_pipe_response(GtkWidget * dialog, gint response,
 {
     LibBalsaMailbox *mailbox = info->mailbox;
 
-    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer) & mailbox);
+    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer *) & mailbox);
 
     if (response == GTK_RESPONSE_OK) {
         gchar *pipe_cmd;
@@ -2623,7 +2634,7 @@ bndx_pipe_response(GtkWidget * dialog, gint response,
 
     if (!mailbox)
         return;
-    g_object_remove_weak_pointer(G_OBJECT(mailbox), (gpointer) & mailbox);
+    g_object_remove_weak_pointer(G_OBJECT(mailbox), (gpointer *) & mailbox);
 
     libbalsa_mailbox_close(mailbox, balsa_app.expunge_on_close);
     g_object_set_data(G_OBJECT(mailbox), BALSA_INDEX_PIPE_INFO, NULL);
@@ -2648,7 +2659,8 @@ balsa_index_pipe(BalsaIndex * index)
         g_object_get_data(G_OBJECT(index->mailbox_node->mailbox),
                           BALSA_INDEX_PIPE_INFO);
     if (info) {
-        gtk_window_present(GTK_WINDOW(info->dialog));
+        gtk_window_present_with_time(GTK_WINDOW(info->dialog),
+                                     gtk_get_current_event_time());
         return;
     }
 
