@@ -188,7 +188,7 @@ bmbl_class_init(BalsaMBListClass * klass)
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET(BalsaMBListClass, has_unread_mailbox),
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__INT,
+                     NULL,
                      G_TYPE_NONE,
                      1, G_TYPE_INT);
 
@@ -482,7 +482,7 @@ balsa_mblist_get_store(void)
                                G_TYPE_STRING      /* TOTAL_COLUMN  */
             );
         g_object_add_weak_pointer(G_OBJECT(balsa_app.mblist_tree_store),
-                                  (gpointer) & balsa_app.
+                                  (gpointer *) & balsa_app.
                                   mblist_tree_store);
     }
 
@@ -990,7 +990,7 @@ update_mailbox_idle(struct update_mbox_data *umd)
 
     if (umd->mailbox) {
         g_object_remove_weak_pointer(G_OBJECT(umd->mailbox),
-                                     (gpointer) & umd->mailbox);
+                                     (gpointer *) & umd->mailbox);
         g_object_set_data(G_OBJECT(umd->mailbox), "mblist-update", NULL);
 
         if (balsa_app.mblist_tree_store) {
@@ -1042,7 +1042,7 @@ bmbl_mailbox_changed_cb(LibBalsaMailbox * mailbox, gpointer data)
         g_object_set_data(G_OBJECT(mailbox), "mblist-update", umd);
         umd->mailbox = mailbox;
         g_object_add_weak_pointer(G_OBJECT(mailbox),
-                                  (gpointer) & umd->mailbox);
+                                  (gpointer *) & umd->mailbox);
         g_idle_add((GSourceFunc) update_mailbox_idle, umd);
     }
 
@@ -1269,7 +1269,7 @@ balsa_mblist_default_signal_bindings(BalsaMBList * mblist)
                            G_CALLBACK(bmbl_column_resize), NULL);
     gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(mblist),
                                          bmbl_drop_types,
-                                         ELEMENTS(bmbl_drop_types),
+										 G_N_ELEMENTS(bmbl_drop_types),
                                          GDK_ACTION_DEFAULT |
                                          GDK_ACTION_COPY |
                                          GDK_ACTION_MOVE);
