@@ -33,11 +33,6 @@
  */
 struct _BalsaMimeWidgetImage {
     BalsaMimeWidget  parent;
-
-#if !GTK_CHECK_VERSION(3, 15, 0)
-    GtkStyleContext *context;
-    gulong           context_changed_handler_id;
-#endif
 };
 
 struct _BalsaMimeWidgetImageClass {
@@ -56,18 +51,7 @@ balsa_mime_widget_image_init(BalsaMimeWidgetImage * mwi)
 static void
 balsa_mime_widget_image_dispose(GObject * obj)
 {
-#if !GTK_CHECK_VERSION(3, 15, 0)
-    BalsaMimeWidgetImage *mwi = BALSA_MIME_WIDGET_IMAGE(obj);
-
-    if (mwi->context_changed_handler_id) {
-        g_signal_handler_disconnect(mwi->context,
-                                    mwi->context_changed_handler_id);
-        mwi->context_changed_handler_id = 0;
-    }
-#endif
-
-    (*G_OBJECT_CLASS(balsa_mime_widget_image_parent_class)->
-          dispose) (obj);
+    G_OBJECT_CLASS(balsa_mime_widget_image_parent_class)->dispose(obj);
 }
 
 static void
@@ -85,19 +69,6 @@ static gboolean balsa_image_button_press_cb(GtkWidget * widget, GdkEventButton *
 					    GtkMenu * menu);
 static gboolean img_check_size(GtkImage ** widget_p);
 
-#if !GTK_CHECK_VERSION(3, 15, 0)
-static void
-bmwi_context_changed_cb(GtkStyleContext * context, BalsaMimeWidget * mw)
-{
-    GdkRGBA rgba;
-
-    gtk_style_context_get_background_color(context,
-                                           GTK_STATE_FLAG_NORMAL, &rgba);
-    gtk_widget_override_background_color(mw->widget,
-                                         GTK_STATE_FLAG_NORMAL, &rgba);
-}
-#endif
-
 BalsaMimeWidget *
 balsa_mime_widget_new_image(BalsaMessage * bm,
                             LibBalsaMessageBody * mime_body,
@@ -130,15 +101,6 @@ balsa_mime_widget_new_image(BalsaMessage * bm,
     g_signal_connect(G_OBJECT(mw->widget), "button-press-event",
                      G_CALLBACK(balsa_image_button_press_cb), data);
 
-#if !GTK_CHECK_VERSION(3, 15, 0)
-    mwi->context =
-        gtk_widget_get_style_context(GTK_WIDGET(bm->scroll));
-    bmwi_context_changed_cb(mwi->context, mw);
-    mwi->context_changed_handler_id =
-        g_signal_connect(mwi->context, "changed",
-                         G_CALLBACK(bmwi_context_changed_cb), mw);
-#endif
-
     image = gtk_image_new_from_icon_name("image-missing",
                                          GTK_ICON_SIZE_BUTTON);
     g_object_set_data(G_OBJECT(image), "orig-width",
@@ -166,7 +128,7 @@ balsa_mime_widget_image_resize_all(GtkWidget * widget, gpointer user_data)
         g_object_set_data(G_OBJECT(widget), "check_size_sched",
                           GINT_TO_POINTER(TRUE));
         *widget_p = widget;
-        g_object_add_weak_pointer(G_OBJECT(widget), (gpointer) widget_p);
+        g_object_add_weak_pointer(G_OBJECT(widget), (gpointer *) widget_p);
         g_idle_add((GSourceFunc) img_check_size, widget_p);
     }
 }
@@ -176,7 +138,7 @@ static gboolean
 balsa_image_button_press_cb(GtkWidget * widget, GdkEventButton * event,
                             GtkMenu * menu)
 {
-    if (menu && event->type == GDK_BUTTON_PRESS && event->button == 3) {
+    if (gdk_event_triggers_context_menu((GdkEvent *) event)) {
 #if GTK_CHECK_VERSION(3, 22, 0)
         gtk_menu_popup_at_pointer(menu, (GdkEvent *) event);
 #else                           /*GTK_CHECK_VERSION(3, 22, 0) */
@@ -198,15 +160,13 @@ img_check_size(GtkImage ** widget_p)
     gint curr_w, dst_w;
     GtkAllocation allocation;
 
-    gdk_threads_enter();
-
     widget = *widget_p;
-    g_free(widget_p);
     if (!widget) {
-        gdk_threads_leave();
+        g_free(widget_p);
 	return FALSE;
     }
-    g_object_remove_weak_pointer(G_OBJECT(widget), (gpointer) widget_p);
+    g_object_remove_weak_pointer(G_OBJECT(widget), (gpointer *) widget_p);
+    g_free(widget_p);
 
     viewport = gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_VIEWPORT);
     orig_width = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget),
@@ -217,7 +177,6 @@ img_check_size(GtkImage ** widget_p)
                       GINT_TO_POINTER(FALSE));
     g_warn_if_fail(viewport && mime_body && orig_width > 0);
     if (!(viewport && mime_body && orig_width > 0)) {
-        gdk_threads_leave();
         return FALSE;
     }
 
@@ -253,7 +212,6 @@ img_check_size(GtkImage ** widget_p)
 			          load_err->message);
 		g_error_free(load_err);
 	    }
-            gdk_threads_leave();
 	    return FALSE;
 	}
 	dst_h = (gfloat)dst_w /
@@ -265,6 +223,5 @@ img_check_size(GtkImage ** widget_p)
 	g_object_unref(scaled_pixbuf);
     }
 
-    gdk_threads_leave();
     return FALSE;
 }
