@@ -119,7 +119,7 @@ static GList *fill_text_buf_cited(GtkWidget *widget, const gchar *text_body,
     "balsa-mime-widget-text-new-notified"
 
 #define BALSA_LEFT_MARGIN   2
-#define BALSA_RIGHT_MARGIN 15
+#define BALSA_RIGHT_MARGIN  2
 
 BalsaMimeWidget *
 balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
@@ -179,8 +179,15 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 
     /* configure text or source view */
     gtk_text_view_set_editable(GTK_TEXT_VIEW(mw->widget), FALSE);
+#if GTK_CHECK_VERSION(3, 23, 1)
     gtk_text_view_set_left_margin(GTK_TEXT_VIEW(mw->widget),  BALSA_LEFT_MARGIN);
     gtk_text_view_set_right_margin(GTK_TEXT_VIEW(mw->widget), BALSA_RIGHT_MARGIN);
+#else  /* GTK_CHECK_VERSION(3, 23, 1) */
+    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(mw->widget), 0);
+    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(mw->widget), 0);
+    gtk_widget_set_margin_start(mw->widget, BALSA_LEFT_MARGIN);
+    gtk_widget_set_margin_end(mw->widget, BALSA_RIGHT_MARGIN);
+#endif /* GTK_CHECK_VERSION(3, 23, 1) */
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(mw->widget), GTK_WRAP_WORD_CHAR);
 
     /* set the message font */
@@ -461,7 +468,7 @@ url_send_cb(GtkWidget * menu_item, message_url_t * uri)
     BalsaSendmsg * newmsg;
 
     newmsg = sendmsg_window_compose();
-    sendmsg_window_set_field(newmsg, "body", uri->url);
+    sendmsg_window_set_field(newmsg, "body", uri->url, FALSE);
 }
 
 static gboolean
@@ -678,7 +685,7 @@ prepare_url_offsets(GtkTextBuffer * buffer, GList * url_list)
 #ifdef BUG_102711_FIXED
         gtk_text_iter_backward_to_tag_toggle(&iter, url_tag);
 #else
-        while (gtk_text_iter_backward_char(&iter))
+        while (gtk_text_iter_backward_char(&iter)) {
 #if GTK_CHECK_VERSION(3, 19, 0)
             if (gtk_text_iter_starts_tag(&iter, url_tag))
                 break;
@@ -686,6 +693,7 @@ prepare_url_offsets(GtkTextBuffer * buffer, GList * url_list)
             if (gtk_text_iter_begins_tag(&iter, url_tag))
                 break;
 #endif                          /* GTK_CHECK_VERSION(3, 20, 0) */
+        }
 #endif                          /* BUG_102711_FIXED */
         url->start = gtk_text_iter_get_offset(&iter);
     }
@@ -757,16 +765,16 @@ find_url(GtkWidget * widget, gint x, gint y, GList * url_list)
 static gboolean
 statusbar_pop(gpointer data)
 {
-    GtkStatusbar *statusbar;
-    guint context_id;
+    if (BALSA_IS_WINDOW(balsa_app.main_window)
+        && GTK_IS_STATUSBAR(balsa_app.main_window->statusbar)) {
+        GtkStatusbar *statusbar;
+        guint context_id;
 
-    gdk_threads_enter();
+        statusbar = (GtkStatusbar *) balsa_app.main_window->statusbar;
+        context_id = gtk_statusbar_get_context_id(statusbar, "BalsaMimeWidget message");
+        gtk_statusbar_pop(statusbar, context_id);
+    }
 
-    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
-    context_id = gtk_statusbar_get_context_id(statusbar, "BalsaMimeWidget message");
-    gtk_statusbar_pop(statusbar, context_id);
-
-    gdk_threads_leave();
     return FALSE;
 }
 
@@ -778,12 +786,14 @@ handle_url(const gchar * url)
 {
     if (!g_ascii_strncasecmp(url, "mailto:", 7)) {
         BalsaSendmsg *snd = sendmsg_window_compose();
-        sendmsg_window_process_url(url + 7, sendmsg_window_set_field, snd);
+        sendmsg_window_process_url(snd, url + 7, FALSE);
     } else {
         GtkStatusbar *statusbar;
         guint context_id;
         gchar *notice = g_strdup_printf(_("Calling URL %s…"), url);
+#if !GTK_CHECK_VERSION(3, 22, 0)
         GdkScreen *screen;
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
         GError *err = NULL;
 
         statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
@@ -793,8 +803,13 @@ handle_url(const gchar * url)
         gtk_statusbar_push(statusbar, context_id, notice);
         SCHEDULE_BAR_REFRESH();
         g_free(notice);
+#if GTK_CHECK_VERSION(3, 22, 0)
+        gtk_show_uri_on_window(GTK_WINDOW(balsa_app.main_window), url,
+                               gtk_get_current_event_time(), &err);
+#else  /* GTK_CHECK_VERSION(3, 22, 0) */
         screen = gtk_widget_get_screen(GTK_WIDGET(balsa_app.main_window));
         gtk_show_uri(screen, url, gtk_get_current_event_time(), &err);
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
                               _("Error showing %s: %s\n"),
@@ -1141,7 +1156,7 @@ static gboolean
 balsa_gtk_html_button_press_cb(GtkWidget * html, GdkEventButton * event,
                                BalsaMessage * bm)
 {
-    return ((event->type == GDK_BUTTON_PRESS && event->button == 3)
+    return (gdk_event_triggers_context_menu((GdkEvent *) event)
             ? balsa_gtk_html_popup(html, bm) : FALSE);
 }
 
