@@ -1184,7 +1184,7 @@ void build_condition_dialog(GtkWidget * condition_dialog)
     label = gtk_label_new_with_mnemonic(_("Search T_ype:"));
     gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 5);
     fe_search_option_menu = 
-        fe_build_option_menu(fe_search_type, ELEMENTS(fe_search_type),
+        fe_build_option_menu(fe_search_type, G_N_ELEMENTS(fe_search_type),
                              G_CALLBACK(fe_typesmenu_cb), field_frame);
     gtk_box_pack_start(GTK_BOX(box), fe_search_option_menu, FALSE, FALSE, 5);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), fe_search_option_menu);
@@ -1251,7 +1251,7 @@ fe_edit_condition(GtkWidget * throwaway,gpointer is_new_cnd)
 	libbalsa_macosx_menu_for_parent(condition_dialog, GTK_WINDOW(fe_window));
 #endif
         g_object_add_weak_pointer(G_OBJECT(condition_dialog),
-                                  (gpointer) &condition_dialog);
+                                  (gpointer *) &condition_dialog);
 
         g_signal_connect(G_OBJECT(condition_dialog), "response",
                          G_CALLBACK(condition_dialog_response), NULL);
@@ -1645,7 +1645,7 @@ fe_remove_pressed(GtkWidget * widget, gpointer throwaway)
 static void
 change_filter_name(gchar * old_name,gchar * new_name)
 {
-    if (!new_name || strcmp(old_name,new_name)!=0) {
+    if (g_strcmp0(old_name,new_name)!=0) {
         GList * lst;
         filters_names_rec * p=NULL;
 
@@ -1672,13 +1672,15 @@ change_filter_name(gchar * old_name,gchar * new_name)
           if yes we must change it to : any name -> new_name
           else we create a new record
 	*/
-	for (lst=filters_names_changes;lst;lst=g_list_next(lst))
-            if (((filters_names_rec *)lst->data)->new_name && 
-                strcmp(((filters_names_rec *)lst->data)->new_name,old_name)==0) {
-                p=(filters_names_rec *)lst->data;
+	for (lst=filters_names_changes;lst;lst=g_list_next(lst)) {
+            filters_names_rec *q = lst->data;
+
+            if (g_strcmp0(q->new_name, old_name) == 0) {
+                p = q;
                 g_free(p->new_name);
                 break;
             }
+        }
         if (!lst) {
             /* New name change, create record */
             p=g_new(filters_names_rec,1);
@@ -1688,7 +1690,7 @@ change_filter_name(gchar * old_name,gchar * new_name)
         /* Record exists yet, test if we can collapse it (in case his
          * old_name==new_name) It's only a small optimization
          */
-        else if (new_name && strcmp(p->old_name,new_name)==0) {
+        else if (g_strcmp0(p->old_name,new_name)==0) {
 	    g_free(p->old_name);
 	    g_free(p);
             filters_names_changes=
@@ -1738,6 +1740,7 @@ fe_new_pressed(GtkWidget * widget, gpointer data)
 
     if (filter_errno!=FILTER_NOERR) {
         filter_perror(filter_strerror(filter_errno));
+        g_free(new_item);
         return;
     }
 
@@ -1758,6 +1761,8 @@ fe_new_pressed(GtkWidget * widget, gpointer data)
     /* Adds "New Filter" to the list of actual new filters names */
     new_filters_names=g_list_prepend(new_filters_names,g_strdup(new_item));
     gtk_widget_grab_focus(fe_name_entry);
+
+    g_free(new_item);
 }                       /* end fe_new_pressed() */
 
 /*
