@@ -221,7 +221,8 @@ build_left_side(void)
     gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 2);
 
     /* new button */
-    fe_new_button = gtk_button_new_with_mnemonic(_("_New"));
+    /* Translators: button "New" filter */
+    fe_new_button = gtk_button_new_with_mnemonic(C_("filter", "_New"));
     g_signal_connect(G_OBJECT(fe_new_button), "clicked",
 		     G_CALLBACK(fe_new_pressed), NULL);
     gtk_container_add(GTK_CONTAINER(bbox), fe_new_button);
@@ -278,7 +279,7 @@ build_match_page()
     gtk_grid_attach(GTK_GRID(page), box, 1, 1, 1, 1);
 
     fe_op_codes_option_menu = fe_build_option_menu(fe_op_codes,
-						ELEMENTS(fe_op_codes),
+						G_N_ELEMENTS(fe_op_codes),
 						NULL, NULL);
     g_signal_connect(fe_op_codes_option_menu, "changed",
                      G_CALLBACK(fe_action_changed), NULL);
@@ -311,7 +312,8 @@ build_match_page()
     gtk_container_add(GTK_CONTAINER(box), fe_condition_edit_button);
     g_signal_connect(G_OBJECT(fe_condition_edit_button), "clicked",
                      G_CALLBACK(fe_edit_condition), GINT_TO_POINTER(0));
-    button = gtk_button_new_with_mnemonic(_("Ne_w"));
+    /* Translators: button "New" filter match */
+    button = gtk_button_new_with_mnemonic(C_("filter match", "Ne_w"));
     gtk_container_add(GTK_CONTAINER(box), button);
     g_signal_connect(G_OBJECT(button), "clicked",
                      G_CALLBACK(fe_edit_condition), GINT_TO_POINTER(1));
@@ -443,7 +445,7 @@ build_action_page(GtkWindow * window)
     gtk_container_add(GTK_CONTAINER(frame), box);
 
     fe_action_option_menu =
-        fe_build_option_menu(fe_actions, ELEMENTS(fe_actions),
+        fe_build_option_menu(fe_actions, G_N_ELEMENTS(fe_actions),
                              G_CALLBACK(fe_action_selected), NULL);
     gtk_box_pack_start(GTK_BOX(box), fe_action_option_menu,
                        TRUE, FALSE, 1);
@@ -556,7 +558,8 @@ filters_edit_dialog(GtkWindow * parent)
 	return;
     }
     if (fe_already_open) {
-	gtk_window_present(GTK_WINDOW(fe_window));
+	gtk_window_present_with_time(GTK_WINDOW(fe_window),
+                                     gtk_get_current_event_time());
 	return;
     }
     
