@@ -166,7 +166,7 @@ balsa_filter_run_dialog_class_init(BalsaFilterRunDialogClass * klass)
                      G_SIGNAL_RUN_FIRST,
 		     G_STRUCT_OFFSET(BalsaFilterRunDialogClass, refresh),
                      NULL, NULL,
-		     g_cclosure_marshal_VOID__OBJECT,
+		     NULL,
                      G_TYPE_NONE, 1, G_TYPE_OBJECT);
 
     object_class->dispose = balsa_filter_run_dispose;
@@ -183,9 +183,7 @@ balsa_filter_run_dialog_new(LibBalsaMailbox * mbox, GtkWindow * parent)
 
     g_return_val_if_fail(mbox, NULL);
 
-#if GTK_CHECK_VERSION(3, 12, 0)
     use_headerbar = (libbalsa_dialog_flags() & GTK_DIALOG_USE_HEADER_BAR) != 0;
-#endif
     p = g_object_new(BALSA_TYPE_FILTER_RUN_DIALOG,
                      "transient-for", parent,
                      "use-header-bar", use_headerbar,
@@ -498,7 +496,8 @@ filters_run_dialog(LibBalsaMailbox * mbox,
     }
     if (lst) {
 	/* If there was yet a dialog box for this mailbox, we raise it */
-	gtk_window_present(GTK_WINDOW(lst->data));
+	gtk_window_present_with_time(GTK_WINDOW(lst->data),
+                                     gtk_get_current_event_time());
 	return;
     }
 
