@@ -134,7 +134,7 @@ balsa_mailbox_node_class_init(BalsaMailboxNodeClass * klass)
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET(BalsaMailboxNodeClass, save_config),
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__POINTER,
+                     NULL,
 		     G_TYPE_NONE, 1,
 	             G_TYPE_POINTER);
     balsa_mailbox_node_signals[LOAD_CONFIG] =
@@ -143,7 +143,7 @@ balsa_mailbox_node_class_init(BalsaMailboxNodeClass * klass)
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET(BalsaMailboxNodeClass, load_config),
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__POINTER,
+                     NULL,
 		     G_TYPE_NONE, 1,
 	             G_TYPE_POINTER);
     balsa_mailbox_node_signals[SHOW_PROP_DIALOG] =
@@ -153,7 +153,7 @@ balsa_mailbox_node_class_init(BalsaMailboxNodeClass * klass)
                      G_STRUCT_OFFSET(BalsaMailboxNodeClass,
                                      show_prop_dialog),
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__VOID,
+                     NULL,
 		     G_TYPE_NONE, 0);
     balsa_mailbox_node_signals[APPEND_SUBTREE] =
 	g_signal_new("append-subtree",
@@ -161,7 +161,7 @@ balsa_mailbox_node_class_init(BalsaMailboxNodeClass * klass)
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET(BalsaMailboxNodeClass, append_subtree),
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__VOID,
+                     NULL,
 		     G_TYPE_NONE, 0);
 
     klass->save_config = balsa_mailbox_node_real_save_config;
@@ -193,11 +193,12 @@ balsa_mailbox_node_dispose(GObject * object)
     LibBalsaMailbox *mailbox = mn->mailbox;
 
     if (mailbox) {
-        libbalsa_mailbox_set_open(mailbox,
-                                  libbalsa_mailbox_is_open(mailbox));
-        config_save_mailbox_view(mailbox->url, mailbox->view);
-	if (balsa_app.main_window)
-	    balsa_window_close_mbnode(balsa_app.main_window, mn);
+        if (!LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
+            libbalsa_mailbox_set_open(mailbox, libbalsa_mailbox_is_open(mailbox));
+            config_save_mailbox_view(mailbox->url, mailbox->view);
+            if (balsa_app.main_window != NULL)
+                balsa_window_close_mbnode(balsa_app.main_window, mn);
+        }
 	g_object_unref(mailbox);
 	mn->mailbox = NULL;
     }
@@ -269,7 +270,7 @@ balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * group)
     g_return_if_fail(!mn->parent);
 
     if(mn->name)
-	printf("Saving mailbox node %s with group %s\n", mn->name, group);
+	g_debug("Saving mailbox node %s with group %s", mn->name, group);
     libbalsa_imap_server_save_config(LIBBALSA_IMAP_SERVER(mn->server));
     libbalsa_conf_set_string("Name",      mn->name);
     libbalsa_conf_set_string("Directory", mn->dir);
@@ -282,7 +283,7 @@ balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * group)
         children_names = g_ptr_array_new();
         balsa_mailboxes_append_children(model, &iter,
                                         children_names);
-        printf("Saving %d children\n", children_names->len);
+        g_debug("Saving %d children", children_names->len);
         libbalsa_conf_set_vector("Children", children_names->len,
                                  (const char*const*)(children_names->pdata));
         g_ptr_array_foreach(children_names, (GFunc)g_free, NULL);
@@ -366,9 +367,7 @@ check_local_path(const gchar * path, guint depth)
     libbalsa_conf_foreach_group(VIEW_BY_URL_SECTION_PREFIX,
                                 (LibBalsaConfForeachFunc) check_url_func,
                                 &cpi);
-    if(balsa_app.debug) 
-	printf("check_local_path: path “%s” must_scan %d.\n",
-               cpi.url, cpi.must_scan);
+    g_debug("check_local_path: path “%s” must_scan %d.", cpi.url, cpi.must_scan);
     g_free(cpi.url);
 
     return cpi.must_scan;
@@ -429,9 +428,7 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
     libbalsa_mailbox_remote_set_server(LIBBALSA_MAILBOX_REMOTE(m),
 				       mbnode->server);
     libbalsa_mailbox_imap_set_path(m, isi->fn);
-    if(balsa_app.debug)
-        printf("imap_scan_attach_mailbox: add mbox of name %s "
-	       "(full path %s)\n", isi->fn, mailbox->url);
+    g_debug("imap_scan_attach_mailbox: add mbox of name %s (full path %s)", isi->fn, mailbox->url);
     /* avoid allocating the name again: */
     mailbox->name = mbnode->name;
     mbnode->name = NULL;
@@ -440,9 +437,9 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
     if (isi->special) {
 	if (*isi->special)
 	    g_object_remove_weak_pointer(G_OBJECT(*isi->special),
-					 (gpointer) isi->special);
+					 (gpointer *) isi->special);
         *isi->special = mailbox;
-	g_object_add_weak_pointer(G_OBJECT(m), (gpointer) isi->special);
+	g_object_add_weak_pointer(G_OBJECT(m), (gpointer *) isi->special);
         if (isi->special == &balsa_app.outbox)
             mailbox->no_reassemble = TRUE;
     }
@@ -541,10 +538,10 @@ imap_dir_cb(BalsaMailboxNode* mb)
     }
     imap_scan_destroy_tree(&imap_tree);
 
-    if(balsa_app.debug && mb->name)
-        printf("imap_dir_cb:  main mailbox node %s mailbox is %p\n", 
-               mb->name, mb->mailbox);
-    if(balsa_app.debug) printf("%d: Scanning done.\n", (int)time(NULL));
+    if (mb->name) {
+    	g_debug("imap_dir_cb: main mailbox node %s mailbox is %p", mb->name, mb->mailbox);
+    }
+    g_debug("%d: Scanning done.", (int)time(NULL));
     gtk_statusbar_pop(statusbar, context_id);
 
     /* We can save the cache now... */
@@ -605,9 +602,8 @@ balsa_mailbox_node_new_from_config(const gchar* group)
 
     folder->server = LIBBALSA_SERVER(libbalsa_imap_server_new_from_config());
 
-    if(balsa_app.debug)
-	printf("Server loaded, host: %s, %s.\n", folder->server->host,
-	       folder->server->use_ssl ? "SSL" : "no SSL");
+	g_debug("Server loaded, host: %s, security %d.", folder->server->host,
+		folder->server->security);
     g_signal_connect_swapped(G_OBJECT(folder->server), "config-changed", 
                              G_CALLBACK(config_folder_update), folder);
     g_signal_connect(G_OBJECT(folder), "show-prop-dialog", 
@@ -619,6 +615,9 @@ balsa_mailbox_node_new_from_config(const gchar* group)
     balsa_mailbox_node_load_config(folder, group);
 
     folder->dir = libbalsa_conf_get_string("Directory");
+    if (folder->dir == NULL) {
+    	folder->dir = g_strdup("");
+    }
     folder->subscribed =
 	libbalsa_conf_get_bool("Subscribed"); 
     folder->list_inbox =
@@ -649,22 +648,6 @@ balsa_mailbox_node_new_imap_node(LibBalsaServer* s, const char*p)
     return folder;
 }
 
-BalsaMailboxNode*
-balsa_mailbox_node_new_imap(LibBalsaServer* s, const char*p)
-{
-    BalsaMailboxNode * folder = balsa_mailbox_node_new_imap_node(s, p);
-    g_assert(s);
-
-    folder->mailbox = libbalsa_mailbox_imap_new();
-    g_object_ref(G_OBJECT(folder->mailbox));
-    libbalsa_mailbox_remote_set_server(
-	LIBBALSA_MAILBOX_REMOTE(folder->mailbox), s);
-    libbalsa_mailbox_imap_set_path(LIBBALSA_MAILBOX_IMAP(folder->mailbox), p);
-    load_mailbox_view(folder);
-
-    return folder;
-}
-
 BalsaMailboxNode*
 balsa_mailbox_node_new_imap_folder(LibBalsaServer* s, const char*p)
 {
@@ -795,8 +778,9 @@ restore_children_from_cache(BalsaMailboxNode *mn)
     imap_scan_item isi;
 
     if(!children) {
-        if(mn->name)
-            printf("No cache for %s - quitting.\n", mn->name);
+        if (mn->name) {
+        	g_debug("No cache for %s - quitting.", mn->name);
+        }
         return FALSE;
     }
 
@@ -853,7 +837,7 @@ balsa_mailbox_node_scan_children(BalsaMailboxNode * mbnode)
                                    BALSA_MAILBOX_NODE_LIST_KEY)) {
         BalsaMailboxNode **mn = g_new(BalsaMailboxNode *, 1);
         *mn = mbnode;
-        g_object_add_weak_pointer(G_OBJECT(mbnode), (gpointer) mn);
+        g_object_add_weak_pointer(G_OBJECT(mbnode), (gpointer *) mn);
         g_object_set_data(G_OBJECT(mbnode), BALSA_MAILBOX_NODE_LIST_KEY,
                           g_slist_reverse(list));
         g_idle_add((GSourceFunc) bmbn_scan_children_idle, mn);
@@ -900,7 +884,7 @@ bmbn_scan_children_idle(BalsaMailboxNode ** mbnode)
     if (*mbnode) {
         g_object_set_data(G_OBJECT(*mbnode), BALSA_MAILBOX_NODE_LIST_KEY,
                           NULL);
-        g_object_remove_weak_pointer(G_OBJECT(*mbnode), (gpointer) mbnode);
+        g_object_remove_weak_pointer(G_OBJECT(*mbnode), (gpointer *) mbnode);
     }
     g_free(mbnode);
 
@@ -917,7 +901,7 @@ add_menu_entry(GtkWidget * menu, const gchar * label, GCallback cb,
     GtkWidget *menuitem;
 
     menuitem = label ? gtk_menu_item_new_with_mnemonic(label)
-	: gtk_menu_item_new();
+	: gtk_separator_menu_item_new();
 
     if (cb)
 	g_signal_connect(G_OBJECT(menuitem), "activate",
@@ -983,22 +967,6 @@ mb_draftbox_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
     config_mailbox_set_as_special(mbnode->mailbox, SPECIAL_DRAFT);
 }
 
-static void
-mb_subscribe_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
-{
-    g_return_if_fail(LIBBALSA_IS_MAILBOX_IMAP(mbnode->mailbox));
-    libbalsa_mailbox_imap_subscribe(LIBBALSA_MAILBOX_IMAP(mbnode->mailbox), 
-				    TRUE);
-}
-
-static void
-mb_unsubscribe_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
-{
-    g_return_if_fail(LIBBALSA_IS_MAILBOX_IMAP(mbnode->mailbox));
-    libbalsa_mailbox_imap_subscribe(LIBBALSA_MAILBOX_IMAP(mbnode->mailbox),
-				     FALSE);
-}
-
 static void
 mb_rescan_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
 {
@@ -1038,6 +1006,14 @@ mb_empty_trash_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
     empty_trash(balsa_app.main_window);
 }
 
+gboolean
+balsa_mailbox_node_is_imap(const BalsaMailboxNode *mbnode)
+{
+	return (mbnode != NULL) &&
+		(mbnode->server != NULL) &&
+		(strcmp(mbnode->server->protocol, "imap") == 0);
+}
+
 GtkWidget *
 balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 {
@@ -1061,13 +1037,13 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 		   G_CALLBACK(mailbox_conf_add_maildir_cb), NULL);
     add_menu_entry(submenu, _("Local M_H mailbox…"),
 		   G_CALLBACK(mailbox_conf_add_mh_cb), NULL);
-    add_menu_entry(submenu, _("Remote _IMAP mailbox…"), 
-		   G_CALLBACK(mailbox_conf_add_imap_cb), NULL);
     add_menu_entry(submenu, NULL, NULL, mbnode);
     add_menu_entry(submenu, _("Remote IMAP _folder…"), 
 		   G_CALLBACK(folder_conf_add_imap_cb), NULL);
-    add_menu_entry(submenu, _("Remote IMAP _subfolder…"), 
-		   G_CALLBACK(folder_conf_add_imap_sub_cb), NULL);
+    if (balsa_mailbox_node_is_imap(mbnode)) {
+    	add_menu_entry(submenu, _("Remote IMAP _subfolder…"),
+		   G_CALLBACK(folder_conf_add_imap_sub_cb), mbnode);
+    }
     gtk_widget_show(submenu);
     
     /* Translators: popup menu item "New" mailbox or folder */
@@ -1116,14 +1092,6 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
     if (!special && !mbnode->config_prefix)
 	add_menu_entry(menu, _("_Delete"), G_CALLBACK(mb_del_cb),  mbnode);
 
-    if(LIBBALSA_IS_MAILBOX_IMAP(mailbox)) {
-        add_menu_entry(menu, NULL, NULL, NULL);
-        add_menu_entry(menu, _("_Subscribe"),   
-                       G_CALLBACK(mb_subscribe_cb),   mbnode);
-        add_menu_entry(menu, _("_Unsubscribe"), 
-                       G_CALLBACK(mb_unsubscribe_cb), mbnode);
-    }
-
     if (!special) {
         add_menu_entry(menu, NULL, NULL, NULL);
         add_menu_entry(menu, _("Mark as _Inbox"),    
@@ -1178,15 +1146,15 @@ remove_special_mailbox_by_url(const gchar* url, LibBalsaMailbox *** special)
 {
     LibBalsaMailbox **mailbox;
 
-    if (balsa_app.trash && strcmp(url, balsa_app.trash->url) == 0)
+    if (g_strcmp0(url, balsa_app.trash->url) == 0)
 	mailbox = &balsa_app.trash;
-    else if (balsa_app.inbox && strcmp(url, balsa_app.inbox->url) == 0)
+    else if (g_strcmp0(url, balsa_app.inbox->url) == 0)
 	mailbox = &balsa_app.inbox;
-    else if (balsa_app.outbox && strcmp(url, balsa_app.outbox->url) == 0)
+    else if (g_strcmp0(url, balsa_app.outbox->url) == 0)
 	mailbox = &balsa_app.outbox;
-    else if (balsa_app.sentbox && strcmp(url, balsa_app.sentbox->url) == 0)
+    else if (g_strcmp0(url, balsa_app.sentbox->url) == 0)
 	mailbox = &balsa_app.sentbox;
-    else if (balsa_app.draftbox && strcmp(url, balsa_app.draftbox->url) == 0)
+    else if (g_strcmp0(url, balsa_app.draftbox->url) == 0)
 	mailbox = &balsa_app.draftbox;
     else
         mailbox = NULL;
@@ -1232,7 +1200,7 @@ add_local_mailbox(BalsaMailboxNode *root, const gchar * name,
 	    mailbox = NULL;
 	}
 	if(!mailbox) {/* local mailbox could not be created; privileges? */
-	    printf("Not accessible mailbox %s\n", path);
+		g_warning("Not accessible mailbox %s", path);
 	    return NULL;
 	}
 	mailbox->name = g_strdup(name);
@@ -1401,10 +1369,8 @@ handle_imap_path(const char *fn, char delim, int noselect, int noscan,
 	    /* ignore mailboxes that begin with a dot */
 	    return;
     }
-    if (balsa_app.debug)
-	printf("handle_imap_path: Adding mailbox of path “%s” "
-	       "delim `%c' noselect %d noscan %d\n",
-	       fn, delim, noselect, noscan);
+    g_debug("handle_imap_path: Adding mailbox of path “%s” delim `%c' noselect %d noscan %d",
+    	fn, delim, noselect, noscan);
     add_imap_entry(fn, delim, noscan, !noselect, marked, data);
 }
 
@@ -1431,9 +1397,7 @@ check_imap_path(const gchar *fn, LibBalsaServer * server, guint depth)
     libbalsa_conf_foreach_group(VIEW_BY_URL_SECTION_PREFIX,
                                 (LibBalsaConfForeachFunc) check_url_func,
                                 &cpi);
-    if(balsa_app.debug) 
-	printf("check_imap_path: path “%s” must_scan %d.\n",
-               cpi.url, cpi.must_scan);
+    g_debug("check_imap_path: path “%s” must_scan %d.", cpi.url, cpi.must_scan);
     g_free(cpi.url);
 
     return cpi.must_scan;
@@ -1449,8 +1413,7 @@ mark_imap_path(const gchar * fn, gpointer data)
     imap_scan_tree *tree = data;
     GSList *list;
 
-    if(balsa_app.debug) 
-	printf("mark_imap_path: find path “%s”.\n", fn);
+    g_debug("mark_imap_path: find path “%s”.", fn);
     for (list = tree->list; list; list = list->next) {
         imap_scan_item *item = list->data;
         if (!strcmp(item->fn, fn)) {
@@ -1458,6 +1421,7 @@ mark_imap_path(const gchar * fn, gpointer data)
             break;
         }
     }
-    if (!list && balsa_app.debug)
-	printf(" not found.\n");
+    if (!list) {
+    	g_debug("… not found.");
+    }
 }
