@@ -47,6 +47,8 @@
 #include "information.h"
 #include "imap-server.h"
 #include "libbalsa-conf.h"
+#include "autocrypt.h"
+#include "geometry-manager.h"
 
 #include "libinit_balsa/assistant_init.h"
 
@@ -55,10 +57,6 @@
 #include "libbalsa-gpgme-cb.h"
 #endif
 
-static void config_init(gboolean check_only);
-static void mailboxes_init(gboolean check_only);
-static void balsa_cleanup(void);
-
 /* We need separate variable for storing command line requests to check the
    mail because such selection cannot be stored in balsa_app and later
    saved to the configuration file.
@@ -198,10 +196,9 @@ static void
 balsa_get_stats(long *unread, long *unsent)
 {
 
-    if(balsa_app.inbox && libbalsa_mailbox_open(balsa_app.inbox, NULL) ) {
+    if (balsa_app.inbox && libbalsa_mailbox_open(balsa_app.inbox, NULL)) {
         /* set threading type to load messages */
-        libbalsa_mailbox_set_threading(balsa_app.inbox,
-                                       balsa_app.inbox->view->threading_type);
+        libbalsa_mailbox_set_threading(balsa_app.inbox);
         *unread = balsa_app.inbox->unread_messages;
         libbalsa_mailbox_close(balsa_app.inbox, FALSE);
     } else *unread = -1;
@@ -370,7 +367,7 @@ periodic_expunge_cb(void)
 /*
  * Initialize the progress bar and set text.
  */
-static GTimeVal prev_time_val;
+static gint64   prev_time_val;
 static gdouble  min_fraction;
 static void
 balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
@@ -386,7 +383,7 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
      * for a subthread */
     if (!text || total >= LIBBALSA_PROGRESS_MIN_COUNT)
         rc = balsa_window_setup_progress(balsa_app.main_window, text);
-    g_get_current_time(&prev_time_val);
+    prev_time_val = g_get_monotonic_time();
     min_fraction = LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
 
     *progress = (text && rc) ?
@@ -400,8 +397,8 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
 static void
 balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
 {
-    GTimeVal time_val;
-    guint elapsed;
+    gint64 time_val;
+    gint elapsed;
 
     if (*progress == LIBBALSA_PROGRESS_NO)
         return;
@@ -409,15 +406,13 @@ balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
     if (fraction > 0.0 && fraction < min_fraction)
         return;
 
-    g_get_current_time(&time_val);
-    elapsed = time_val.tv_sec - prev_time_val.tv_sec;
-    elapsed *= G_USEC_PER_SEC;
-    elapsed += time_val.tv_usec - prev_time_val.tv_usec;
+    time_val = g_get_monotonic_time();
+    elapsed = time_val - prev_time_val;
     if (elapsed < LIBBALSA_PROGRESS_MIN_UPDATE_USECS)
         return;
 
-    g_time_val_add(&time_val, LIBBALSA_PROGRESS_MIN_UPDATE_USECS);
-    min_fraction += LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
+    prev_time_val += LIBBALSA_PROGRESS_MIN_UPDATE_USECS;
+    min_fraction  += LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
 
     if (balsa_app.main_window)
         balsa_window_increment_progress(balsa_app.main_window, fraction,
@@ -447,10 +442,9 @@ balsa_check_open_compose_window(void)
 
         if (opt_compose_email) {
             if (g_ascii_strncasecmp(opt_compose_email, "mailto:", 7) == 0)
-                sendmsg_window_process_url(opt_compose_email + 7,
-                                           sendmsg_window_set_field, snd);
+                sendmsg_window_process_url(snd, opt_compose_email + 7, TRUE);
             else
-                sendmsg_window_set_field(snd, "to", opt_compose_email);
+                sendmsg_window_set_field(snd, "to", opt_compose_email, TRUE);
             g_free(opt_compose_email);
             opt_compose_email = NULL;
         }
@@ -468,12 +462,49 @@ balsa_check_open_compose_window(void)
     return FALSE;
 }
 
+/*
+ * Set up GNotification for libbalsa
+ */
+
+#define BALSA_NOTIFICATION "balsa-notification"
+
+static void
+balsa_notification_notify_cb(GNotification *notification,
+                             GParamSpec *pspec,
+                             GApplication *application)
+{
+    gboolean send;
+
+    send = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(notification), "send"));
+    if (send) {
+        g_application_send_notification(application,
+                                        BALSA_NOTIFICATION, notification);
+    } else {
+        g_application_withdraw_notification(application, BALSA_NOTIFICATION);
+    }
+}
+
+static void
+balsa_setup_libbalsa_notification(GApplication *application)
+{
+    GNotification *notification;
+
+    notification = libbalsa_notification_new("Balsa");
+    g_signal_connect(notification, "notify",
+                     G_CALLBACK(balsa_notification_notify_cb), application);
+    g_signal_connect_swapped(application, "shutdown",
+                             G_CALLBACK(g_object_unref), notification);
+}
+
 /* -------------------------- main --------------------------------- */
-static int
-real_main(int argc, char *argv[])
+static void
+balsa_startup_cb(GApplication *application,
+           gpointer      user_data)
 {
-    GtkWidget *window;
     gchar *default_icon;
+#ifdef ENABLE_AUTOCRYPT
+    GError *error = NULL;
+#endif
 
 #ifdef ENABLE_NLS
     /* Initialize the i18n stuff */
@@ -497,7 +528,8 @@ real_main(int argc, char *argv[])
     balsa_app_init();
 
     /* Initialize libbalsa */
-    libbalsa_init((LibBalsaInformationFunc) balsa_information_real);
+    balsa_setup_libbalsa_notification(application);
+    libbalsa_init();
     libbalsa_filters_set_url_mapper(balsa_find_mailbox_by_url);
     libbalsa_filters_set_filter_list(&balsa_app.filters);
 
@@ -507,6 +539,13 @@ real_main(int argc, char *argv[])
 
     libbalsa_mailbox_date_format = &balsa_app.date_string;
 
+#ifdef ENABLE_AUTOCRYPT
+    if (!autocrypt_init(&error)) {
+    	libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("Autocrypt error: %s"), error->message);
+    	g_error_free(error);
+    }
+#endif
+
     /* checking for valid config files */
     config_init(cmd_get_stats);
 
@@ -516,23 +555,12 @@ real_main(int argc, char *argv[])
         g_free(default_icon);
     }
 
-    signal( SIGPIPE, SIG_IGN );
-
-    window = balsa_window_new();
-    balsa_app.main_window = BALSA_WINDOW(window);
-    g_object_add_weak_pointer(G_OBJECT(window),
-			      (gpointer) &balsa_app.main_window);
-
-    /* load mailboxes */
-    config_load_sections();
-    mailboxes_init(cmd_get_stats);
-
     if (cmd_get_stats) {
         long unread, unsent;
         balsa_get_stats(&unread, &unsent);
         printf("Unread: %ld Unsent: %ld\n", unread, unsent);
         g_application_quit(G_APPLICATION(balsa_app.application));
-        return 0;
+        return;
     }
 
 #ifdef HAVE_GPGME
@@ -542,10 +570,44 @@ real_main(int argc, char *argv[])
         libbalsa_gpgme_check_crypto_engine(GPGME_PROTOCOL_CMS);
 #endif /* HAVE_GPGME */
 
+    accel_map_load();
+}
+
+static void
+balsa_shutdown_cb(void)
+{
+    balsa_app_destroy();
+
+    libbalsa_conf_drop_all();
+    accel_map_save();
+    libbalsa_imap_server_close_all_connections();
+    libbalsa_information(LIBBALSA_INFORMATION_MESSAGE, "%s", "");
+}
+
+static void
+balsa_activate_cb(GApplication *application,
+            gpointer      user_data)
+{
+    GtkWidget *window;
+    const geometry_t *main_size;
+
+    if (balsa_app.main_window != NULL) {
+        gtk_window_present_with_time(GTK_WINDOW(balsa_app.main_window),
+                                     gtk_get_current_event_time());
+        return;
+    }
+
+    window = balsa_window_new(GTK_APPLICATION(application));
+    balsa_app.main_window = BALSA_WINDOW(window);
+    g_object_add_weak_pointer(G_OBJECT(window),
+			      (gpointer *) &balsa_app.main_window);
+
     balsa_check_open_compose_window();
 
     g_idle_add((GSourceFunc) scan_mailboxes_idle_cb, NULL);
-    if (balsa_app.mw_maximized) {
+    main_size = geometry_manager_get("MainWindow");
+    g_assert(main_size != NULL);
+    if (main_size->maximized) {
         /*
          * When maximized at startup, the window changes from maximized
          * to not maximized a couple of times, so we wait until it has
@@ -562,22 +624,9 @@ real_main(int argc, char *argv[])
         g_idle_add((GSourceFunc) balsa_main_check_new_messages,
                    balsa_app.main_window);
 
-    accel_map_load();
-    gtk_main();
-
-    balsa_cleanup();
-    accel_map_save();
-
-    libbalsa_imap_server_close_all_connections();
-    return 0;
-}
-
-static void
-balsa_cleanup(void)
-{
-    balsa_app_destroy();
-
-    libbalsa_conf_drop_all();
+    /* load mailboxes */
+    config_load_sections();
+    mailboxes_init(cmd_get_stats);
 }
 
 /*
@@ -611,8 +660,6 @@ parse_options(int                       argc,
         {"get-stats", 's', 0, G_OPTION_ARG_NONE,
          &(cmd_get_stats),
          N_("Prints number unread and unsent messages"), NULL},
-        {"debug-imap", 'D', 0, G_OPTION_ARG_NONE, &ImapDebug,
-         N_("Debug IMAP connection"), NULL},
         {"help", 'h', 0, G_OPTION_ARG_NONE, &help, N_("Show help options"), NULL},
         {"version", 'v', 0, G_OPTION_ARG_NONE, &version, N_("Show version"), NULL},
         /* last but not least a special option that collects filenames */
@@ -716,13 +763,14 @@ handle_remote(int argc, char **argv,
 
         if (!balsa_check_open_compose_window()) {
             /* Move the main window to the request's screen */
-            gtk_window_present(GTK_WINDOW(balsa_app.main_window));
+            gtk_window_present_with_time(GTK_WINDOW(balsa_app.main_window),
+                                         gtk_get_current_event_time());
         }
     }
 }
 
 static int
-command_line_cb(GApplication            * application,
+balsa_command_line_cb(GApplication            * application,
                 GApplicationCommandLine * command_line,
                 gpointer                  user_data)
 {
@@ -745,8 +793,7 @@ command_line_cb(GApplication            * application,
              * handle the command line. */
             handle_remote(argc, argv, command_line);
         } else {
-            /* This is the primary instance; start up as usual. */
-            status = real_main(argc, argv);
+            g_application_activate(application);
         }
     } else if (status == 2) /* handled a "help" or "version" request */
         status = 0;
@@ -766,10 +813,17 @@ main(int argc, char **argv)
     balsa_app.application = application =
         gtk_application_new("org.desktop.Balsa",
                             G_APPLICATION_HANDLES_COMMAND_LINE);
-    g_signal_connect(application, "command-line",
-                     G_CALLBACK(command_line_cb), NULL);
     g_object_set(application, "register-session", TRUE, NULL);
 
+    g_signal_connect(application, "command-line",
+                     G_CALLBACK(balsa_command_line_cb), NULL);
+    g_signal_connect(application, "startup",
+                     G_CALLBACK(balsa_startup_cb), NULL);
+    g_signal_connect(application, "activate",
+                     G_CALLBACK(balsa_activate_cb), NULL);
+    g_signal_connect(application, "shutdown",
+                     G_CALLBACK(balsa_shutdown_cb), NULL);
+
     status = g_application_run(G_APPLICATION(application), argc, argv);
 
     g_object_unref(application);
