@@ -31,6 +31,7 @@
 #include "sendmsg-window.h"
 #include "print.h"
 #include "mailbox-node.h"
+#include "geometry-manager.h"
 
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
@@ -365,7 +366,7 @@ destroy_message_window(GtkWidget * widget, MessageWindow * mw)
     }
 
     if (mw->mailbox) {
-        g_object_remove_weak_pointer(G_OBJECT(mw->mailbox), (gpointer) &mw->mailbox);
+        g_object_remove_weak_pointer(G_OBJECT(mw->mailbox), (gpointer *) &mw->mailbox);
         g_signal_handlers_disconnect_matched(G_OBJECT(mw->mailbox),
                                              G_SIGNAL_MATCH_DATA, 0, 0,
                                              NULL, NULL, mw);
@@ -497,9 +498,7 @@ mw_view_source_activated(GSimpleAction * action, GVariant * parameter,
     MessageWindow *mw = (MessageWindow *) data;
     libbalsa_show_message_source(balsa_app.application,
                                  mw->message, balsa_app.message_font,
-                                 &balsa_app.source_escape_specials,
-                                 &balsa_app.source_width,
-                                 &balsa_app.source_height);
+                                 &balsa_app.source_escape_specials);
 }
 
 static void
@@ -510,25 +509,6 @@ mw_close_activated(GSimpleAction * action, GVariant * parameter,
     gtk_widget_destroy(mw->window);
 }
 
-static void
-size_alloc_cb(GtkWidget * window, GtkAllocation * alloc)
-{
-    GdkWindow *gdk_window;
-
-    gdk_window = gtk_widget_get_window(window);
-    if (gdk_window == NULL)
-        return;
-
-    balsa_app.message_window_maximized =
-        (gdk_window_get_state(gdk_window) &
-         (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) != 0;
-
-    if (!balsa_app.message_window_maximized)
-        gtk_window_get_size(GTK_WINDOW(window),
-                            & balsa_app.message_window_width,
-                            & balsa_app.message_window_height);
-}
-
 static void
 mw_copy_activated(GSimpleAction * action, GVariant * parameter,
                   gpointer data)
@@ -578,7 +558,8 @@ mw_set_selected(MessageWindow * mw, void (*select_func) (BalsaIndex *))
     if ((tmp = g_object_get_data(G_OBJECT(message),
                                  BALSA_MESSAGE_WINDOW_KEY))) {
         if (tmp == mw) {
-            gtk_window_present(GTK_WINDOW(tmp->window));
+            gtk_window_present_with_time(GTK_WINDOW(tmp->window),
+                                         gtk_get_current_event_time());
             g_object_unref(message);
             return;
         }
@@ -819,7 +800,8 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
     if (message 
         && (mw = g_object_get_data(G_OBJECT(message), 
                                    BALSA_MESSAGE_WINDOW_KEY))) {
-        gtk_window_present(GTK_WINDOW(mw->window));
+        gtk_window_present_with_time(GTK_WINDOW(mw->window),
+                                     gtk_get_current_event_time());
         g_object_unref(message);
         return;
     }
@@ -863,8 +845,6 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
 
     g_signal_connect(G_OBJECT(window), "destroy",
 		     G_CALLBACK(destroy_message_window), mw);
-    g_signal_connect(G_OBJECT(window), "size_allocate",
-                     G_CALLBACK(size_alloc_cb), NULL);
     
     mw->bindex = balsa_find_index_by_mailbox(mailbox);
     g_object_weak_ref(G_OBJECT(mw->bindex), mw_bindex_closed_cb, mw);
@@ -872,7 +852,7 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
 			     G_CALLBACK(mw_set_buttons_sensitive), mw);
 
     mw->mailbox = mailbox;
-    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer) &mw->mailbox);
+    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer *) &mw->mailbox);
     g_signal_connect(mailbox, "message_expunged",
                      G_CALLBACK(mw_expunged_cb), mw);
 
@@ -910,11 +890,7 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
     mw_set_enabled(mw, "reply-group",
                    libbalsa_message_get_user_header(message, "list-post") != NULL);
 
-    gtk_window_set_default_size(GTK_WINDOW(window),
-                                balsa_app.message_window_width, 
-                                balsa_app.message_window_height);
-    if (balsa_app.message_window_maximized)
-        gtk_window_maximize(GTK_WINDOW(window));
+    geometry_manager_attach(GTK_WINDOW(window),"MessageWindow" );
 
     gtk_widget_show(window);
     mw_set_message(mw, message);
