@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2016 Stuart Parmenter and others,
+ * Copyright (C) 1997-2019 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,9 @@
 #include "balsa-print-object.h"
 #include "balsa-print-object-decor.h"
 #include "balsa-print-object-header.h"
+#ifdef HAVE_GPGME
+#include "libbalsa-gpgme.h"
+#endif
 
 #if HAVE__NL_MEASUREMENT_MEASUREMENT
 #include <langinfo.h>
@@ -47,9 +50,13 @@ typedef struct {
     GtkWidget *margin_bottom;
     GtkWidget *margin_left;
     GtkWidget *margin_right;
+#ifdef HAVE_HTML_WIDGET
+    GtkWidget *html_print;
+    GtkWidget *html_load_imgs;
+    BalsaPrintSetup *setup;
+#endif
 } BalsaPrintPrefs;
 
-
 typedef struct {
     /* related message */
     LibBalsaMessage *message;
@@ -111,6 +118,90 @@ print_header_footer(GtkPrintContext * context, cairo_t * cairo_ctx,
 }
 
 
+/*
+ * Scan the parts of a multipart/alternative as to find the proper one for printing.  According to RFC 2046, Sect. 5.1.4, the first
+ * /should/ be the "plain" one, and the following the "fancier" ones.  This is not guaranteed, though.
+ * 
+ * Furthermore, we may have cases like
+ * +- multipart/alternative
+ *      +- text/plain
+ *      +- multipart/mixed
+ *           +- text/html
+ *           +- ...
+ */
+static LibBalsaMessageBody *
+find_alt_part(LibBalsaMessageBody *parts,
+			  gboolean			   print_alt_html)
+{
+	LibBalsaMessageBody *use_part;
+
+	/* scan the parts */
+	for (use_part = parts; use_part != NULL; use_part = use_part->next) {
+		gchar *mime_type;
+		
+		mime_type = libbalsa_message_body_get_mime_type(use_part);
+		if ((g_ascii_strncasecmp(mime_type, "multipart/", 10U) == 0) && (use_part->parts != NULL)) {
+			/* consider the first child of a multipart */
+			g_free(mime_type);
+			mime_type = libbalsa_message_body_get_mime_type(use_part->parts);
+		}
+
+		if (((g_ascii_strcasecmp(mime_type, "text/plain") == 0) && !print_alt_html) ||
+			((g_ascii_strcasecmp(mime_type, "text/html") == 0) && print_alt_html)) {
+			g_free(mime_type);
+			return use_part;
+		}
+		g_free(mime_type);
+	}
+	
+	/* nothing found, fall back to the first part in the chain */
+	return parts;
+}
+
+
+#ifdef HAVE_GPGME
+static inline GList *
+begin_crypto_frame(GList 		   *bpo_list,
+				   BalsaPrintSetup *psetup,
+				   gboolean 		is_signed,
+				   gboolean 		is_encrypted)
+{
+	if (is_signed) {
+		if (is_encrypted) {
+			bpo_list = balsa_print_object_frame_begin(bpo_list, _("Signed and encrypted"), psetup);
+		} else {
+			bpo_list = balsa_print_object_frame_begin(bpo_list, _("Signed"), psetup);
+		}
+	} else if (is_encrypted) {
+		bpo_list = balsa_print_object_frame_begin(bpo_list, _("Encrypted"), psetup);
+	} else {
+		/* no crypto, nothing to do... */
+	}
+	return bpo_list;
+}
+#endif				/* HAVE_GPGME */
+
+
+static GList *
+print_single_part(GList 			  *bpo_list,
+				  GtkPrintContext     *context,
+				  BalsaPrintSetup     *psetup,
+				  LibBalsaMessageBody *body,
+				  gboolean 			   no_first_sep,
+				  gboolean			   add_signature)
+{
+	if (!no_first_sep) {
+		bpo_list = balsa_print_object_separator(bpo_list, psetup);
+	}
+	if (add_signature) {
+#ifdef HAVE_GPGME
+		bpo_list = begin_crypto_frame(bpo_list, psetup, TRUE, body->was_encrypted);
+#endif				/* HAVE_GPGME */
+	}
+	return balsa_print_objects_append_from_body(bpo_list, context, body, psetup);
+}
+
+
 /*
  * scan the body list and prepare print data according to the content type
  */
@@ -118,11 +209,11 @@ static GList *
 scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 	  LibBalsaMessageBody * body, gboolean no_first_sep)
 {
+    gboolean add_signature = FALSE;
 #ifdef HAVE_GPGME
-    gboolean add_signature;
     gboolean have_crypto_frame;
+	gboolean is_mp_signed;
 #endif				/* HAVE_GPGME */
-
     while (body) {
 	gchar *conttype;
 
@@ -132,55 +223,41 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 	    g_ascii_strcasecmp(conttype, "application/pgp-signature") &&
 	    g_ascii_strcasecmp(conttype, "application/pkcs7-signature") &&
 	    g_ascii_strcasecmp(conttype, "application/x-pkcs7-signature");
-	if (!g_ascii_strcasecmp("multipart/signed", conttype) &&
-	    body->parts && body->parts->next
-	    && body->parts->next->sig_info) {
+
+	is_mp_signed = libbalsa_message_body_multipart_signed(body);
+	if (is_mp_signed || (!add_signature && body->was_encrypted)) {
 	    have_crypto_frame = TRUE;
 	    bpo_list = balsa_print_object_separator(bpo_list, psetup);
 	    no_first_sep = TRUE;
-	    if (body->was_encrypted)
-		bpo_list = balsa_print_object_frame_begin(bpo_list,
-							  _("Signed and encrypted matter"),
-							  psetup);
-	    else
-		bpo_list = balsa_print_object_frame_begin(bpo_list,
-							  _("Signed matter"),
-							  psetup);
-	} else if (!add_signature && body->was_encrypted) {
-	    have_crypto_frame = TRUE;
-	    bpo_list = balsa_print_object_separator(bpo_list, psetup);
-	    no_first_sep = TRUE;
-	    bpo_list = balsa_print_object_frame_begin(bpo_list,
-						      _("Encrypted matter"),
-						      psetup);
-	} else
+	    bpo_list = begin_crypto_frame(bpo_list, psetup, is_mp_signed, body->was_encrypted);
+	} else {
 	    have_crypto_frame = FALSE;
+	}
 #endif				/* HAVE_GPGME */
 
 	if (g_ascii_strncasecmp(conttype, "multipart/", 10)) {
-	    if (no_first_sep)
+		bpo_list = print_single_part(bpo_list, context, psetup, body, no_first_sep, add_signature);
 		no_first_sep = FALSE;
-	    else
-		bpo_list = balsa_print_object_separator(bpo_list, psetup);
 #ifdef HAVE_GPGME
-	    if (add_signature) {
-		if (body->was_encrypted)
-		    bpo_list = balsa_print_object_frame_begin(bpo_list,
-							      _("Signed and encrypted matter"),
-							      psetup);
-		else
-		    bpo_list = balsa_print_object_frame_begin(bpo_list,
-							      _("Signed matter"),
-							      psetup);
-	    }
+	} else if (add_signature && !is_mp_signed) {
+		if (!no_first_sep) {
+			bpo_list = balsa_print_object_separator(bpo_list, psetup);
+			no_first_sep = TRUE;
+		}
+		bpo_list = begin_crypto_frame(bpo_list, psetup, TRUE, body->was_encrypted);
 #endif				/* HAVE_GPGME */
-	    bpo_list = balsa_print_objects_append_from_body(bpo_list, context,
-							    body, psetup);
 	}
 
 	if (body->parts) {
-	    bpo_list = scan_body(bpo_list, context, psetup, body->parts, no_first_sep);
-	    no_first_sep = FALSE;
+		if (g_ascii_strcasecmp(conttype, "multipart/alternative") == 0) {
+			LibBalsaMessageBody *print_part;
+
+			print_part = find_alt_part(body->parts, psetup->print_alt_html);
+			bpo_list = print_single_part(bpo_list, context, psetup, print_part, no_first_sep, add_signature);
+		} else {
+			bpo_list = scan_body(bpo_list, context, psetup, body->parts, no_first_sep);
+		}
+		no_first_sep = FALSE;
 	}
 
 	/* end the frame for an embedded message or encrypted stuff */
@@ -193,13 +270,8 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 
 #ifdef HAVE_GPGME
 	if (add_signature) {
-	    gchar *header =
-		g_strdup_printf(_("This is an inline %s signed %s message part:"),
-				body->sig_info->protocol == GPGME_PROTOCOL_OpenPGP ?
-				_("OpenPGP") : _("S/MIME"), conttype);
 	    bpo_list = balsa_print_object_separator(bpo_list, psetup);
-	    bpo_list = balsa_print_object_header_crypto(bpo_list, context, body, header, psetup);
-	    g_free(header);
+	    bpo_list = balsa_print_object_header_crypto(bpo_list, context, body, psetup);
 	    bpo_list = balsa_print_object_frame_end(bpo_list, psetup);
 	}
 #endif				/* HAVE_GPGME */
@@ -211,7 +283,6 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
     return bpo_list;
 }
 
-
 static void
 begin_print(GtkPrintOperation * operation, GtkPrintContext * context,
 	    BalsaPrintData * pdata)
@@ -277,12 +348,15 @@ begin_print(GtkPrintOperation * operation, GtkPrintContext * context,
 	footer_string = NULL;
 
     date = libbalsa_message_date_to_utf8(pdata->message, balsa_app.date_string);
-    if (footer_string) {
-	footer_string = g_string_append(footer_string, " \342\200\224 ");
-	footer_string = g_string_append(footer_string, date);
-    } else
-	footer_string = g_string_new(date);
-    g_free(date);
+    if (date != NULL) {
+    	if (footer_string != NULL) {
+    		footer_string = g_string_append(footer_string, " \342\200\224 ");
+    		footer_string = g_string_append(footer_string, date);
+    	} else {
+    		footer_string = g_string_new(date);
+    	}
+    	g_free(date);
+    }
 
     if (pdata->message->headers->from) {
 	gchar *from =
@@ -417,12 +491,12 @@ add_font_button(const gchar * text, const gchar * font, GtkGrid * grid,
     gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_widget_set_halign(label, GTK_ALIGN_START);
-    gtk_grid_attach(grid, label, 0, row, 1, 1);
+    gtk_grid_attach(grid, label, 1, row, 1, 1);
 
     font_button = gtk_font_button_new_with_font(font);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), font_button);
     gtk_widget_set_hexpand(font_button, TRUE);
-    gtk_grid_attach(grid, font_button, 1, row, 1, 1);
+    gtk_grid_attach(grid, font_button, 2, row, 1, 1);
 
     return font_button;
 }
@@ -440,7 +514,7 @@ add_margin_spinbtn(const gchar * text, gdouble min, gdouble max, gdouble dflt,
     gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_widget_set_halign(label, GTK_ALIGN_START);
-    gtk_grid_attach(grid, label, 0, row, 1, 1);
+    gtk_grid_attach(grid, label, 1, row, 1, 1);
 
     if (get_default_user_units() == GTK_UNIT_INCH) {
 	unit = _("inch");
@@ -459,13 +533,13 @@ add_margin_spinbtn(const gchar * text, gdouble min, gdouble max, gdouble dflt,
     }
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinbtn), TRUE);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), spinbtn);
-    gtk_grid_attach(grid, spinbtn, 1, row, 1, 1);
+    gtk_grid_attach(grid, spinbtn, 2, row, 1, 1);
 
     label = gtk_label_new(unit);
     gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_widget_set_halign(label, GTK_ALIGN_START);
-    gtk_grid_attach(grid, label, 2, row, 1, 1);
+    gtk_grid_attach(grid, label, 3, row, 1, 1);
 
     return spinbtn;
 }
@@ -481,116 +555,90 @@ check_margins(GtkAdjustment * adjustment, GtkAdjustment * other)
                                  gtk_adjustment_get_value(other));
 }
 
+static GtkWidget *
+create_options_group(const gchar *label_str, GtkWidget *parent_grid, gint parent_col, gint parent_row, gint parent_width)
+{
+	GtkWidget *group;
+	GtkWidget *label;
+    GtkWidget *grid;
+    gchar *markup;
+
+    group = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
+    gtk_grid_attach(GTK_GRID(parent_grid), group, parent_col, parent_row, parent_width, 1);
+
+    label = gtk_label_new(NULL);
+    markup = g_strdup_printf("<b>%s</b>", label_str);
+    gtk_label_set_markup(GTK_LABEL(label), markup);
+    g_free(markup);
+    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+    gtk_widget_set_halign(label, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(group), label, FALSE, FALSE, 0);
+
+	grid = gtk_grid_new();
+    gtk_box_pack_start(GTK_BOX(group), grid, FALSE, FALSE, 0);
+	gtk_grid_set_column_spacing(GTK_GRID(grid), 0);
+	gtk_grid_set_row_spacing(GTK_GRID(grid), 6);
+	gtk_grid_attach(GTK_GRID(grid), gtk_label_new("    "), 0, 0, 1, 1);
+	return grid;
+}
+
 static GtkWidget *
 message_prefs_widget(GtkPrintOperation * operation,
 		     BalsaPrintPrefs * print_prefs)
 {
     GtkWidget *page;
-    GtkWidget *group;
-    GtkWidget *label;
-    GtkWidget *hbox;
-    GtkWidget *vbox;
     GtkWidget *grid;
+#ifdef HAVE_HTML_WIDGET
+    GtkWidget *dummy;
+#endif
     GtkPageSetup *pg_setup;
-    gchar *markup;
 
     gtk_print_operation_set_custom_tab_label(operation, _("Message"));
 
-    page = gtk_box_new(GTK_ORIENTATION_VERTICAL, 18);
+    page = gtk_grid_new();
+    gtk_grid_set_column_spacing(GTK_GRID(page), 18);
+    gtk_grid_set_row_spacing(GTK_GRID(page), 18);
     gtk_container_set_border_width(GTK_CONTAINER(page), 12);
 
-    group = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
-    gtk_box_pack_start(GTK_BOX(page), group, FALSE, TRUE, 0);
+    /* fonts */
+    grid = create_options_group(_("Fonts"), page, 0, 0, 3);
 
-    label = gtk_label_new(NULL);
-    markup = g_strdup_printf("<b>%s</b>", _("Fonts"));
-    gtk_label_set_markup(GTK_LABEL(label), markup);
-    g_free(markup);
-    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-    gtk_widget_set_halign(label, GTK_ALIGN_START);
-    gtk_box_pack_start(GTK_BOX(group), label, FALSE, FALSE, 0);
+    print_prefs->header_font = add_font_button(_("_Header Font:"), balsa_app.print_header_font, GTK_GRID(grid), 0);
+    print_prefs->body_font = add_font_button(_("B_ody Font:"), balsa_app.print_body_font, GTK_GRID(grid), 1);
+    print_prefs->footer_font = add_font_button(_("_Footer Font:"), balsa_app.print_footer_font, GTK_GRID(grid), 2);
 
-    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_box_pack_start(GTK_BOX(group), hbox, TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("    "),
-		       FALSE, FALSE, 0);
-    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
-
-    grid = gtk_grid_new();
-    gtk_grid_set_row_spacing(GTK_GRID(grid), 6);
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 6);
-
-    gtk_box_pack_start(GTK_BOX(vbox), grid, FALSE, TRUE, 0);
-
-    print_prefs->header_font =
-	add_font_button(_("_Header Font:"), balsa_app.print_header_font,
-			GTK_GRID(grid), 0);
-    print_prefs->body_font =
-	add_font_button(_("B_ody Font:"), balsa_app.print_body_font,
-			GTK_GRID(grid), 1);
-    print_prefs->footer_font =
-	add_font_button(_("_Footer Font:"), balsa_app.print_footer_font,
-			GTK_GRID(grid), 2);
-
-    group = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
-    gtk_box_pack_start(GTK_BOX(page), group, FALSE, TRUE, 0);
-
-    label = gtk_label_new(NULL);
-    markup = g_strdup_printf("<b>%s</b>", _("Highlighting"));
-    gtk_label_set_markup(GTK_LABEL(label), markup);
-    g_free(markup);
-    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-    gtk_widget_set_halign(label, GTK_ALIGN_START);
-    gtk_box_pack_start(GTK_BOX(group), label, FALSE, FALSE, 0);
-
-    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_box_pack_start(GTK_BOX(group), hbox, TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("    "),
-		       FALSE, FALSE, 0);
-    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+    /* syntax highlighting */
+    grid = create_options_group(_("Highlighting"), page, 0, 1, 1);
 
     print_prefs->highlight_cited =
 	gtk_check_button_new_with_mnemonic(_("Highlight _cited text"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
-				 (print_prefs->highlight_cited),
-				 balsa_app.print_highlight_cited);
-    gtk_box_pack_start(GTK_BOX(vbox), print_prefs->highlight_cited, FALSE,
-		       TRUE, 0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print_prefs->highlight_cited), balsa_app.print_highlight_cited);
+    gtk_grid_attach(GTK_GRID(grid), print_prefs->highlight_cited, 1, 0, 1, 1);
 
-    print_prefs->highlight_phrases =
-	gtk_check_button_new_with_mnemonic(_
-					   ("Highlight _structured phrases"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
-				 (print_prefs->highlight_phrases),
-				 balsa_app.print_highlight_phrases);
-    gtk_box_pack_start(GTK_BOX(vbox), print_prefs->highlight_phrases,
-		       FALSE, TRUE, 0);
+    print_prefs->highlight_phrases = gtk_check_button_new_with_mnemonic(_("Highlight _structured phrases"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print_prefs->highlight_phrases), balsa_app.print_highlight_phrases);
+    gtk_grid_attach(GTK_GRID(grid), print_prefs->highlight_phrases, 1, 1, 1, 1);
 
-    group = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
-    gtk_box_pack_start(GTK_BOX(page), group, FALSE, TRUE, 0);
+#ifdef HAVE_HTML_WIDGET
+    /* treatment of HTML messages and parts */
+    grid = create_options_group(_("Highlighting"), page, 1, 1, 1);
 
-    label = gtk_label_new(NULL);
-    markup = g_strdup_printf("<b>%s</b>", _("Margins"));
-    gtk_label_set_markup(GTK_LABEL(label), markup);
-    g_free(markup);
-    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-    gtk_widget_set_halign(label, GTK_ALIGN_START);
-    gtk_box_pack_start(GTK_BOX(group), label, FALSE, FALSE, 0);
+    print_prefs->html_print = gtk_check_button_new_with_mnemonic(_("Prefer text/plain over HTML"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print_prefs->html_print), balsa_app.display_alt_plain);
+    gtk_grid_attach(GTK_GRID(grid), print_prefs->html_print, 1, 0, 1, 1);
 
-    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_box_pack_start(GTK_BOX(group), hbox, TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("    "),
-		       FALSE, FALSE, 0);
-    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+    print_prefs->html_load_imgs = gtk_check_button_new_with_mnemonic(_("Download images from remote servers (may be dangerous)"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print_prefs->html_load_imgs), FALSE);
+    gtk_grid_attach(GTK_GRID(grid), print_prefs->html_load_imgs, 1, 1, 1, 1);
 
-    grid = gtk_grid_new();
-    gtk_grid_set_row_spacing(GTK_GRID(grid), 6);
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 6);
+    /* phantom alignment */
+    dummy = gtk_label_new(" ");
+    gtk_widget_set_hexpand(dummy, TRUE);
+    gtk_grid_attach(GTK_GRID(grid), dummy, 2, 1, 1, 1);
+#endif
 
-    gtk_box_pack_start(GTK_BOX(vbox), grid, FALSE, TRUE, 0);
+    /* margins */
+    grid = create_options_group(_("Margins"), page, 0, 2, 2);
 
     pg_setup = gtk_print_operation_get_default_page_setup(operation);
     print_prefs->margin_top =
@@ -642,16 +690,13 @@ message_prefs_apply(GtkPrintOperation * operation, GtkWidget * widget,
 {
     g_free(balsa_app.print_header_font);
     balsa_app.print_header_font =
-	g_strdup(gtk_font_button_get_font_name
-		 (GTK_FONT_BUTTON(print_prefs->header_font)));
+	gtk_font_chooser_get_font(GTK_FONT_CHOOSER(print_prefs->header_font));
     g_free(balsa_app.print_body_font);
     balsa_app.print_body_font =
-	g_strdup(gtk_font_button_get_font_name
-		 (GTK_FONT_BUTTON(print_prefs->body_font)));
+	gtk_font_chooser_get_font(GTK_FONT_CHOOSER(print_prefs->body_font));
     g_free(balsa_app.print_footer_font);
     balsa_app.print_footer_font =
-	g_strdup(gtk_font_button_get_font_name
-		 (GTK_FONT_BUTTON(print_prefs->footer_font)));
+	gtk_font_chooser_get_font(GTK_FONT_CHOOSER(print_prefs->footer_font));
     balsa_app.print_highlight_cited =
 	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                      (print_prefs->highlight_cited));
@@ -674,6 +719,12 @@ message_prefs_apply(GtkPrintOperation * operation, GtkWidget * widget,
 	balsa_app.margin_left /= 25.4;
 	balsa_app.margin_right /= 25.4;
     }
+#ifdef HAVE_HTML_WIDGET
+    print_prefs->setup->print_alt_html =
+    	!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(print_prefs->html_print));
+    print_prefs->setup->html_load_images =
+    	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(print_prefs->html_load_imgs));
+#endif
 }
 
 
@@ -724,6 +775,9 @@ message_print(LibBalsaMessage * msg, GtkWindow * parent)
     /* create a print context */
     print_data = g_new0(BalsaPrintData, 1);
     print_data->message = msg;
+#ifdef HAVE_HTML_WIDGET
+    print_prefs.setup = &print_data->setup;
+#endif
 
     g_signal_connect(print, "begin_print", G_CALLBACK(begin_print), print_data);
     g_signal_connect(print, "draw_page", G_CALLBACK(draw_page), print_data);
