@@ -37,6 +37,8 @@
 #include "filter-funcs.h"
 #include "mailbox-filter.h"
 #include "libbalsa-conf.h"
+#include "net-client-utils.h"
+#include "geometry-manager.h"
 
 #include "smtp-server.h"
 #include "send.h"
@@ -65,6 +67,8 @@ static void config_identities_load(void);
 
 static void config_filters_load(void);
 
+static inline gboolean is_special_name(const gchar *name);
+
 #define folder_section_path(mn) \
     BALSA_MAILBOX_NODE(mn)->config_prefix ? \
     g_strdup(BALSA_MAILBOX_NODE(mn)->config_prefix) : \
@@ -75,10 +79,14 @@ static void config_filters_load(void);
     g_strdup(LIBBALSA_MAILBOX(mbox)->config_prefix) : \
     config_get_unused_group(MAILBOX_SECTION_PREFIX)
 
-#define address_book_section_path(ab) \
-    LIBBALSA_ADDRESS_BOOK(ab)->config_prefix ? \
-    g_strdup(LIBBALSA_ADDRESS_BOOK(ab)->config_prefix) : \
-    config_get_unused_group(ADDRESS_BOOK_SECTION_PREFIX)
+static gchar *
+address_book_section_path(LibBalsaAddressBook *address_book)
+{
+    const gchar *config_prefix = libbalsa_address_book_get_config_prefix(address_book);
+
+    return config_prefix != NULL ? g_strdup(config_prefix) :
+        config_get_unused_group(ADDRESS_BOOK_SECTION_PREFIX);
+}
 
 gint config_load(void)
 {
@@ -98,16 +106,74 @@ config_load_section(const gchar * key, const gchar * value, gpointer data)
     return FALSE;
 }
 
+static gboolean
+migrate_imap_mailboxes(const gchar *key, const gchar *value, gpointer data)
+{
+	gchar *type;
+	GHashTable *migrated = (GHashTable *) data;
+
+    libbalsa_conf_push_group(key);
+	type = libbalsa_conf_get_string("Type");
+	if ((type != NULL) && (strcmp(type, "LibBalsaMailboxImap") == 0) && !libbalsa_conf_get_bool("Migrated")) {
+		BalsaMailboxNode *mbnode;
+
+		/* try to load the IMAP mailbox as folder */
+		libbalsa_conf_pop_group();
+		mbnode = balsa_mailbox_node_new_from_config(key);
+	    if (mbnode != NULL) {
+	    	gchar *folder_key;
+	    	gchar *oldname;
+
+	    	/* do not add the same folder multiple times */
+	    	folder_key = g_strconcat(mbnode->server->user, "@", mbnode->server->host, NULL);
+	    	oldname = g_strdup(mbnode->name);
+	    	if (g_hash_table_contains(migrated, folder_key)) {
+	    		g_object_unref(mbnode);
+	    		g_free(folder_key);
+	    	} else {
+	    		g_hash_table_add(migrated, folder_key);
+		    	g_free(mbnode->name);
+		    	mbnode->name = g_strdup(mbnode->server->host);
+		    	balsa_mblist_mailbox_node_append(NULL, mbnode);
+
+		    	g_free(mbnode->config_prefix);
+		    	mbnode->config_prefix = config_get_unused_group(FOLDER_SECTION_PREFIX);
+		    	g_signal_connect_swapped(mbnode->server, "config-changed", G_CALLBACK(config_folder_update), mbnode);
+	    	}
+
+	    	if (!is_special_name(value)) {
+				libbalsa_conf_remove_group(key);
+				libbalsa_conf_private_remove_group(key);
+				libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
+						"Migrated IMAP mailbox “%s” to IMAP folder. Please review its configuration.", oldname);
+			}
+	    	g_free(oldname);
+	    }
+	} else {
+		libbalsa_conf_pop_group();
+	}
+	g_free(type);
+
+    return FALSE;
+}
+
 void
 config_load_sections(void)
 {
+	GHashTable *migrated;
+
+	/* hack - migrate all LibBalsaMailboxImap mailboxes to IMAP folders */
+	migrated = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+    libbalsa_conf_foreach_group(MAILBOX_SECTION_PREFIX, migrate_imap_mailboxes, migrated);
+    g_hash_table_destroy(migrated);
+
+    libbalsa_conf_foreach_group(FOLDER_SECTION_PREFIX,
+                                config_load_section,
+                                config_folder_init);
     libbalsa_conf_foreach_group(MAILBOX_SECTION_PREFIX,
                                 config_load_section,
                                 config_mailbox_init);
     balsa_app.inbox_input = g_list_reverse(balsa_app.inbox_input);
-    libbalsa_conf_foreach_group(FOLDER_SECTION_PREFIX,
-                                config_load_section,
-                                config_folder_init);
 }
 
 static gint
@@ -168,6 +234,19 @@ sr_special_notify(gpointer data, GObject * mailbox)
 static gchar *specialNames[] = {
     INBOX_NAME, SENTBOX_NAME, TRASH_NAME, DRAFTS_NAME, OUTBOX_NAME
 };
+
+static inline gboolean
+is_special_name(const gchar *name)
+{
+	guint n;
+	gboolean res = FALSE;
+
+	for (n = 0U; !res && (n < G_N_ELEMENTS(specialNames)); n++) {
+		res = strcmp(name, specialNames[n]) == 0;
+	}
+	return res;
+}
+
 void
 config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
 {
@@ -252,9 +331,11 @@ config_address_book_save(LibBalsaAddressBook * ab)
 void
 config_address_book_delete(LibBalsaAddressBook * ab)
 {
-    if (ab->config_prefix) {
-	libbalsa_conf_remove_group(ab->config_prefix);
-	libbalsa_conf_private_remove_group(ab->config_prefix);
+    const gchar *config_prefix = libbalsa_address_book_get_config_prefix(ab);
+
+    if (config_prefix != NULL) {
+	libbalsa_conf_remove_group(config_prefix);
+	libbalsa_conf_private_remove_group(config_prefix);
 	libbalsa_conf_queue_sync();
     }
 }
@@ -375,17 +456,18 @@ static gint
 config_mailbox_init(const gchar * prefix)
 {
     LibBalsaMailbox *mailbox;
-    const gchar *key = prefix + strlen(MAILBOX_SECTION_PREFIX);
+    const gchar *key;
 
     g_return_val_if_fail(prefix != NULL, FALSE);
+    key = prefix + strlen(MAILBOX_SECTION_PREFIX);
 
-    mailbox = libbalsa_mailbox_new_from_config(prefix);
+    mailbox = libbalsa_mailbox_new_from_config(prefix, is_special_name(key));
     if (mailbox == NULL)
 	return FALSE;
     if (LIBBALSA_IS_MAILBOX_REMOTE(mailbox)) {
 	LibBalsaServer *server = LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox);
         libbalsa_server_connect_signals(server,
-                                        G_CALLBACK(ask_password), mailbox);
+                                        G_CALLBACK(ask_password), NULL);
 	g_signal_connect_swapped(server, "config-changed",
                                  G_CALLBACK(config_mailbox_update),
 				 mailbox);
@@ -422,6 +504,10 @@ config_mailbox_init(const gchar * prefix)
 	    *special = mailbox;
             g_object_weak_ref(G_OBJECT(mailbox),
                               (GWeakNotify) sr_special_notify, special);
+            if (LIBBALSA_IS_MAILBOX_IMAP(mailbox)) {
+            	/* IMAP folder, used as special mailbox: remember that we migrated it */
+            	libbalsa_mailbox_save_config(mailbox, prefix);
+            }
 	}
 
         balsa_mblist_mailbox_node_append(NULL, mbnode);
@@ -607,23 +693,22 @@ config_global_load(void)
 	d_get_gint("IndexSizeWidth", SIZE_DEFAULT_WIDTH);
 
     /* ... window sizes */
-    balsa_app.mw_width = libbalsa_conf_get_int("MainWindowWidth=640");
-    balsa_app.mw_height = libbalsa_conf_get_int("MainWindowHeight=480");
-    balsa_app.mw_maximized =
-        libbalsa_conf_get_bool("MainWindowMaximized=false");
+    geometry_manager_init("MainWindow", 640, 480, FALSE);
     balsa_app.mblist_width = libbalsa_conf_get_int("MailboxListWidth=130");
-    /* sendmsg window sizes */
-    balsa_app.sw_width = libbalsa_conf_get_int("SendMsgWindowWidth=640");
-    balsa_app.sw_height = libbalsa_conf_get_int("SendMsgWindowHeight=480");
-    balsa_app.sw_maximized =
-        libbalsa_conf_get_bool("SendmsgWindowMaximized=false");
-    /* message window sizes */
-    balsa_app.message_window_width =
-        libbalsa_conf_get_int("MessageWindowWidth=400");
-    balsa_app.message_window_height =
-        libbalsa_conf_get_int("MessageWindowHeight=500");
-    balsa_app.message_window_maximized =
-        libbalsa_conf_get_bool("MessageWindowMaximized=false");
+    geometry_manager_init("SendMsgWindow", 640, 480, FALSE);
+    geometry_manager_init("SelectReplyParts", 400, 200, FALSE);
+    geometry_manager_init("MessageWindow", 400, 500, FALSE);
+    geometry_manager_init("SourceView", 500, 400, FALSE);
+    geometry_manager_init("IMAPSubscriptions", 200, 160, FALSE);
+    geometry_manager_init("IMAPSelectParent", 200, 160, FALSE);
+#ifdef HAVE_GPGME
+    geometry_manager_init("KeyDialog", 400, 200, FALSE);
+    geometry_manager_init("KeyList", 300, 200, FALSE);
+#ifdef ENABLE_AUTOCRYPT
+    geometry_manager_init("AutocryptDB", 300, 200, FALSE);
+#endif  /* ENABLE_AUTOCRYPT */
+#endif  /* HAVE_GPGME */
+
     /* FIXME: PKGW: why comment this out? Breaks my Transfer context menu. */
     if (balsa_app.mblist_width < 100)
 	balsa_app.mblist_width = 170;
@@ -792,8 +877,6 @@ config_global_load(void)
     libbalsa_conf_push_group("SourcePreview");
     balsa_app.source_escape_specials = 
         libbalsa_conf_get_bool("EscapeSpecials=true");
-    balsa_app.source_width  = libbalsa_conf_get_int("Width=500");
-    balsa_app.source_height = libbalsa_conf_get_int("Height=400");
     libbalsa_conf_pop_group();
 
     /* MRU mailbox tree ... */
@@ -926,24 +1009,17 @@ config_global_load(void)
 	libbalsa_server_set_username(server,
 		libbalsa_conf_get_string("ESMTPUser"));
 
-        passphrase = libbalsa_conf_private_get_string("ESMTPPassphrase");
+        passphrase = libbalsa_conf_private_get_string("ESMTPPassphrase", TRUE);
 	if (passphrase) {
-            gchar* passphrase_rot = libbalsa_rot(passphrase);
-            g_free(passphrase); 
-            libbalsa_server_set_password(server, passphrase_rot);
-	    g_free(passphrase_rot);
+            libbalsa_server_set_password(server, passphrase, FALSE);
+            net_client_free_authstr(passphrase);
         }
 
-        /* default set to "Use TLS if possible" */
-	server->tls_mode = libbalsa_conf_get_int("ESMTPTLSMode=1");
-
 	passphrase =
-	    libbalsa_conf_private_get_string("ESMTPCertificatePassphrase");
+	    libbalsa_conf_private_get_string("ESMTPCertificatePassphrase", TRUE);
 	if (passphrase) {
-            gchar* passphrase_rot = libbalsa_rot(passphrase);
-            g_free(passphrase);
-	    g_free(server->cert_passphrase);
-	    server->cert_passphrase = passphrase_rot;
+        libbalsa_server_set_password(server, passphrase, TRUE);
+        net_client_free_authstr(passphrase);
 	}
     }
 
@@ -991,6 +1067,10 @@ config_global_load(void)
     g_free(balsa_app.compose_headers);
     balsa_app.compose_headers =
 	libbalsa_conf_get_string("ComposeHeaders=Recipients Subject");
+#ifdef HAVE_GPGME
+    balsa_app.warn_reply_decrypted =
+    	libbalsa_conf_get_bool("WarnReplyDecrypted=true");
+#endif
 
     /* Obsolete. */
     libbalsa_conf_get_bool_with_default("RequestDispositionNotification=false",
@@ -1183,24 +1263,9 @@ config_save(void)
     libbalsa_conf_set_int("IndexDateWidth", balsa_app.index_date_width);
     libbalsa_conf_set_int("IndexSizeWidth", balsa_app.index_size_width);
 
-    libbalsa_conf_set_int("MainWindowWidth", balsa_app.mw_width);
-    libbalsa_conf_set_int("MainWindowHeight", balsa_app.mw_height);
-    libbalsa_conf_set_bool("MainWindowMaximized",
-                           !!balsa_app.mw_maximized);
+    geometry_manager_save();
     libbalsa_conf_set_int("MailboxListWidth", balsa_app.mblist_width);
 
-    libbalsa_conf_set_int("SendMsgWindowWidth", balsa_app.sw_width);
-    libbalsa_conf_set_int("SendMsgWindowHeight", balsa_app.sw_height);
-    libbalsa_conf_set_bool("SendmsgWindowMaximized",
-                           !!balsa_app.sw_maximized);
-
-    libbalsa_conf_set_int("MessageWindowWidth",
-                         balsa_app.message_window_width);
-    libbalsa_conf_set_int("MessageWindowHeight",
-                         balsa_app.message_window_height);
-    libbalsa_conf_set_bool("MessageWindowMaximized",
-                           !!balsa_app.message_window_maximized);
-
     libbalsa_conf_set_int("NotebookHeight", balsa_app.notebook_height);
 
     libbalsa_conf_pop_group();
@@ -1283,8 +1348,6 @@ config_save(void)
     libbalsa_conf_push_group("SourcePreview");
     libbalsa_conf_set_bool("EscapeSpecials",
                           balsa_app.source_escape_specials);
-    libbalsa_conf_set_int("Width",  balsa_app.source_width);
-    libbalsa_conf_set_int("Height", balsa_app.source_height);
     libbalsa_conf_pop_group();
 
     /* MRU mailbox tree ... */
@@ -1412,6 +1475,9 @@ config_save(void)
     libbalsa_conf_set_string("ComposeHeaders", balsa_app.compose_headers);
     libbalsa_conf_set_bool("ExternEditorEditHeaders", balsa_app.edit_headers);
     libbalsa_conf_set_string("QuoteString", balsa_app.quote_str);
+#ifdef HAVE_GPGME
+    libbalsa_conf_set_bool("WarnReplyDecrypted", balsa_app.warn_reply_decrypted);
+#endif
 
     libbalsa_conf_pop_group();
 
@@ -1434,10 +1500,10 @@ config_save(void)
 
     libbalsa_conf_set_bool("EmptyTrash", balsa_app.empty_trash_on_exit);
 
-    if (balsa_app.default_address_book)
+    if (balsa_app.default_address_book != NULL)
         libbalsa_conf_set_string("DefaultAddressBook",
-                                 balsa_app.default_address_book->
-                                 config_prefix);
+                                 libbalsa_address_book_get_config_prefix
+                                 (balsa_app.default_address_book));
     else
 	libbalsa_conf_clean_key("DefaultAddressBook");
 
@@ -1486,8 +1552,7 @@ config_get_unused_group(const gchar * prefix)
     libbalsa_conf_foreach_group(prefix, config_get_used_group, &max);
 
     name = g_strdup_printf("%s%d", prefix, ++max);
-    if (balsa_app.debug)
-        g_print("config_mailbox_get_highest_number: name='%s'\n", name);
+    g_debug("config_mailbox_get_highest_number: name='%s'", name);
 
     return name;
 }
@@ -1734,6 +1799,9 @@ config_load_mailbox_view(const gchar * url)
     if (libbalsa_conf_has_key("Open"))
         view->open = libbalsa_conf_get_bool("Open");
 
+    if (libbalsa_conf_has_key("Position"))
+        view->position = libbalsa_conf_get_int("Position");
+
 #ifdef HAVE_GPGME
     if (libbalsa_conf_has_key("CryptoMode"))
         view->gpg_chk_mode = libbalsa_conf_get_int("CryptoMode");
@@ -1787,9 +1855,12 @@ config_save_mailbox_view(const gchar * url, LibBalsaMailboxView * view)
 	libbalsa_conf_set_int("Subscribe",   view->subscribe);
     if (view->exposed        != libbalsa_mailbox_get_exposed(NULL))
 	libbalsa_conf_set_bool("Exposed",    view->exposed);
-    if (balsa_app.remember_open_mboxes &&
-        view->open           != libbalsa_mailbox_get_open(NULL))
-	libbalsa_conf_set_bool("Open",       view->open);
+    if (balsa_app.remember_open_mboxes) {
+        if (view->open       != libbalsa_mailbox_get_open(NULL))
+            libbalsa_conf_set_bool("Open",   view->open);
+        if (view->position   != libbalsa_mailbox_get_position(NULL))
+            libbalsa_conf_set_int("Position", view->position);
+    }
 #ifdef HAVE_GPGME
     if (view->gpg_chk_mode   != libbalsa_mailbox_get_crypto_mode(NULL))
 	libbalsa_conf_set_int("CryptoMode",  view->gpg_chk_mode);
@@ -2062,3 +2133,32 @@ config_mailbox_was_exposed(const gchar * url)
 {
     return config_mailbox_had_property(url, "Exposed");
 }
+
+static gint
+config_mailbox_get_int_property(const gchar * url, const gchar * key)
+{
+    gchar *prefix;
+    gint retval = -1;
+
+    prefix = view_by_url_prefix(url);
+    if (!libbalsa_conf_has_group(prefix)) {
+        g_free(prefix);
+        return retval;
+    }
+
+    libbalsa_conf_push_group(prefix);
+
+    if (libbalsa_conf_has_key(key))
+        retval = libbalsa_conf_get_int(key);
+
+    libbalsa_conf_pop_group();
+    g_free(prefix);
+
+    return retval;
+}
+
+gint
+config_mailbox_get_position(const gchar * url)
+{
+    return config_mailbox_get_int_property(url, "Position");
+}
