@@ -89,7 +89,8 @@ balsa_store_address_from_messages(GList * messages)
     if (!message_list) {
         /* All messages are already showing. */
         if (info)
-            gtk_window_present(GTK_WINDOW(info->dialog));
+            gtk_window_present_with_time(GTK_WINDOW(info->dialog),
+                                         gtk_get_current_event_time());
         return;
     }
 
@@ -307,7 +308,7 @@ store_address_book_frame(StoreAddressInfo * info)
             info->current_address_book = address_book;
 
         gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_box),
-                                       address_book->name);
+                                       libbalsa_address_book_get_name(address_book));
         if (address_book == balsa_app.default_address_book)
             default_ab_offset = off;
     }
@@ -375,8 +376,11 @@ store_address_add_address(StoreAddressInfo * info,
 
     text = internet_address_to_string(ia, FALSE);
     address = libbalsa_address_new();
-    address->full_name =
-        g_strdup(ia->name ? ia->name : group ? group->name : NULL);
+    if (ia->name != NULL) {
+    	address->full_name = g_strdup(ia->name);
+    } else {
+    	address->full_name = g_strdup((group != NULL) ? group->name : NULL);
+    }
     if (INTERNET_ADDRESS_IS_GROUP(ia)) {
         InternetAddressList *members;
         int j;
@@ -453,7 +457,7 @@ store_address_add_list(StoreAddressInfo    * info,
 
         if (INTERNET_ADDRESS_IS_MAILBOX(ia)) {
             store_address_add_address(info, label, ia, NULL);
-        } else if (info->current_address_book->dist_list_mode) {
+        } else if (libbalsa_address_book_get_dist_list_mode(info->current_address_book)) {
             store_address_add_address(info, label, ia, ia);
         } else {
             InternetAddressList *members =
