@@ -89,7 +89,7 @@ balsa_toolbar_model_class_init(BalsaToolbarModelClass* klass)
                      G_SIGNAL_RUN_FIRST,
                      0,
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+                     NULL, G_TYPE_NONE, 0);
 
     object_class = G_OBJECT_CLASS(klass);
     object_class->finalize = balsa_toolbar_model_finalize;
@@ -177,8 +177,7 @@ button_data toolbar_buttons[]={
     {"gtk-edit",             N_("Edit"),            FALSE},
 };
 
-const int toolbar_button_count =
-    sizeof(toolbar_buttons) / sizeof(button_data);
+const int toolbar_button_count = G_N_ELEMENTS(toolbar_buttons);
 
 /* Public methods. */
 const gchar *
