@@ -128,7 +128,8 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
 
     /* There can only be one */
     if (customize_widget) {
-        gtk_window_present(GTK_WINDOW(customize_widget));
+        gtk_window_present_with_time(GTK_WINDOW(customize_widget),
+                                     gtk_get_current_event_time());
         return;
     }
 
@@ -144,7 +145,7 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
     libbalsa_macosx_menu_for_parent(customize_widget, GTK_WINDOW(active_window));
 #endif
     g_object_add_weak_pointer(G_OBJECT(customize_widget),
-                              (gpointer) & customize_widget);
+                              (gpointer *) & customize_widget);
     g_signal_connect(G_OBJECT(customize_widget), "response",
                      G_CALLBACK(tp_dialog_response_cb), NULL);
 
@@ -718,9 +719,8 @@ tp_page_refresh_preview(ToolbarPage * page)
 
         gtk_tree_model_get(model, &iter, TP_ITEM_COLUMN, &item, -1);
         if (item >= 0 && item < toolbar_button_count)
-            balsa_toolbar_model_insert_icon(page->model,
-                                            toolbar_buttons[item].pixmap_id,
-                                            -1);
+            balsa_toolbar_model_append_icon(page->model,
+                                            toolbar_buttons[item].pixmap_id);
     }
 }
 
@@ -834,24 +834,29 @@ tp_page_swap_rows(ToolbarPage * page, gboolean forward)
 static void
 tp_store_set(GtkListStore * store, GtkTreeIter * iter, gint item)
 {
-    GdkPixbuf *pixbuf;
+    GdkPixbuf *pixbuf = NULL;
     gchar *text;
 
     text = g_strdup(balsa_toolbar_button_text(item));
     replace_nl_with_space(text);
-    pixbuf =
-        (item > 0
-         ? gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                    balsa_icon_id(toolbar_buttons[item].pixmap_id),
-                                    GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL)
-         : NULL);
+
+    if (item > 0) {
+        const gchar *icon_id;
+
+        icon_id = balsa_icon_id(toolbar_buttons[item].pixmap_id);
+        if (icon_id != NULL)
+            pixbuf =
+                gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), icon_id,
+                                         GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
+    }
+
     gtk_list_store_set(store, iter,
                        TP_TEXT_COLUMN, text,
                        TP_ICON_COLUMN, pixbuf,
                        TP_ITEM_COLUMN, item,
                        -1);
     g_free(text);
-    if (pixbuf)
+    if (pixbuf != NULL)
         g_object_unref(pixbuf);
 }
 
